#include <QApplication>
#include <QBuffer>
#include <QDir>
#include <QStringList>

#include <KLocale>
#include <KStandardDirs>
#include <KSelectAction>
#include <kabc/stdaddressbook.h>
#include <kpimutils/email.h>
#include <kpimutils/linklocator.h>

#include "KMessageStyle.h"
#include "KMessageDummy.h"

#include "fallback_html.h"


class KMessageStyle::Private
{
public:
    Private() : senderPicURLMessage( 0 ) {}
    ~Private() {}
    QStringList cssTokens, htmlTokens;
    QString senderPicURL, style, baseUrl;
    const KMessage *senderPicURLMessage;
};

KMessageStyle::KMessageStyle() : d( new Private() )
{
}

KMessageStyle::~KMessageStyle()
{
    delete d;
}

const QString&
KMessageStyle::currentStyle() const
{
    return d->style;
}

QStringList
KMessageStyle::list()
{
    QStringList styles;
    styles << i18n( "Brief" ) << i18n( "Standard" ) << i18n( "Long" ) << i18n( "Fancy" );
    
    // gather all pot. style base dirs ( ~/.kde/share/apps/kmail/mailStyle )
    QStringList styleHomes;
    styleHomes << KStandardDirs::locateLocal( "data", "kdepim/messageStyle" ) <<
                  KStandardDirs::locate( "data", "kdepim/messageStyle" );

    foreach ( QString styleHome, styleHomes )
    {
        if ( styleHome.isEmpty() )
            continue;
        QDir dir( styleHome );
        if ( !( dir.exists() && dir.isReadable() ) )
            continue; // this is not an acces. dir

        // gather all style ( i.e. subdirs )
        QStringList cStyles = dir.entryList( QDir::Dirs | QDir::Readable | QDir::Executable | QDir::NoDotAndDotDot, QDir::Name | QDir::IgnoreCase );

        foreach ( QString style, cStyles )
        {
            if ( !styles.contains( style ) )
                styles << style; // TODO use i18n?!
        }
    }
    return styles;
}

QString
KMessageStyle::makeUp( const KMessage &msg ) const
{
    d->senderPicURL = senderPicURL( &msg );
    //TODO use clean xhtml?
    QString ret = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
    "<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">\n"
    "<html><head><title></title>"
    "<style type=\"text/css\"> \n"
    "<!--\n"; // TODO: commenting should not be necessary for khtml


    ret += makeUpCss( msg );

    ret += "--> </style>\n"
    "</head>\n"
    "<body>\n";

    ret += makeUpHtml( msg );

    ret += "<p>" + KMessage::html2source(msg.bodyDecoded()) + "</p>";
    ret += "\n</body>";
    return ret;
}

QString
KMessageStyle::makeUpCss( const KMessage &msg ) const
{
    bool isVar = false;
    QString ret;
    foreach ( QString token, d->cssTokens )
    {
        ret.append( isVar ? substitute( token, msg ) : token );
        isVar = !isVar;
    }

    ret += "\n";
# if 0 // vCard is kmail only?
    if ( vCardName.isEmpty() )
#endif
        ret += ".vCard { display:none; }\n";

    if ( senderPicURL(&msg).isEmpty() )
      ret += ".senderpic { display:none; }\n";

    if ( msg.headerField( "Resent-From" ).isEmpty() )
      ret += ".resentFrom { display:none; }\n";

    if ( msg.replyTo().isEmpty() )
      ret += ".reply { display:none; }\n";

    if ( msg.headerField("Organization").isEmpty() )
      ret += ".organization { display:none; }\n";

    if ( msg.cc().isEmpty())
      ret += ".cc { display:none; }\n";

    if ( msg.bcc().isEmpty())
      ret += ".bcc { display:none; }\n";

    //TODO use GlobalSettings in kmail
    if ( /* !GlobalSettings::self()->showUserAgent() || */msg.headerField("User-Agent").isEmpty() )
      ret += ".userAgent { display:none; }\n";

    if ( msg.headerField("X-Mailer").isEmpty() )
      ret += ".xmailer { display:none; }\n";

/* THIS  IS NO MORE GENERIC TODO use in KMail reimplemetation
    KConfigGroup configReader( KMKernel::config(), "Reader" );
    int n = 10;
    if ( !configReader.readEntry( "showSpamStatus", true ))
      n = -1;
    else {
      int scores = SpamHeaderAnalyzer::getSpamScores( message ).size();
      n = scores ? scores : -1;
    }
    if (n < 0) {
      ret += ".spamStatus { display:none; }\n";
      n = 0;
    }
    for ( ; n < 10; ++n )
      ret += QString(".spamStatus%1 { display:none; }\n").arg(n);
*/
    
    return ret;
}

QString
KMessageStyle::makeUpHtml( const KMessage &msg ) const
{
    bool isVar = false;
    QString ret;
    foreach ( QString token, d->htmlTokens )
    {
        ret.append( isVar ? substitute( token, msg ) : token );
        isVar = !isVar;
    }
    return ret;
}

KSelectAction *
KMessageStyle::selector( const QString &title, QWidget *parent )
{
    KSelectAction *ret = new KSelectAction( title, parent );
    ret->setItems( list() );
    return ret;
}

QString
KMessageStyle::imgToDataUrl( const QImage &image ) const
{
  QByteArray ba;
  QBuffer buffer( &ba );
  buffer.open( QIODevice::WriteOnly );
  image.save( &buffer, "PNG" );
  return QString::fromLatin1("data:image/%1;base64,%2")
  .arg( QString::fromLatin1( "PNG" ), QString::fromLatin1( ba.toBase64() ) );
}

// just to avoid launching akonadi during debug
const QString _senderpicurl = "nix senderpic";

const QString &
KMessageStyle::senderPicURL( const KMessage *msg ) const
{
    return _senderpicurl;
    if ( msg == d->senderPicURLMessage )
    return d->senderPicURL;

    d->senderPicURLMessage = msg;
    d->senderPicURL = QString();
    
    KABC::AddressBook *addressBook = KABC::StdAddressBook::self( true );
    KABC::Addressee::List addresses = addressBook->findByEmail( KPIMUtils::firstEmailAddress( msg->from() ) );

    if( addresses.count() == 1 )
    {
        // picture
        if ( addresses[0].photo().isIntern() ) {
        // get photo data and convert to data: url
        QImage photo = addresses[0].photo().data();
        if ( !photo.isNull() )
            d->senderPicURL = imgToDataUrl( photo );
        }
        else {
        d->senderPicURL = addresses[0].photo().url();
        if ( d->senderPicURL.startsWith( '/' ) )
            d->senderPicURL.prepend( "file:" );
        }
    }

    if( d->senderPicURL.isEmpty() ) {
        // no photo, look for a Face header
        QString faceheader = msg->headerField( "Face" );
        if ( !faceheader.isEmpty() ) {
        QImage faceimage;

        kDebug( 5006 ) <<"Found Face: header";

        QByteArray facestring = faceheader.toUtf8();
        // Spec says header should be less than 998 bytes
        // Face: is 5 characters
        if ( facestring.length() < 993 ) {
            QByteArray facearray = QByteArray::fromBase64( facestring );

            QImage faceimage;
            if ( faceimage.loadFromData( facearray, "png" ) ) {
            // Spec says image must be 48x48 pixels
            if ( faceimage.size() == QSize( 48,48 ) ) {
                d->senderPicURL = imgToDataUrl( faceimage );
            } else {
                kDebug( 5006 ) <<"Face: header image is" << faceimage.width() <<"by" << faceimage.height() <<"not 48x48 Pixels";
            }
            } else {
            kDebug( 5006 ) <<"Failed to load decoded png from Face: header";
            }
        } else {
            kDebug( 5006 ) <<"Face: header too long at" << facestring.length();
        }
        }
    }

//   if( url.isEmpty() )
//   {
//     // no photo, look for a X-Face header
//     QString xfhead = msg->headerField( "X-Face" );
//     if ( !xfhead.isEmpty() ) {
//       KPIM::KXFace xf;
//       url = imgToDataUrl( xf.toImage( xfhead ) );
//     }
//   }
    return d->senderPicURL;
}

bool
KMessageStyle::set( const QString &style )
{
    QStringList styles = list();
    if ( !styles.contains( style ) )
        d->style = "Fancy";
    else
        d->style = style;

    QString message_css;
    QString html_inc;

    QString lStyle = d->style.toLower();
    if ( lStyle == "fancy" )
    {
        html_inc = QString( fancyHeaderHtml );
        message_css = fancyCss;
    }
    else if ( lStyle == "brief" )
        html_inc = QString( briefHeaderHtml );
    else if ( lStyle == "standard" )
        html_inc = QString( stdHeaderHtml );
    else if ( lStyle == "long" )
        html_inc = QString( longHeaderHtml );
    else
    {
//     
//     if ( mUserCss.isEmpty() || mUserCss == "brief" || mUserCss == "standard" ||  mUserCss == "long" )
//       return QString();

//     if (mUserCss == "fancy")
//       return QString(fancyCss);

    // TODO: set doc source?
//     css.replace("${path}", path.section('/', 0, -2));

        QString path = KStandardDirs::locate("data", "kdepim/messageStyle/" + d->style + "/message.css");
        QFile file( path );
        if ( ( file.exists() && file.open( QIODevice::ReadOnly ) ) )
        {
            d->baseUrl = QDir( path ).absolutePath().section('/', 0, -2);
            message_css = QTextStream( &file ).readAll();
            file.close();
            message_css.replace(';', " ! important;");
        }

        path = KStandardDirs::locate("data", "kdepim/messageStyle/" + d->style + "/header.inc");
        file.setFileName( path );
        if ( ( file.exists() && file.open( QIODevice::ReadOnly ) ) )
        {
            html_inc = QTextStream( &file ).readAll();
            file.close();
        }
        else
            html_inc = QString( fancyHeaderHtml );
    }

    tokenize( message_css, d->cssTokens );
    tokenize( html_inc, d->htmlTokens );

    return true; // TODO
}

QString
KMessageStyle::substitute( const QString &var, const KMessage &message ) const
{

    if ( var == "${senderpic}" )
        return senderPicURL( &message );

    if ( var == "${sender}" )
        return KMessage::emailAddrAsAnchor( message.from().isEmpty() ? message.fromStrip() : message.from(), true, "", true );

    if ( var == "${receiver}" )
        return KMessage::emailAddrAsAnchor( message.to(), true, "", false );

    if ( var == "${reply}" )
        return KMessage::emailAddrAsAnchor( message.replyTo(), false );

    if ( var == "${subject}" )
    {
        // TODO: use GlobalSettings in KMAIL
//         const int flags = KPIMUtils::LinkLocator::PreserveSpaces | ( GlobalSettings::self()->showEmoticons() ? KPIMUtils::LinkLocator::ReplaceSmileys : 0 );
        const int flags = KPIMUtils::LinkLocator::PreserveSpaces | KPIMUtils::LinkLocator::ReplaceSmileys;
        return message.subject().isEmpty() ? i18n( "No Subject" ) : KPIMUtils::LinkLocator::convertToHtml( message.subject(), flags );
    }

    if ( var == "${resentFrom}" )
        return  KMessage::emailAddrAsAnchor( message.headerField( "Resent-From" ), true, "", true );

    if ( var == "${organization}" )
        return message.headerField( "Organization" );

    if ( var == "${cc}" )
        return KMessage::emailAddrAsAnchor( message.cc(), true, "", true );

    if ( var == "${bcc}" )
        return KMessage::emailAddrAsAnchor( message.bcc(), true, "", true );

    if ( var == "${date}" )
    {
        // TODO: shouldn't the local version be used anyway?
        QString dateString;
        if( true /*printing*/ ) { // err? sure? why?
            QDateTime dateTime;
            KLocale* locale = KGlobal::locale();
            dateTime.setTime_t( message.date() );
            dateString = locale->formatDateTime( dateTime );
        }
        else {
            dateString = message.dateStr();
        }
        return KPIMUtils::LinkLocator::convertToHtml( dateString, KPIMUtils::LinkLocator::PreserveSpaces );
    }

    if ( var == "${userAgent}" )
        return KPIMUtils::LinkLocator::convertToHtml( message.headerField( "User-Agent" ), KPIMUtils::LinkLocator::PreserveSpaces );

    if ( var == "${xmailer}" )
        return KPIMUtils::LinkLocator::convertToHtml( message.headerField( "X-Mailer" ), KPIMUtils::LinkLocator::PreserveSpaces );

    if ( var == "${path}" )
        return d->baseUrl;
        
//         css.replace("${path}", path.section('/', 0, -2));

// TODO make this dynamic, use .desktops?
    if ( var == "${from}" )
        return i18n( "From: " );
    if ( var == "${replyLabel}" )
        return i18n( "Reply to: " );
    if ( var == "${to}" )
        return i18nc( "To-field of the mail header.","To: " );
    if ( var == "${resentFromLabel}" )
        return i18n( "resent from" );
    if ( var == "${ccLabel}" )
        return i18n( "CC: " );
    if ( var == "${bccLabel}" )
        return i18n( "BCC: " );
    if ( var == "${dateLabel}" )
        return i18n( "Date: " );
    if ( var == "${userAgentLabel}" )
        return i18n( "User-Agent: " );
    if ( var == "${xmailerLabel}" )
        return i18n( "X-Mailer: " );

  // TODO: is it possible to manipulate the css system color palette?
  #define RESOLVE_COLOR(_ROLE_) \
  if ( var == "${" #_ROLE_ "}" ) return QApplication::palette().color( QPalette::_ROLE_ ).name();

    RESOLVE_COLOR( WindowText ); // -> "MenuText"
    RESOLVE_COLOR( Window ); //-> "ActiveBorder"
    RESOLVE_COLOR( AlternateBase );
    RESOLVE_COLOR( Base ); // -> "Window"
    RESOLVE_COLOR( BrightText );
    RESOLVE_COLOR( Text ); // -> "WindowText"
    RESOLVE_COLOR( Light );
    RESOLVE_COLOR( Midlight );
    RESOLVE_COLOR( Dark );
    RESOLVE_COLOR( Mid );
    RESOLVE_COLOR( Shadow );
    RESOLVE_COLOR( HighlightedText );
    RESOLVE_COLOR( Highlight );
    RESOLVE_COLOR( LinkVisited );
    RESOLVE_COLOR( Link );

/* KMAIL:
//     css.replace("${Background}", mBackgroundColor.name() );
//     css.replace("${Foreground}", mForegroundColor.name() );
//     css.replace("${Link}", mLinkColor.name() );
*/
// TODO do we need those?
///     css.replace("${ToolTipBase}", pal.color( QPalette::Window ).name() );
///     css.replace("${ToolTipText}", pal.color( QPalette::Window ).name() );
///     css.replace("${Button}", pal.color( QPalette::Window ).name() );
///     css.replace("${ButtonText}", pal.color( QPalette::Window ).name() );

    return var;
}

void
KMessageStyle::tokenize( const QString &string, QStringList &tokens )
{
    tokens.clear();
    int lastIndex = 0;
    for ( int i = 0; i < string.length(); ++i )
    {
        if ( string.at( i ) == '$' && string.at( i+1 ) == '{' )
        {
            int j = string.indexOf( '}', i+2 );
            if ( j < 0 ) // invalid variable...
            {
                qWarning( "Error: variable %s is not terminated", tokens.last().toLocal8Bit().data() );
                tokens.append( string.mid( lastIndex, -1 ) ); // just append everything as plaintext
                break;
            }
            tokens.append( string.mid( lastIndex, i-lastIndex ) ); // the last plaintext
            tokens.append( string.mid( i, j-i+1 ) ); // the var
            i = lastIndex = j+1; // go on searching behind the var
        }
    }
    tokens.append( string.right( string.length() - lastIndex ) ); // the last plaintext
}
