/*
 *   Copyright (c) 2009 Rudy Commenge <rudywi@gmail.com>
 *
 *   This library is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU Library General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful, but WITHOUT
 *   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *   FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 *   License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB. If not, write to the
 *   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *   02110-1301, USA.
 */ 

#include "kdeforum_resource.h"

#include "settings.h"
#include "settingsadaptor.h"
#include "kdeforumsettings.h"

#include "ForumDescAttribute.h"

#include <QtDBus/QDBusConnection>

#include <QList>
#include <QString>
#include <QDebug>
#include <QUrl>
#include <QFile>
#include <QDir>
#include <QIODevice>
#include <QTextStream>

#include <KDE/KMessageBox>

#include <kabc/addressee.h>
#include <kabc/key.h>
#include <kabc/errorhandler.h>
#include <kconfigdialog.h>

#include <kmime/kmime_message.h>
#include <boost/shared_ptr.hpp>

#include <attica/person.h>
#include <attica/provider.h>
#include <attica/content.h>
#include <attica/knowledgebaseentry.h>
#include <attica/folder.h>
#include <attica/message.h>

#include <akonadi/attributefactory.h>
#include <akonadi/kmime/messageparts.h>
#include <akonadi/cachepolicy.h>
#include <akonadi/itemcreatejob.h>
#include <akonadi/collectionfetchjob.h>

#include <Qt/qfile.h>
#include <akonadi/collectionfetchscope.h>

typedef boost::shared_ptr<KMime::Message> MessagePtr;

using namespace Akonadi;
using namespace Attica;

KDEForum_Resource::KDEForum_Resource( const QString &id )
  : ResourceBase( id )
{
  new SettingsAdaptor( Settings::self() );
  QDBusConnection::sessionBus().registerObject( QLatin1String( "/Settings" ),
                            Settings::self(), QDBusConnection::ExportAdaptors );

  Akonadi::AttributeFactory::registerAttribute<ForumDescAttribute>();

  m_pm = 0;
  init();
  
  // TODO: you can put any resource specific initialization code here.
}

void KDEForum_Resource::init()
{
  generateConfigFile();
  //Initializing the Provider Manager
  if (m_pm != 0)
    delete m_pm;
  m_pm = new ProviderManager();
  QString configPath = "/.akonadi/kde-forum/";
  configPath+=name();
  QString home = QDir::homePath();
  QUrl pathToProviderFile = QUrl(home + configPath);
  m_pm->addProviderFile(pathToProviderFile);
  m_pm->providers().last().saveCredentials(Settings::self()->username(), Settings::self()->password());
}

KDEForum_Resource::~KDEForum_Resource()
{

}

void KDEForum_Resource::retrieveCollections()
{
  // TODO: this method is called when Akonadi wants to have all the
  // collections your resource provides.
  // Be sure to set the remote ID and the content MIME types
  
  writeLog("Username: " + Settings::self()->username());
  writeLog("URL Forum: " + Settings::self()->urlforum());
  
  // Initialize the request counter
  m_cptRequest = CPT_TRY;
  
  // Clear forum list
  m_list.clear();
  
  /* Root Collection */
  Collection c;
  c.setParent(Collection::root());
  c.setRemoteId(identifier());
  c.setName(name());
  c.setContentMimeTypes( QStringList() << Collection::mimeType() );
  c.setRights(Collection::AllRights);
  m_list << c;

  
  /* Forums Root Collection */
  Collection forumsCollection;
  forumsCollection.setParent(c);
  forumsCollection.setRemoteId("forumsCollection");
  forumsCollection.setName("forumsCollection");
  QStringList lst;
  lst << Collection::mimeType();
  lst << "message/rfc822";
  forumsCollection.setContentMimeTypes( lst );
  forumsCollection.setRights(Collection::AllRights);
  m_list << forumsCollection;
  
  /* Members Collection */
  Collection contactsCollection;
  contactsCollection.setParent(c);
  contactsCollection.setRemoteId("contactsCollection");
  contactsCollection.setName("contactsCollection");
  contactsCollection.setContentMimeTypes( QStringList() << KABC::Addressee::mimeType() );
  contactsCollection.setRights(Collection::AllRights);
  m_list << contactsCollection;
  
  /* Private Messages Folders Collection */
  Collection privateMsgCollection;
  privateMsgCollection.setParent(c);
  privateMsgCollection.setRemoteId("privateMsgCollection");
  privateMsgCollection.setName("privateMsgCollection");
  privateMsgCollection.setContentMimeTypes( QStringList() << Collection::mimeType() );
  privateMsgCollection.setRights(Collection::AllRights);
  m_list << privateMsgCollection;
  
  m_nbCollec = m_list.length();
  
  /* Sub Forums Collection */
  writeLog("RETRIVING FORUMS LIST BEGIN");
  getForums(0);
}

void KDEForum_Resource::retrieveItems( const Akonadi::Collection &collection )
{
  // TODO: this method is called when Akonadi wants to know about all the
  // items in the given collection. You can but don't have to provide all the
  // data for each item, remote ID and MIME type are enough at this stage.
  // Depending on how your resource accesses the data, there are several
  // different ways to tell Akonadi when you are done.
  
  writeLog("Forum name: " + collection.name());  
  
  if (collection.remoteId() == "contactsCollection")
  {
    m_items.clear();
    writeLog("RETRIVING CONTACTS LIST BEGIN");
    // Initialize the request counter
    m_cptRequest = CPT_TRY;
    getContacts(0);
  }
  
  if (collection.remoteId().startsWith("pm_"))
  {
    m_items.clear();
    writeLog("RETRIVING PM LIST OF " + collection.name() + " PM BEGIN");
    // Initialize the request counter
    m_cptRequest = CPT_TRY;
    getPm(collection.remoteId().mid(3));
  }
  
  if (collection.remoteId() != identifier() && collection.remoteId() != "contactsCollection" && collection.remoteId() != "forumsCollection" && collection.remoteId() != "privateMsgCollection" && !collection.remoteId().startsWith("pm_"))
  {
    m_items.clear();
    writeLog("RETRIVING TOPICS LIST OF " + collection.name() + " FORUM BEGIN");
    // Initialize the request counter
    m_cptRequest = CPT_TRY;
    getTopics(0,collection.remoteId());
  }
}

bool KDEForum_Resource::retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
  Q_UNUSED( parts );

  qDebug() << "Retrieve Messages of Topic: " << item.remoteId();
  
  // TODO: this method is called when Akonadi wants more data for a given item.
  // You can only provide the parts that have been requested but you are allowed
  // to provide all in one go
  if( item.remoteId().startsWith("topic_")) {
    writeLog("RETRIVING ITEM LIST OF " + item.remoteId() + " BEGIN");
    m_item = item;
    getMessages(0);
  }

  return true;
}

void KDEForum_Resource::aboutToQuit()
{
  // TODO: any cleanup you need to do while there is still an active
  // event loop. The resource will terminate after this method returns
  delete m_pm;
  
  QString home = QDir::homePath();
  if(QFile::exists(home + "/" + ".akonadi/kde-forum/" + name()))
     QFile::remove(home + "/" + ".akonadi/kde-forum/" + name());
}

void KDEForum_Resource::configure( WId windowId )
{
  KDEForumSettings dlg( windowId );

  // UNDER KDE 4.4 //////
  if ( dlg.exec() ) {
    Settings::self()->writeConfig();
    init();
    init();
    emit configurationDialogAccepted();
  } else {
    emit configurationDialogRejected();
  }
  //////////////////////

  // UNDER OLDER THAN KDE 4.4 ///////
  //int res=dlg.exec();
  //if(res == QDialog::Accepted)
  //{
  //  Settings::self()->writeConfig();
  //  init();  
  //}
  //////////////////////

}

void KDEForum_Resource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
{
  Q_UNUSED( item );
  Q_UNUSED( collection );

  // TODO: this method is called when somebody else, e.g. a client application,
  // has created an item in a collection managed by your resource.

  // NOTE: There is an equivalent method for collections, but it isn't part
  // of this template code to keep it simple
}

void KDEForum_Resource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
  // TODO: this method is called when somebody else, e.g. a client application,
  // has changed an item managed by your resource.
  
  QList<Provider> proList = m_pm->providers();
  PostJob *job = 0;
  
  if (parts.contains("FLAGS") && item.remoteId().startsWith("pm_")) {
    writeLog("FLAG on private message deleted: before request");
    job = proList.last().voteForContent(item.remoteId(), true);
  }
  
  if (parts.contains("FLAGS") && item.remoteId().startsWith("topic_")) {
    writeLog("FLAG on topic deleted: before request");
    job = proList.last().voteForContent("t_" + item.remoteId().mid(6), true);
  }
  
  connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(checkItemRead(Attica::BaseJob*)));
  job->start();
}

void KDEForum_Resource::checkItemRead(Attica::BaseJob* baseJob) {
  PostJob* pJob = static_cast<PostJob*>(baseJob);
  writeLog("StatusString : " + pJob->metadata().statusString());
  writeLog("Status Code : " + QString::number(pJob->metadata().statusCode()));
  if (pJob->metadata().statusCode() != 100) {
    writeLog("ERROR : Update of message read status FAILED!");
    writeLog(pJob->metadata().statusString());
    writeLog(QString::number(pJob->metadata().statusCode()));
  }
  else {
    writeLog("SUCCESS : Update of message read status OK.");
  }
}

void KDEForum_Resource::itemRemoved( const Akonadi::Item &item )
{
  Q_UNUSED( item );

  // TODO: this method is called when somebody else, e.g. a client application,
  // has deleted an item managed by your resource.

  // NOTE: There is an equivalent method for collections, but it isn't part
  // of this template code to keep it simple
}

void KDEForum_Resource::getContacts(const int page)
{
  QList<Provider> proList = m_pm->providers();
  
  if (page == 0 && m_items.size() != 0)
    m_items.clear();
  
  writeLog("CONTACTS: before request " + QString::number(page));
  
  // [url form not yet supported by rudy.commenge.free.fr]
  ListJob<Person> *job = proList.last().requestPersonSearchByLocation(0.0, 0.0, 0.0, page, 1000);
  
  connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(contactsReceived(Attica::BaseJob*)));
  job->start();
  
}

void KDEForum_Resource::getForums(const int page)
{
  QList<Provider> proList = m_pm->providers();

  //Empty category list meaning "give me all yar fora"
  Category::List categories;
  categories.clear();
  
  writeLog("FORUMS: before request " + QString::number(page));
  ListJob<Content> *job = proList.last().searchContents(categories, QString("forums"), Attica::Provider::Newest, page, 1000);
  
  connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(forumsReceived(Attica::BaseJob*)));
  job->start();
}

void KDEForum_Resource::getTopics(const int page, const QString idForum)
{
  QList<Provider> proList = m_pm->providers();
  
  if (page == 0 && m_items.size() != 0)
    m_items.clear();

  //Category list with a category containing the forum id meaning "give me all about this forum"
  Category::List categories;
  categories.clear();
  Category cat;
  cat.setId(idForum);
  categories << cat;
  
  m_currentForum = idForum;
  
  writeLog("TOPICS: before request " + QString::number(page));
  ListJob<Content> *job = proList.last().searchContents(categories, QString("topics"), Attica::Provider::Newest, page, 1000);
  
  connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(topicsReceived(Attica::BaseJob*)));
  job->start();
}

void KDEForum_Resource::getMessages(const int page) {
    QList<Provider> proList = m_pm->providers();

    if (page == 0 && m_items.size() != 0)
      m_items.clear();
    
    writeLog("MESSAGES: Topic ID: " + m_item.remoteId().mid(6));

    Content c;
    c.setId(m_item.remoteId().mid(6));

    writeLog("MESSAGES: before request " + QString::number(page));
    ListJob<KnowledgeBaseEntry> *job = proList.last().searchKnowledgeBase(c, QString(""), Attica::Provider::Newest, page, 1000);

    connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(messagesReceived(Attica::BaseJob*)));
    job->start();
}

void KDEForum_Resource::getPmFolders()
{
  QList<Provider> proList = m_pm->providers();
  
  writeLog("PM FOLDERS: before request");
  ListJob<Folder>* job = proList.last().requestFolders();
  
  connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(pmFoldersReceived(Attica::BaseJob*)));
  job->start();
}

void KDEForum_Resource::getPm(const QString idFolder)
{
  QList<Provider> proList = m_pm->providers();
  
  if (m_items.size() != 0)
      m_items.clear();
  
  m_currentPmFolder = idFolder;
  
  Folder folder = Folder();
  folder.setId(idFolder);
  ListJob<Message>* job = proList.last().requestMessages(folder);
  
  connect(job, SIGNAL(finished(Attica::BaseJob*)), SLOT(pmReceived(Attica::BaseJob*)));
  job->start();
}

void KDEForum_Resource::contactsReceived(Attica::BaseJob* baseJob) {
  ListJob<Person>* listJob = static_cast<ListJob<Person>*>(baseJob);
  Person::List persons = listJob->itemList();
  
  if ((persons.size() != listJob->metadata().itemsPerPage()) && ((listJob->metadata().totalItems() - m_items.size() - persons.size()) != 0)) {
      qDebug() << "ERROR : Incorrect number of contacts retrieved !";
      writeLog("ERROR : Incorrect number of contacts retrieved !");
      m_cptRequest--;
      if (m_cptRequest == 0) {
	m_cptRequest = CPT_TRY;
	qDebug() << "Abort : " << CPT_TRY << " errors in a row !"; // TODO alert window
	itemsRetrieved(m_items);
      }
      else
	getContacts(m_items.size() / listJob->metadata().itemsPerPage());
  }
  else {
    m_cptRequest = CPT_TRY;
    foreach(const Person& person, persons) {
      
      Item item;
      item.setMimeType( KABC::Addressee::mimeType() );
      
      KABC::Addressee addressee;
      
      // name
      addressee.setNameFromString(person.id());
      // email
      addressee.insertEmail(person.extendedAttribute(QString("email")), true);
      // group
      addressee.insertCustom(QString("PHPBBForum"),QString("group"),person.extendedAttribute(QString("group")));
      //language
      addressee.insertCustom(QString("PHPBBForum"),QString("language"),person.extendedAttribute(QString("language")));
      
      item.setRemoteId(person.extendedAttribute(QString("userid")));

      item.setPayload<KABC::Addressee>(addressee);

      m_items << item;
    }
    
    qDebug() << "Number of contacts retrieved :" << m_items.size() << " /" << listJob->metadata().totalItems();
    writeLog("Number of contacts retrieved: " + QString::number(m_items.size()) + " / " + QString::number(listJob->metadata().totalItems()));
    
    if (m_items.size() < (listJob->metadata().totalItems()))
      getContacts(m_items.size() / listJob->metadata().itemsPerPage());
    else {
      writeLog("CONTACTS: AFTER ALL REQUESTS");
      itemsRetrieved(m_items);
    }
  }
}

void KDEForum_Resource::generateConfigFile() {

    QString fRep = ".akonadi";
    QString sRep = "kde-forum";
    QString home = QDir::homePath();
    QString path(home + "/" + fRep);
    if(!QDir(path).exists())
    {
        QDir(home).mkdir(fRep);
    }
    path+="/" + sRep;
    if(!QDir(path).exists())
    {
        QDir(home + "/" + fRep).mkdir(sRep);
    }
    QFile file(path + "/" + name());
    file.open(QIODevice::WriteOnly | QIODevice::Truncate);
    QString url(Settings::self()->urlforum());
    if(!url.endsWith('/'))
        url+='/';
    QString text(
	  "<providers>\n"
	  "\t<provider>\n"
	  "\t\t<id>testBE</id>\n"
	  "\t\t<location>" + url + "akonadi/</location>\n"
	  "\t\t<name>BE-Akonadi.free.fr</name>\n"
	  "\t\t<icon></icon>\n"
	  "\t\t<termsofuse></termsofuse>\n"
	  "\t\t<register></register>\n"
	  "\t\t<services>\n"
	  "\t\t\t<person ocsversion=\"1.3\" />\n"
	  "\t\t\t<content ocsversion=\"1.3\" />\n"
	  "\t\t\t<message ocsversion=\"1.3\" />\n"
	  "\t\t\t<knowledgebase ocsversion=\"1.3\" />\n"
	  "\t\t</services>\n"
	  "\t</provider>\n"
	  "</providers>");
    QTextStream flux(&file);
    flux << text;
    file.close();

}

void KDEForum_Resource::forumsReceived(Attica::BaseJob* baseJob) {
  
  ListJob<Content>* listJob = static_cast<ListJob<Content>*>(baseJob);
  Content::List contents = listJob->itemList();
  
  int nbReceived = m_list.size() - m_nbCollec;
  
  if ((contents.size() != listJob->metadata().itemsPerPage()) && ((listJob->metadata().totalItems() - nbReceived - contents.size()) != 0)) {
      qDebug() << "ERROR : Incorrect number of forums retrieved !";
      writeLog("ERROR : Incorrect number of forums retrieved !");
      m_cptRequest--;
      if (m_cptRequest == 0) {
	m_cptRequest = CPT_TRY;
	qDebug() << "Abort : " << CPT_TRY << " errors in a row !"; // TODO alert window
	/* Box for Private Messages Collection */
	writeLog("RETRIVING PRIVATE BOX LIST BEGIN");
	getPmFolders();
      }
      else
	getForums(nbReceived / listJob->metadata().itemsPerPage());
  }
  else {
    m_cptRequest = CPT_TRY;
    foreach(const Content& content, contents) {
      
      Collection c;
      
      QString rId;
      if (QString(content.attribute("parentid")) == "0")
	rId = "forumsCollection";
      else
	rId = content.attribute("parentid");
      
      c.setParentRemoteId(rId);
      c.setRights(Collection::AllRights);

      c.setRemoteId(content.id());
      c.setName(content.name());
      QStringList lst;
      lst << Collection::mimeType();
      lst << "message/rfc822";
      c.setContentMimeTypes( lst );

      c.addAttribute(new ForumDescAttribute(QString(content.attribute("desc"))));
      m_list << c;
    }
    nbReceived = m_list.size() - m_nbCollec;
    
    qDebug() << "Number forums retrieved :" << nbReceived << " /" << listJob->metadata().totalItems();
    writeLog("Number of forums retrieved: " + QString::number(nbReceived) + " / " + QString::number(listJob->metadata().totalItems()));
    
    if (nbReceived < (listJob->metadata().totalItems()))
      getForums(nbReceived / listJob->metadata().itemsPerPage());
    else {
      writeLog("FORUMS: AFTER ALL REQUESTS");
      /* Box for Private Messages Collection */
      writeLog("RETRIVING PRIVATE BOX LIST BEGIN");
      getPmFolders();
    }
  }
}

void KDEForum_Resource::topicsReceived(Attica::BaseJob* baseJob) {
  ListJob<Content>* listJob = static_cast<ListJob<Content>*>(baseJob);
  Content::List topics = listJob->itemList();
  
  if ((topics.size() != listJob->metadata().itemsPerPage()) && ((listJob->metadata().totalItems() - m_items.size() - topics.size()) != 0)) {
      qDebug() << "ERROR : Incorrect number of topics retrieved !";
      writeLog("ERROR : Incorrect number of topics retrieved !");
      m_cptRequest--;
      if (m_cptRequest == 0) {
	m_cptRequest = CPT_TRY;
	qDebug() << "Abort : " << CPT_TRY << " errors in a row !"; // TODO alert window
	itemsRetrieved(m_items);
      }
      else
	getTopics(m_items.size() / listJob->metadata().itemsPerPage(), m_currentForum);
  }
  else {
    m_cptRequest = CPT_TRY;
    foreach(const Content& msg, topics) {
      
      Item item;
      item.setMimeType( "message/rfc822" );
      
      MessagePtr m(new KMime::Message) ;

      m->subject()->fromUnicodeString(msg.name(), "utf-8");
      m->from()->fromUnicodeString(msg.author(), "utf-8");
      m->date()->fromUnicodeString(msg.created().toString(), "utf-8");

      m->assemble();

      qDebug() << "statut topic: " << msg.attribute("new") << "\n";
      
      item.clearFlag("unread");
      if (msg.attribute("new") == "1")
	item.setFlag("unread");

      item.setRemoteId("topic_"+msg.id());
      item.setPayload(MessagePtr(m));

      m_items << item;
    }
    
    qDebug() << "Number of topics retrieved :" << m_items.size() << " /" << listJob->metadata().totalItems();
    writeLog("Number of topics retrieved: " + QString::number(m_items.size()) + " / " + QString::number(listJob->metadata().totalItems()));
    
    if (m_items.size() < (listJob->metadata().totalItems()))
      getTopics(m_items.size() / listJob->metadata().itemsPerPage(),m_currentForum);
    else {
      writeLog("TOPICS: AFTER ALL REQUESTS");
      itemsRetrieved(m_items);
    }
  }
}

void KDEForum_Resource::messagesReceived(Attica::BaseJob* baseJob) {
    ListJob<KnowledgeBaseEntry>* listJob = static_cast<ListJob<KnowledgeBaseEntry>*>(baseJob);
    KnowledgeBaseEntry::List messages = listJob->itemList();

    if ((messages.size() != listJob->metadata().itemsPerPage()) && ((listJob->metadata().totalItems() - m_items.size() - messages.size()) != 0)) {
        qDebug() << "ERROR : Incorrect number of messages retrieved !";
	writeLog("ERROR : Incorrect number of messages retrieved !");
        m_cptRequest--;
        if (m_cptRequest == 0) {
          m_cptRequest = CPT_TRY;
          qDebug() << "Abort : " << CPT_TRY << " errors in a row !"; // TODO alert window
	  return;
        }
        else
          getMessages(m_items.size() / listJob->metadata().itemsPerPage());
    }
    else {
      m_cptRequest = CPT_TRY;
      foreach(const KnowledgeBaseEntry& msg, messages) {
	
	Item item(m_item);
	if (msg.id() != m_item.remoteId().mid(6)) {
	  item.setRemoteId(msg.id());
	}
	else {
	  item.setRemoteId("topic_"+msg.id());
	}
	
	item.setMimeType( "message/rfc822" );

        MessagePtr m(new KMime::Message) ;

	m->inReplyTo()->fromUnicodeString(QString(msg.contentId()), "utf-8");
        m->subject()->fromUnicodeString(msg.name(), "utf-8");
        m->from()->fromUnicodeString(msg.user(), "utf-8");
        m->date()->fromUnicodeString(msg.changed().toString(), "utf-8");
	m->setBody(msg.description().toUtf8());
	m->assemble();

        item.setPayload(MessagePtr(m));
	
	m_items << item;
      }

      qDebug() << "Number of messages retrieved :" << m_items.size() << " /" << listJob->metadata().totalItems();
      writeLog("Number of messages retrieved: " + QString::number(m_items.size()) + " / " + QString::number(listJob->metadata().totalItems()));      

      if (m_items.size() < (listJob->metadata().totalItems()))
        getMessages(m_items.size() / listJob->metadata().itemsPerPage());
      else {
	
	CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive, this );
	job->fetchScope().setContentMimeTypes( QStringList() << "message/rfc822" );
	Collection currentColl;
	
	
	if ( job->exec() ) {
	  Collection::List collections = job->collections();
	  for ( int i=0 ; i < collections.size() ; i++) {	
	    if (collections.at(i).id() == m_item.storageCollectionId())
	      currentColl = collections.at(i);
	  }
	}
	else
	  qDebug() << "Error Insert Item:" << job->errorString();
	
	for (int i=1 ; i < m_items.size() ; i++) {
	  Akonadi::ItemCreateJob *jobI = new Akonadi::ItemCreateJob( m_items.at(i), currentColl );
	  if ( jobI->exec() )
	    qDebug() << "Item created successfully";
	  else
	    qDebug() << "Error Insert Item:" << jobI->errorString();
	}
	
	
	writeLog("MESSAGES: AFTER ALL REQUESTS");
	itemRetrieved(m_items.at(0));
      }
    }
}

void KDEForum_Resource::pmFoldersReceived(Attica::BaseJob* baseJob) {
  ListJob<Folder>* listJob = static_cast<ListJob<Folder>*>(baseJob);
  Folder::List folders = listJob->itemList();
  
  if ((folders.size() != listJob->metadata().itemsPerPage()) && ((listJob->metadata().totalItems() - folders.size()) != 0)) {
      qDebug() << "ERROR : Incorrect number of private folders retrieved !";
      writeLog("ERROR : Incorrect number of private folders retrieved !");
      m_cptRequest--;
      if (m_cptRequest == 0) {
	m_cptRequest = CPT_TRY;
	qDebug() << "Abort : " << CPT_TRY << " errors in a row !"; // TODO alert window
	collectionsRetrieved(m_list);
      }
      else
	getPmFolders();
  }
  else {
    m_cptRequest = CPT_TRY;
    foreach(const Folder& folder, folders) {
      
      Collection c;
      
      c.setParentRemoteId("privateMsgCollection");
      c.setRights(Collection::AllRights);

      c.setRemoteId(QString("pm_" + folder.id()));
      c.setName(folder.name());
      c.setContentMimeTypes( QStringList() << "message/rfc822" );

      m_list << c;
    }
    
    qDebug() << "Number private folders retrieved :" << folders.size() << " /" << listJob->metadata().totalItems();
    writeLog("Number of private folders retrieved: " + QString::number(folders.size()) + " / " + QString::number(listJob->metadata().totalItems()));
    
    writeLog("PRIVATE FOLDERS: AFTER ALL REQUESTS");
    collectionsRetrieved(m_list);
  }
}

void KDEForum_Resource::pmReceived(Attica::BaseJob* baseJob) {
  ListJob<Message>* listJob = static_cast<ListJob<Message>*>(baseJob);
  Message::List messages = listJob->itemList();
  
  if (messages.size() != listJob->metadata().totalItems()) {
      qDebug() << "ERROR : Incorrect number of private messages retrieved !";
      writeLog("ERROR : Incorrect number of private messages retrieved !");
      m_cptRequest--;
      if (m_cptRequest == 0) {
	m_cptRequest = CPT_TRY;
	qDebug() << "Abort : " << CPT_TRY << " errors in a row !"; // TODO alert window
	itemsRetrieved(m_items);
      }
      else
	getPm(m_currentPmFolder);
  }
  else {
    m_cptRequest = CPT_TRY;
    foreach(const Message& message, messages) {
      
      Item item;
      item.setRemoteId("pm_" + message.id());
      
      item.setMimeType( "message/rfc822" );

      MessagePtr m(new KMime::Message) ;

      m->from()->fromUnicodeString(QString(message.from()), "utf-8");
      m->to()->fromUnicodeString(QString(message.to()), "utf-8");
      m->subject()->fromUnicodeString(message.subject(), "utf-8");
      m->date()->fromUnicodeString(message.sent().toString(), "utf-8");
      m->setBody(message.body().toUtf8());
      m->assemble();

      
      item.clearFlags();
      
      if (Attica::Message::Unread == message.status())
	item.setFlag("unread");
	
      item.setPayload(MessagePtr(m));
      
      m_items << item;
    }
    
    qDebug() << "Number of private messages retrieved :" << m_items.size() << " /" << listJob->metadata().totalItems();
    writeLog("Number of private messages retrieved: " + QString::number(m_items.size()) + " / " + QString::number(listJob->metadata().totalItems()));
    
    if (m_items.size() < (listJob->metadata().totalItems()))
      getPm(m_currentPmFolder);
    else {
      writeLog("PRIVATE MESSAGES: AFTER ALL REQUESTS");
      itemsRetrieved(m_items);
    }
  }
}

void KDEForum_Resource::writeLog(const QString &text) {
  QFile file(QDir::QDir::homePath() + "/" + name());
  file.open(QIODevice::WriteOnly | QIODevice::Append);  
  QTextStream flux(&file);  
  flux << "[" + KDateTime::currentLocalDateTime().time().toString() + "] ";
  flux << text + "\n";
  file.close();
}

AKONADI_RESOURCE_MAIN( KDEForum_Resource )

#include "kdeforum_resource.moc"
