/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "akonaditelepathyresource.h"

#include "booleanattribute.h"
#include "presencetypeattribute.h"
#include "presencemessageattribute.h"
#include "presencestatusattribute.h"
#include "subscriptionattribute.h"

#include "settings.h"
#include "settingsadaptor.h"

#include "telepathyclient.h"
#include "telepathyaccount.h"
#include "telepathycontact.h"
#include "telepathyaccountfetchjob.h"
#include "telepathycontactfetchjob.h"
#include "telepathycollectionfetchjob.h"
#include "telepathyitemfetchjob.h"

#include <QtDBus/QDBusConnection>
#include <kabc/addressee.h>
#include <akonadi/attributefactory.h>
#include <akonadi/changerecorder.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/itemcreatejob.h>
#include <akonadi/itemmodifyjob.h>
#include <akonadi/itemdeletejob.h>
#include <klocale.h>

using namespace Akonadi;

AkonadiTelepathyResource::AkonadiTelepathyResource( const QString &id )
  : ResourceBase( id )
{
  new SettingsAdaptor( Settings::self() );
  QDBusConnection::sessionBus().registerObject( QLatin1String( "/Settings" ),
                            Settings::self(), QDBusConnection::ExportAdaptors );

  changeRecorder()->itemFetchScope().setAncestorRetrieval( ItemFetchScope::Parent );

  setName( "Akonadi + Telepathy" );

  AttributeFactory::registerAttribute<SubscriptionAttribute>();
  AttributeFactory::registerAttribute<PresenceTypeAttribute>();
  AttributeFactory::registerAttribute<PresenceStatusAttribute>();
  AttributeFactory::registerAttribute<PresenceMessageAttribute>();
  AttributeFactory::registerAttribute<BooleanAttribute>();
}

AkonadiTelepathyResource::~AkonadiTelepathyResource()
{
}

void AkonadiTelepathyResource::retrieveCollections()
{
    mCollectionList.clear();

    /* create parent collection */
    Collection parent;
    parent.setParentCollection( Collection::root() );
    parent.setRemoteId( "akonadi-telepathy" );
    parent.setName( name() );

    QStringList mimeTypes;
    mimeTypes << Collection::mimeType();
    parent.setContentMimeTypes( mimeTypes );

    Collection::List list;
    list << parent;

    /* now grab the children */

    QStringList childrenMimeTypes;
    childrenMimeTypes << KABC::Addressee::mimeType();

    const QList<TelepathyAccount *> accounts = mTelepathy.accountList();

    foreach ( TelepathyAccount *account, accounts ) {

        Collection collection;
        collection.setParentCollection( parent );
        collection.setRemoteId( account->id() );
        collection.setName( account->id() );
        collection.setContentMimeTypes( childrenMimeTypes );
        collection.setRights( Collection::ReadOnly );

        list << collection;
        mCollectionList << collection;

        connect( account,
                SIGNAL( contactsAdded( QList<TelepathyContact*> ) ),
                SLOT( contactsAdded( QList<TelepathyContact*> ) ) );
        connect( account,
                SIGNAL( contactAdded( TelepathyContact * ) ),
                SLOT( contactAdded( TelepathyContact * ) ) );
        connect( account,
                SIGNAL( contactRemoved( TelepathyContact * ) ),
                SLOT( contactRemoved( TelepathyContact * ) ) );
        connect( account,
                SIGNAL( contactChanged( const TelepathyContact * ) ),
                SLOT( contactChanged( const TelepathyContact * ) ) );

        kWarning() << "Retrieved collection: " << collection.remoteId();
    }

    collectionsRetrieved( list );
}

void AkonadiTelepathyResource::contactChanged( const TelepathyContact *contact )
{
    Akonadi::Item item;

    item.setRemoteId( contact->id() ); 

    TelepathyItemFetchJob *fetchJob = new TelepathyItemFetchJob( item );
    fetchJob->setCollection( collectionForContact( contact ) );
    fetchJob->setTelepathyContact( contact );

    connect( fetchJob, SIGNAL( result( KJob * ) ), SLOT( itemFetchedForChange( KJob * ) ) );
}

void AkonadiTelepathyResource::contactRemoved( TelepathyContact *contact )
{
    Akonadi::Item item;

    item.setRemoteId( contact->id() );

    TelepathyItemFetchJob *fetchJob = new TelepathyItemFetchJob( item );
    fetchJob->setCollection( collectionForContact( contact ) );

    connect( fetchJob, SIGNAL( result( KJob * ) ), SLOT( itemFetchedForRemoval( KJob * ) ) );

    contact->deleteLater();
}

void AkonadiTelepathyResource::itemCreateJobFinished( KJob *job )
{
    if ( job->error() )
        kDebug() << "Error creating item" << job->error();
    else
        kDebug() << "Contact item created successfully";
}

void AkonadiTelepathyResource::itemModifyJobFinished( KJob *job )
{
    if ( job->error() )
        kDebug() << "Error modifying item: " << job->error();
    else
        kDebug() << "Contact item modified successfully";
}

void AkonadiTelepathyResource::itemDeleteJobFinished( KJob *job )
{
    if ( job->error() )
        kDebug() << "Error deleting item: " << job->error();
    else
        kDebug() << "Contact item deleted successfully";
}

void AkonadiTelepathyResource::itemFetchedForChange( KJob *job )
{
    if ( job->error() ) {
        kDebug() << "Error fetching item!";
        return;
    }

    TelepathyItemFetchJob *fetchJob = qobject_cast<TelepathyItemFetchJob*>( job );

    Akonadi::Item item = fetchJob->items().first();

    const TelepathyContact *contact = fetchJob->contact();

    fillInItem( contact, item );

    Akonadi::ItemModifyJob *modifyJob = new Akonadi::ItemModifyJob( item );
    modifyJob->disableRevisionCheck();
    connect( modifyJob, SIGNAL( result( KJob * ) ), SLOT( itemModifyJobFinished( KJob * ) ) );
}

void AkonadiTelepathyResource::itemFetchedForRemoval( KJob *job )
{
    if ( job->error() ) {
        kDebug() << "Error fetching item!";
        return;
    }

    TelepathyItemFetchJob *fetchJob = qobject_cast<TelepathyItemFetchJob*>( job );

    Akonadi::Item item = fetchJob->items().first();

    Akonadi::ItemDeleteJob *deleteJob = new Akonadi::ItemDeleteJob( item );
    connect( deleteJob, SIGNAL( result( KJob * ) ), SLOT( itemDeleteJobFinished( KJob * ) ) );
}

void AkonadiTelepathyResource::retrieveItems( const Akonadi::Collection &collection )
{
    TelepathyAccount *account = accountForCollection( collection );

    if ( !account  || !account->isOnline() ) {
        kWarning() << "Account is null or offline";
        cancelTask( i18n( "Account %1 null or offline" ).arg( account->id() ) );
        return;
    }

    TelepathyAccountFetchJob *job = new TelepathyAccountFetchJob( account, collection, this );

    connect(job,
            SIGNAL(finished(const TelepathyAccount *, const Akonadi::Collection &)),
            SLOT(accountGotReady(const TelepathyAccount *, const Akonadi::Collection &)));

    job->fetch();
}

void AkonadiTelepathyResource::accountGotReady( const TelepathyAccount *account, 
       const Akonadi::Collection &collection  )
{
    const QList<TelepathyContact *> contactList = account->contactList();

    Item::List items;

    foreach ( TelepathyContact *contact, contactList ) {
        Item item;
        item.setRemoteId( contact->id() );
        item.setMimeType( KABC::Addressee::mimeType() );
        item.setParentCollection( collection );

        items << item;
    }

    itemsRetrieved( items );
}

bool AkonadiTelepathyResource::retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
    Q_UNUSED( parts );


    const TelepathyContact *contact = contactForItem( item );

    if ( !contact ) {
        kWarning() << "Contact is NULL";
        cancelTask( i18n( "Contact is null" ) );
        return false;
    }

    TelepathyContactFetchJob *job = new TelepathyContactFetchJob( contact, item, this );

    connect( job,
            SIGNAL(finished(const TelepathyContact *, const Akonadi::Item &)),
            SLOT(contactGotReady(const TelepathyContact *, const Akonadi::Item &) ) );

      job->fetch();

    return true;
}

void AkonadiTelepathyResource::contactGotReady( const TelepathyContact *contact, const Akonadi::Item &item )
{
    Item newItem( item );

    fillInItem( contact, newItem );

    itemRetrieved( newItem );
}

void AkonadiTelepathyResource::contactsAdded( QList<TelepathyContact*> contacts )
{
    foreach ( TelepathyContact *contact, contacts ) {
        contactAdded( contact );
    }
}

void AkonadiTelepathyResource::contactAdded( TelepathyContact *contact )
{
    Akonadi::Collection collection;
    collection.setRemoteId( contact->accountId() );

    TelepathyCollectionFetchJob *job = 
        new TelepathyCollectionFetchJob( collection, CollectionFetchJob::Base );
    job->setTelepathyContact( contact );

    connect( job, SIGNAL( result( KJob * ) ), SLOT( collectionFetchedForAdd( KJob * ) ) );
}

void AkonadiTelepathyResource::collectionFetchedForAdd( KJob *job )
{
    if ( job->error() ) {
        kDebug() << "Error fetching collection!";
        return;
    }

    TelepathyCollectionFetchJob *fetchJob = qobject_cast<TelepathyCollectionFetchJob*>( job );

    const TelepathyContact *contact = fetchJob->contact();

    if ( fetchJob->collections().size() <= 0 ) {
        kDebug() << "Couldn't fetch collection";
        return;
    }

    Akonadi::Collection collection = fetchJob->collections().first();

    kDebug() << "fetched collection" << collection.remoteId();

    Akonadi::Item item;

    fillInItem( contact, item );

    Akonadi::ItemCreateJob *createJob = new Akonadi::ItemCreateJob( item, collection );
    connect( createJob, SIGNAL( result( KJob* ) ), SLOT( itemCreateJobFinished( KJob* ) ) );
}

void AkonadiTelepathyResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
{
  Q_UNUSED( item );
  Q_UNUSED( collection );
}

void AkonadiTelepathyResource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
  Q_UNUSED( item );
  Q_UNUSED( parts );
}

void AkonadiTelepathyResource::itemRemoved( const Akonadi::Item &item )
{
  Q_UNUSED( item );
}

void AkonadiTelepathyResource::doSetOnline(bool online)
{
    if (online)
        synchronizeCollectionTree();
}

void AkonadiTelepathyResource::fillInItem( const TelepathyContact *contact, Akonadi::Item &item )
{
    item.setRemoteId( contact->id() );
    item.setMimeType( KABC::Addressee::mimeType() );

    kDebug() << "Retreiving contact data for " << item.remoteId();

    KABC::Addressee addressee;
    KABC::PhoneNumber number;
    KABC::Address address;

    /* name */
    addressee.setNameFromString( contact->fullName() );

    /* nickname */
    addressee.setNickName( contact->alias() );

    /* email */
    addressee.insertEmail( contact->email(), true );

    /* phone number */
    number.setNumber( contact->tel() );
    addressee.insertPhoneNumber( number );

    /* company */
    addressee.setOrganization( contact->org() );

    /* birthday */
    addressee.setBirthday( QDateTime::fromString( contact->birthday(), Qt::ISODate ) );

    /* product id */
    addressee.setProductId( contact->productId() );

    /* title */
    addressee.setTitle( contact->title() );

    /* address */

    address.setStreet( contact->street() );
    address.setLocality( contact->city() );
    address.setRegion( contact->state() );
    address.setPostalCode( contact->zip() );
    address.setCountry( contact->country() );

    addressee.insertAddress( address );

    /* picture */
    KABC::Picture picture( contact->avatar() );
    
    addressee.setPhoto( picture );

    item.setPayload<KABC::Addressee>(addressee);

    /* attributes */

    SubscriptionAttribute *sattr = item.attribute<SubscriptionAttribute>( Item::AddIfMissing );

    if ( contact->subscription() == TelepathyContact::Subscribed )
        sattr->setSubscription( SubscriptionAttribute::Subscribed );
    else if ( contact->subscription() == TelepathyContact::Unsubscribed )
        sattr->setSubscription( SubscriptionAttribute::Unsubscribed );
    else if ( contact->subscription() == TelepathyContact::Ask )
        sattr->setSubscription( SubscriptionAttribute::Ask );

    PresenceTypeAttribute *tattr = item.attribute<PresenceTypeAttribute>( Item::AddIfMissing );
    tattr->setTypeVal( contact->presenceType() );

    PresenceMessageAttribute *mattr = item.attribute<PresenceMessageAttribute>( Item::AddIfMissing );
    mattr->setMessage( contact->presenceMessage() );

    PresenceStatusAttribute *stattr = item.attribute<PresenceStatusAttribute>( Item::AddIfMissing );
    stattr->setStatus( contact->presenceStatus() );

    BooleanAttribute *supportsTextChat = new BooleanAttribute( "supports-text-chat", contact->supportsTextChats() );
    item.addAttribute( supportsTextChat );

    BooleanAttribute *supportsMediaCalls = new BooleanAttribute( "supports-media-calls", contact->supportsMediaCalls() );
    item.addAttribute( supportsMediaCalls );

    BooleanAttribute *supportsAudioCalls = new BooleanAttribute( "supports-audio-calls", contact->supportsAudioCalls() );
    item.addAttribute( supportsAudioCalls );

    BooleanAttribute *supportsVideoCalls = new BooleanAttribute( "supports-video-calls", contact->supportsVideoCalls() );
    item.addAttribute( supportsVideoCalls );

    BooleanAttribute *supportsUpgradingCalls = new BooleanAttribute( "supports-upgrading-calls", contact->supportsUpgradingCalls() );
    item.addAttribute( supportsUpgradingCalls );
}

TelepathyAccount * AkonadiTelepathyResource::accountForCollection( const Akonadi::Collection &collection )
{
    const QList<TelepathyAccount *> accountList = mTelepathy.accountList();

    foreach ( TelepathyAccount *account, accountList ) {
        if ( account->id() == collection.remoteId() ) 
            return account;
    }

    return 0;
}

TelepathyContact * AkonadiTelepathyResource::contactForItem( const Akonadi::Item &item )
{
    const TelepathyAccount *account = accountForCollection( item.parentCollection() );

    if ( !account ) {
        kDebug() << "Account is null!";
        kDebug() << "Item parent name: " << item.parentCollection().name();
        return 0;
    }
    
    const QList<TelepathyContact *> contactList = account->contactList();
    
    foreach (TelepathyContact *contact, contactList) {
        if ( contact->id() == item.remoteId() )
            return contact;
    }

    return 0;
}

Akonadi::Collection AkonadiTelepathyResource::collectionForContact( const TelepathyContact *contact )
{
    foreach ( Akonadi::Collection collection, mCollectionList ) {
        if ( collection.remoteId() == contact->accountId() )
            return collection;
    }

    return Akonadi::Collection();
}

AKONADI_RESOURCE_MAIN( AkonadiTelepathyResource )

#include "akonaditelepathyresource.moc"
