/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <QTimer>
#include <TelepathyQt4/PendingReady>
#include <TelepathyQt4/Connection>
#include <TelepathyQt4/ContactManager>
#include <TelepathyQt4/PendingContacts>

#include "telepathyaccount.h"
#include "telepathycontact.h"

TelepathyAccount::TelepathyAccount(const Tp::AccountPtr &acc, QObject *parent) :
    QObject(parent),
    mAccount(acc),
    mReady(false)
{
    mDisplayName = acc->displayName();

    connect(this, SIGNAL(accountReady()), SLOT(handleAccountReady()));
    connect(this, SIGNAL(connectionReady()), SLOT(handleConnectionReady()));

    Tp::Features features;
    features << Tp::Account::FeatureCore;
    features << Tp::Account::FeatureProtocolInfo;
    features << Tp::Account::FeatureAvatar;

    if (mAccount->isReady(features)) {
        emit accountReady();
    } else {
        connect(mAccount->becomeReady(features),
                SIGNAL(finished(Tp::PendingOperation *)),
                SLOT(onAccountReady(Tp::PendingOperation *)));
    }
}

TelepathyAccount::~TelepathyAccount()
{
    clearContactList();
}

QString TelepathyAccount::id() const
{
    return mAccount->uniqueIdentifier();
}

bool TelepathyAccount::isReady() const
{
    return mReady;
}

bool TelepathyAccount::isOnline() const
{
    Tp::ConnectionStatus status = mAccount->connectionStatus();

    return (status == Tp::ConnectionStatusConnected);
}

QList<TelepathyContact *> TelepathyAccount::contactList() const
{
    return mContactList;
}

QString TelepathyAccount::displayName() const
{
    return mDisplayName;
}

void TelepathyAccount::onAccountReady(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qDebug() << "Error: " << op->errorName() << op->errorMessage();
        return;
    }

    emit accountReady();
}

void TelepathyAccount::handleAccountReady()
{
    connect(mAccount.data(),
            SIGNAL(haveConnectionChanged(bool)),
            SLOT(connectionChanged(bool)));

    if (mAccount.data()->haveConnection() && mConnection.isNull()) 
        connectionChanged(true);
}

void TelepathyAccount::connectionChanged(bool online)
{
    if (online) {
        if (!mConnection.isNull()) {
            qDebug() << "Connection should be null.";
            return;
        }

        mConnection = mAccount->connection();

        Tp::Features features;
        features << Tp::Connection::FeatureCore;
        features << Tp::Connection::FeatureSimplePresence;
        features << Tp::Connection::FeatureSelfContact;
        features << Tp::Connection::FeatureRoster;
        //features << Tp::Connection::FeatureRosterGroups;

        if (mConnection->isReady(features)) {
            emit connectionReady();
        } else {
            connect(mConnection->becomeReady(features),
                    SIGNAL(finished(Tp::PendingOperation*)),
                    SLOT(onConnectionReady(Tp::PendingOperation*)));
        }
    } else {
        qDebug() << "Connection is down";
        mConnection.reset();
        clearContactList();
    }
}

void TelepathyAccount::onConnectionReady(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qDebug() << "Error: " << op->errorName() << op->errorMessage();
        mConnection.reset();
        return;
    }

    emit connectionReady();
}

void TelepathyAccount::handleConnectionReady()
{
    if (!mConnection->contactManager()) {
        qDebug() << "ContactManager is null";
        return;
    }

    Tp::ContactManager *contactManager = mConnection->contactManager();

    Tp::Contacts contacts = contactManager->allKnownContacts();

    foreach (Tp::ContactPtr contact, contacts) {
        if (!hasContact(contact))
            addContact(contact);
    }

    QSet<Tp::Contact::Feature> features;

    features << Tp::Contact::FeatureAlias;
    features << Tp::Contact::FeatureInfo;
    features << Tp::Contact::FeatureSimplePresence;
    features << Tp::Contact::FeatureCapabilities;
    features << Tp::Contact::FeatureAvatarToken;

    if (mConnection->avatarsInterface()) {
        connect(mConnection->avatarsInterface(),
                SIGNAL(AvatarRetrieved(uint, QString, QByteArray, QString)),
                SLOT(onContactAvatarRetrieved(uint, QString, QByteArray, QString)));
        connect(mConnection->avatarsInterface(),
                SIGNAL(AvatarUpdated(uint, const QString&)),
                SLOT(onContactAvatarUpdated(uint, const QString&)));
    } else {
        qDebug() << "AvatarsInterface not supported for this protocol";
    }
    
    if (mConnection->contactInfoInterface()) {
        connect(contactManager->upgradeContacts(contacts.toList(), features),
                SIGNAL(finished(Tp::PendingOperation*)),
                SLOT(onUpgradeContacts(Tp::PendingOperation*)));

        connect(mConnection->contactInfoInterface(),
                SIGNAL(ContactInfoChanged(uint, const Tp::ContactInfoFieldList&)),
                SLOT(contactInfoStub(uint, const Tp::ContactInfoFieldList&)));
    } else {
        qDebug() << "ContactInfoInterface not supported for this protocol";
    }

    connect(contactManager,
            SIGNAL(allKnownContactsChanged(const Tp::Contacts&, const Tp::Contacts&)),
            SLOT(onAllKnownContactsChanged(const Tp::Contacts&, const Tp::Contacts&)));
    connect(contactManager,
            SIGNAL(groupMembersChanged(const QString&, const Tp::Contacts&, const Tp::Contacts&)),
            SLOT(onGroupMembersChanged(const QString&, const Tp::Contacts&, const Tp::Contacts&)));
}

void TelepathyAccount::onUpgradeContacts(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qDebug() << "Error: " << op->errorName() << op->errorMessage();
        return;
    }

    Tp::PendingContacts *pendingContacts = 
        qobject_cast<Tp::PendingContacts*>(op);

    QList<Tp::ContactPtr> contacts = pendingContacts->contacts();

    Tp::UIntList avatarsToRetrieve;

    foreach (Tp::ContactPtr contact, contacts) {

        TelepathyContact *telepathyContact = getContactByPtr(contact);
        telepathyContact->updateInfo();

        if (telepathyContact && telepathyContact->avatarToken().isEmpty())
            avatarsToRetrieve << contact->handle().toList();
    }

    retrieveAvatars(avatarsToRetrieve);

    setReady(true);
}

void TelepathyAccount::onUpgradeNewContacts(Tp::PendingOperation *op)
{
    qDebug() << "onUpgradeNewContacts";
    if (op->isError()) {
        qDebug() << "Error: " << op->errorName() << op->errorMessage();
        return;
    }

    Tp::PendingContacts *pendingContacts = 
        qobject_cast<Tp::PendingContacts*>(op);

    QList<Tp::ContactPtr> contacts = pendingContacts->contacts();

    Tp::UIntList avatarsToRetrieve;
    QList<TelepathyContact*> addedContacts;

    foreach (Tp::ContactPtr contact, contacts) {

        TelepathyContact *telepathyContact = getContactByPtr(contact);

        qDebug() << "Adding" << telepathyContact->id() << "to addedContacts";
        telepathyContact->updateInfo();

        if (telepathyContact && telepathyContact->avatarToken().isEmpty())
            avatarsToRetrieve << contact->handle().toList();

        addedContacts << telepathyContact;
    }

    retrieveAvatars(avatarsToRetrieve);

    emit contactsAdded(addedContacts);
}

void TelepathyAccount::retrieveAvatars(const Tp::UIntList &avatarsToRetrieve)
{
    if (!avatarsToRetrieve.isEmpty()) {
        QDBusPendingReply<Tp::AvatarTokenMap> reply=
            mConnection->avatarsInterface()->GetKnownAvatarTokens(avatarsToRetrieve);

        reply.waitForFinished();

        if (!reply.value().isEmpty()) {
            Tp::AvatarTokenMap result = reply.value();

            for (Tp::AvatarTokenMap::const_iterator i = result.constBegin();
                    i != result.constEnd(); i++) {
                if (!i.value().isEmpty())
                    onContactAvatarUpdated(i.key(), i.value());
            }
        }
    }
}

void TelepathyAccount::clearContactList()
{
    while (!mContactList.isEmpty()) {
        TelepathyContact *contact = mContactList.takeFirst();

        delete contact;
    }
}

void TelepathyAccount::setReady(bool r)
{
    mReady = r;

    if (mReady)
        emit gotReady(this);
}

void TelepathyAccount::onContactAvatarRetrieved(uint contact, const QString &token,
                const QByteArray &avatar, const QString &)
{
    TelepathyContact *telepathyContact = getContactByHandle(contact);

    if (telepathyContact) {
        telepathyContact->setAvatar(token, avatar);
        emit contactChanged(telepathyContact);
    }
}

TelepathyContact * TelepathyAccount::getContactByHandle(uint handle)
{
    Tp::ContactPtr tpContact = mConnection->contactManager()->lookupContactByHandle(handle);

    foreach (TelepathyContact *contact, mContactList) {
        if (contact->tpContact() == tpContact)
            return contact;
    }

    return 0;
}

void TelepathyAccount::onContactAvatarUpdated(uint contact, const QString &token)
{
    TelepathyContact *telepathyContact = getContactByHandle(contact);

    if (telepathyContact) {
        if (telepathyContact->avatarToken() != token) {
            QDBusPendingReply<void> reply =
                mConnection->avatarsInterface()->RequestAvatars(telepathyContact->tpContact()->handle().toList());
        }
    }
}

void TelepathyAccount::onAllKnownContactsChanged(const Tp::Contacts &added, const Tp::Contacts &removed)
{
    handleNewContacts(added);
    handleRemovedContacts(removed);
}

void TelepathyAccount::handleNewContacts(const Tp::Contacts &added)
{
    qDebug() << "handleNewContacts()";
    Tp::Contacts realAdded;

    foreach (const Tp::ContactPtr &contact, added) {
        if (!hasContact(contact))
            realAdded << contact;
    }

    if (!realAdded.isEmpty()) {
        foreach (Tp::ContactPtr contact, realAdded) {
            addContact(contact);
        }

        QSet<Tp::Contact::Feature> features;
        features << Tp::Contact::FeatureAlias;
        features << Tp::Contact::FeatureInfo;
        features << Tp::Contact::FeatureSimplePresence;
        features << Tp::Contact::FeatureCapabilities;
        features << Tp::Contact::FeatureAvatarToken;

        connect(mConnection->contactManager()->upgradeContacts(realAdded.toList(), features),
                SIGNAL(finished(Tp::PendingOperation*)),
                SLOT(onUpgradeNewContacts(Tp::PendingOperation*)));
    }
}

void TelepathyAccount::handleRemovedContacts(const Tp::Contacts &removed)
{
    Tp::Contacts realRemoved;

    foreach (const Tp::ContactPtr &contact, removed) {
        if (hasContact(contact))
            realRemoved << contact;
    }

    if (!realRemoved.isEmpty()) {
        foreach (const Tp::ContactPtr &contact, realRemoved) {
            TelepathyContact *telepathyContact = getContactByPtr(contact);

            if (telepathyContact) {
                mContactList.removeAll(telepathyContact);

                emit contactRemoved(telepathyContact);
            }
        }
    }
}

void TelepathyAccount::onGroupMembersChanged(const QString &group, const Tp::Contacts &added, const Tp::Contacts &removed)
{
    foreach( const Tp::ContactPtr &contact, added ) {
        qDebug() << "Contact added:" << contact->id();
    }

    foreach( const Tp::ContactPtr &contact, removed ) {
        qDebug() << "Contact removed:" << contact->id();
    }
}

void TelepathyAccount::onContactDestroyed(const Tp::ContactPtr &tpContact)
{
    TelepathyContact *contact = getContactByPtr(tpContact);

    if (contact) {
        mContactList.removeAll(contact);
        contact->deleteLater();
    }
}

void TelepathyAccount::onContactSubscriptionChanged(TelepathyContact *contact)
{
    qDebug() << "Contact" << contact->id() << "subscription changed to"
        << contact->subscription();
    emit contactChanged(contact);
}

void TelepathyAccount::onContactInfoUpdated(TelepathyContact *contact)
{
    qDebug() << "Contact info updated for contact" << contact->id();
    emit contactChanged(contact);
}

void TelepathyAccount::onContactPresenceChanged(TelepathyContact *contact)
{
    qDebug() << "Presence changed for contact" << contact->id();
    emit contactChanged(contact);
}

void TelepathyAccount::onContactCapabilitiesChanged(TelepathyContact *contact)
{
    qDebug() << "Capabilities changed for contact" << contact->id();
    emit contactChanged(contact);
}

void TelepathyAccount::contactInfoStub(uint contact, const Tp::ContactInfoFieldList &contactInfo)
{
    qDebug() << "Contact info stub: info updated for contact" << 
        getContactByHandle(contact)->id();
}

void TelepathyAccount::addContact(Tp::ContactPtr &tpContact)
{
    TelepathyContact *telepathyContact = new TelepathyContact(tpContact, this);
    telepathyContact->setAccountId(id());

    connect(telepathyContact,
            SIGNAL(subscriptionChanged(TelepathyContact*)),
            SLOT(onContactSubscriptionChanged(TelepathyContact*)));

    connect(telepathyContact,
            SIGNAL(infoUpdated(TelepathyContact*)),
            SLOT(onContactInfoUpdated(TelepathyContact*)));

    connect(telepathyContact,
            SIGNAL(presenceChanged(TelepathyContact*)),
            SLOT(onContactPresenceChanged(TelepathyContact*)));

    connect(telepathyContact,
            SIGNAL(capabilitiesChanged(TelepathyContact*)),
            SLOT(onContactCapabilitiesChanged(TelepathyContact*)));

    qDebug() << "Added new contact to cache " << telepathyContact->id();

    mContactList << telepathyContact;
}

TelepathyContact * TelepathyAccount::getContactByPtr(const Tp::ContactPtr &tpContact)
{
    foreach (TelepathyContact *contact, mContactList) {
        if (contact->tpContact() == tpContact)
            return contact;
    }

    return 0;
}

bool TelepathyAccount::hasContact(Tp::ContactPtr tpContact) const
{
    foreach (TelepathyContact *contact, mContactList) {
        if (contact->tpContact() == tpContact)
            return true;
    }

    return false;
}
