/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <TelepathyQt4/PendingContactInfo>
#include <TelepathyQt4/ContactCapabilities>
#include <QImage>

#include "telepathycontact.h"

enum ConnectionPresenceType { 
	ConnectionPresenceTypeUnknown = 7,
	ConnectionPresenceTypeError = 8
};

TelepathyContact::TelepathyContact(Tp::ContactPtr cnt, QObject *parent) :
    QObject(parent),
    mContact(cnt),
    mId("unknown mId"),
    mAlias(""),
    mFullName(""),
    mEmail(""),
    mUrl(""),
    mTel(""),
    mOrg(""),
    mSubscription( Unsubscribed ),
    mSupportsTextChats(false),
    mSupportsMediaCalls(false),
    mSupportsAudioCalls(false),
    mSupportsVideoCalls(false),
    mSupportsUpgradingCalls(false),
    mReady(true)
{
    setId(cnt->id());

    connect(mContact.data(), 
            SIGNAL(infoChanged(const Tp::ContactInfoFieldList&)),
            SLOT(handleInfoChanged(const Tp::ContactInfoFieldList&)));

    connect(mContact.data(),
            SIGNAL(subscriptionStateChanged(Tp::Contact::PresenceState)),
            SLOT(onSubscriptionStateChanged(Tp::Contact::PresenceState)));

    connect(mContact.data(),
            SIGNAL(aliasChanged(const QString&)),
            SLOT(onAliasChanged(const QString&)));

    connect(mContact.data(),
            SIGNAL(simplePresenceChanged(const QString&, uint, const QString &)),
            SLOT(onPresenceChanged(const QString&, uint, const QString&)));

    connect(mContact.data(),
            SIGNAL(capabilitiesChanged(Tp::ContactCapabilities *)),
            SLOT(onCapabilitiesChanged(Tp::ContactCapabilities *)));
    
    if (mContact->capabilities())
        onCapabilitiesChanged(mContact->capabilities());
}

TelepathyContact::~TelepathyContact()
{
}

void TelepathyContact::updateInfo()
{
    if (mContact->presenceType() == ConnectionPresenceTypeUnknown || 
    	    mContact->presenceType() == ConnectionPresenceTypeError) {
        return;
    }

    Tp::PendingContactInfo *contactInfo = mContact->requestInfo();

    connect(contactInfo,
            SIGNAL(finished(Tp::PendingOperation*)),
            SLOT(onRefreshInfo(Tp::PendingOperation*)));
}

void TelepathyContact::handleInfoChanged(const Tp::ContactInfoFieldList& mInfo)
{
    foreach(Tp::ContactInfoField field, mInfo) {


        if (field.fieldValue.isEmpty())
            continue;
        if (field.fieldName == "fn")
            setFullName(field.fieldValue.first());
        else if (field.fieldName == "mEmail")
            setEmail(field.fieldValue.first());
        else if (field.fieldName == "email")
            setEmail(field.fieldValue.first());
        else if (field.fieldName == "nickname")
            setAlias(field.fieldValue.first());
        else if (field.fieldName == "mUrl")
            setUrl(field.fieldValue.first());
        else if (field.fieldName == "url")
            setUrl(field.fieldValue.first());
        else if (field.fieldName == "mTel")
            setTel(field.fieldValue.first());
        else if (field.fieldName == "tel")
            setTel(field.fieldValue.first());
        else if (field.fieldName == "mOrg")
            setOrg(field.fieldValue.first());
        else if (field.fieldName == "org")
            setOrg(field.fieldValue.first());
        else if (field.fieldName == "bday" )
            setBirthday(field.fieldValue.first());
        else if (field.fieldName == "prodid")
            setProductId(field.fieldValue.first());
        else if (field.fieldName == "title")
            setTitle(field.fieldValue.first());
        else if (field.fieldName == "adr")
            setAddressInfo(field.fieldValue);
    }

    if ( mContact->subscriptionState() == Tp::Contact::PresenceStateYes )
        setSubscription( Subscribed );
    else if ( mContact->subscriptionState() == Tp::Contact::PresenceStateNo )
        setSubscription( Unsubscribed );
    else if ( mContact->subscriptionState() == Tp::Contact::PresenceStateAsk )
        setSubscription( Ask );

    emit infoUpdated(this);
}

void TelepathyContact::onRefreshInfo(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qDebug() << "Error refreshing mContact mInfo";
        return;
    }

    Tp::PendingContactInfo *contactInfo = 
        qobject_cast<Tp::PendingContactInfo*>(op);

    handleInfoChanged(contactInfo->info());
}

void TelepathyContact::onAliasChanged(const QString &alias)
{
    setAlias(alias);

    emit infoUpdated(this);
}

void TelepathyContact::onPresenceChanged(const QString &status, uint type,
        const QString &message)
{
    setPresenceStatus(status);
    setPresenceType(type);
    setPresenceMessage(message);

    emit presenceChanged(this);
}

void TelepathyContact::onCapabilitiesChanged(Tp::ContactCapabilities *capabilities)
{
    setSupportsUpgradingCalls(capabilities->supportsUpgradingCalls());
    setSupportsVideoCalls(capabilities->supportsVideoCalls());
    setSupportsMediaCalls(capabilities->supportsMediaCalls());
    setSupportsTextChats(capabilities->supportsTextChats());
    setSupportsAudioCalls(capabilities->supportsAudioCalls());

    emit capabilitiesChanged(this);
}

void TelepathyContact::onSubscriptionStateChanged(Tp::Contact::PresenceState state)
{
    if (state == Tp::Contact::PresenceStateYes) {
        setSubscription( Subscribed );
        emit subscriptionChanged(this);
    } else if (state == Tp::Contact::PresenceStateNo) {
        setSubscription( Unsubscribed );
        emit subscriptionChanged(this);
    } else if (state == Tp::Contact::PresenceStateAsk) {
        setSubscription( Ask );
        emit subscriptionChanged(this);
    }
}

void TelepathyContact::setId(const QString &newId)
{
    mId = newId;
}

void TelepathyContact::setAlias(const QString &al)
{
    mAlias = al;
}

void TelepathyContact::setFullName(const QString &fn)
{
    mFullName = fn;
}

void TelepathyContact::setEmail(const QString &em)
{
    mEmail = em;
}

void TelepathyContact::setUrl(const QString &u)
{
    mUrl = u;
}

void TelepathyContact::setTel(const QString &t)
{
    mTel = t;
}

void TelepathyContact::setOrg(const QString &o)
{
    mOrg = o;
}

void TelepathyContact::setBirthday(const QString &b)
{
    mBirthday = b;
}

void TelepathyContact::setProductId(const QString &p)
{
    mProductId = p;
}

void TelepathyContact::setTitle(const QString &t)
{
    mTitle = t;
}

void TelepathyContact::setStreet(const QString &s)
{
    mStreet = s;
}

void TelepathyContact::setCity(const QString &c)
{
    mCity = c;
}

void TelepathyContact::setState(const QString &s)
{
    mState = s;
}

void TelepathyContact::setZip(const QString &z)
{
    mZip = z;
}

void TelepathyContact::setCountry(const QString &c)
{
    mCountry = c;
}

void TelepathyContact::setReady(bool isReady)
{
    mReady = isReady;

    if (mReady)
        emit gotReady(this);
}

void TelepathyContact::setAddressInfo(const QStringList &info)
{
    if (info.size() >= 2 )
        setStreet(info.at(2));
    if (info.size() >= 3 )
        setCity(info.at(3));
    if (info.size() >= 4 )
        setState(info.at(4));
    if (info.size() >= 5 )
        setZip(info.at(5));
    if (info.size() >= 6 )
        setCountry(info.at(6));
}

void TelepathyContact::setSubscription( Subscription subscription )
{
    mSubscription = subscription;
}

void TelepathyContact::setPresenceStatus(const QString &status)
{
    mPresenceStatus = status;
}

void TelepathyContact::setPresenceMessage(const QString &message)
{
    mPresenceMessage = message;
}

void TelepathyContact::setPresenceType(uint type)
{
    mPresenceType = type;
}

void TelepathyContact::setSupportsTextChats(bool supportsTextChats)
{
    mSupportsTextChats = supportsTextChats;
}

void TelepathyContact::setSupportsMediaCalls(bool supportsMediaCalls)
{
    mSupportsMediaCalls = supportsMediaCalls;
}

void TelepathyContact::setSupportsAudioCalls(bool supportsAudioCalls)
{
    mSupportsAudioCalls = supportsAudioCalls;
}

void TelepathyContact::setSupportsVideoCalls(bool supportsVideoCalls)
{
    mSupportsVideoCalls = supportsVideoCalls;
}

void TelepathyContact::setSupportsUpgradingCalls(bool supportsUpgradingCalls)
{
    mSupportsUpgradingCalls = supportsUpgradingCalls;
}

QString TelepathyContact::id() const
{
    return mId;
}

QString TelepathyContact::alias() const
{
    return mAlias;
}

QString TelepathyContact::fullName() const
{
    return mFullName;
}

QString TelepathyContact::email() const
{
    return mEmail;
}

QString TelepathyContact::url() const
{
    return mUrl;
}

QString TelepathyContact::tel() const
{
    return mTel;
}

QString TelepathyContact::org() const
{
    return mOrg;
}

QString TelepathyContact::birthday() const
{
    return mBirthday;
}

QString TelepathyContact::productId() const
{
    return mProductId;
}

QString TelepathyContact::title() const
{
    return mTitle;
}

QString TelepathyContact::street() const
{
    return mStreet;
}

QString TelepathyContact::city() const
{
    return mCity;
}

QString TelepathyContact::state() const
{
    return mState;
}

QString TelepathyContact::zip() const
{
    return mZip;
}

QString TelepathyContact::country() const
{
    return mCountry;
}

QString TelepathyContact::presenceStatus() const
{
    return mPresenceStatus;
}

QString TelepathyContact::presenceMessage() const
{
    return mPresenceMessage;
}

uint TelepathyContact::presenceType() const
{
    return mPresenceType;
}

bool TelepathyContact::isReady() const
{
    return mReady;
}

bool TelepathyContact::supportsTextChats() const
{
    return mSupportsTextChats;
}

bool TelepathyContact::supportsMediaCalls() const
{
    return mSupportsMediaCalls;
}

bool TelepathyContact::supportsAudioCalls() const
{
    return mSupportsAudioCalls;
}

bool TelepathyContact::supportsVideoCalls() const
{
    return mSupportsVideoCalls;
}

bool TelepathyContact::supportsUpgradingCalls() const
{
    return mSupportsUpgradingCalls;
}

void TelepathyContact::setAvatar(const QString &token, const QByteArray &avatar)
{
    mAvatarToken = token;
    mAvatar.loadFromData(avatar);
}

void TelepathyContact::setAccountId(const QString &id)
{
    mAccountId = id;
}

const Tp::ContactPtr TelepathyContact::tpContact() const
{
    return mContact;
}

const QString TelepathyContact::avatarToken() const
{
    return mAvatarToken;
}

const QString TelepathyContact::accountId() const
{
    return mAccountId;
}

const QImage & TelepathyContact::avatar() const
{
    return mAvatar;
}

TelepathyContact::Subscription TelepathyContact::subscription() const
{
    return mSubscription;
}
