/* GroupDAV Akonadi Resource
 * Copyright (C) 2009 Marco Nelles (marco.nelles@credativ.de)
 * <http://www.credativ.de/>
 * <http://opensource.maniatek.com/kgroupdav/>
 *
 * credativ GmbH
 * Hohenzollernstr. 133
 * 41061 Mönchengladbach
 * E-Mail: info@credativ.de
 *
 * RFC 2445 (iCalendar)
 * RFC 2518 (WebDAV)
 * RFC 2616 (HTTP)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "groupdavresource.h"

using namespace Akonadi;

GroupDAVResource::GroupDAVResource(const QString &id) : ResourceBase(id) {

  new SettingsAdaptor(Settings::self());
  QDBusConnection::sessionBus().registerObject(QLatin1String("/Settings"), Settings::self(), QDBusConnection::ExportAdaptors);

  changeRecorder()->itemFetchScope().fetchFullPayload();

  //p_monitor = new Monitor(this);
  //p_monitor->itemFetchScope().fetchFullPayload();

  connect(&p_proxy, SIGNAL(itemAdded(const quint32)), this, SLOT(proxyItemAdd(const quint32)));
  connect(&p_proxy, SIGNAL(itemModed(const quint32)), this, SLOT(proxyItemMod(const quint32)));
  connect(&p_proxy, SIGNAL(itemDeleted(const quint32)), this, SLOT(proxyItemDel(const quint32)));

}

GroupDAVResource::~GroupDAVResource() {

  //delete p_monitor;

}

// this method is called when Akonadi wants to have all the
// collections your resource provides.
// Be sure to set the remote ID and the content MIME types
void GroupDAVResource::retrieveCollections() {

  Collection c;
  c.setParentCollection(Collection::root());
  c.setRemoteId("calgo-groupdav-calendar");
  c.setName(name());

  QStringList mimeTypes;
  mimeTypes << "text/calendar";
  c.setContentMimeTypes(mimeTypes);

  Collection::List list;
  list << c;
  collectionsRetrieved(list);

}

// this method is called when Akonadi wants to know about all the
// items in the given collection. You can but don't have to provide all the
// data for each item, remote ID and MIME type are enough at this stage.
// Depending on how your resource accesses the data, there are several
// different ways to tell Akonadi when you are done.
void GroupDAVResource::retrieveItems(const Akonadi::Collection &collection) {

  Q_UNUSED(collection);

  if (p_proxy.syncInProgress()) {
    kDebug() << "Sync in progress. Cancel retrieve items.";
    cancelTask();
    return;
  }

  kDebug() << "retrieve items";

  Item::List items;
  IdList ids = p_proxy.ids();
  for (int i = 0; i < ids.length(); ++i) {
    Item item(QLatin1String("text/calendar"));
    item.setRemoteId(QString("%1-%2").arg(ids.value(i)).arg(p_proxy.checksum(ids.value(i))));
    items << item;
  }

  itemsRetrieved(items);

}

// this method is called when Akonadi wants more data for a given item.
// You can only provide the parts that have been requested but you are allowed
// to provide all in one go
bool GroupDAVResource::retrieveItem(const Akonadi::Item &item, const QSet<QByteArray> &parts) {

  Q_UNUSED(parts);

  bool ok;
  int dPos = item.remoteId().indexOf('-');
  quint32 id = item.remoteId().left(dPos).toUInt(&ok);
  if (!ok) {
    kDebug() << "Invalid remote id.";
    cancelTask();
    return FALSE;
  }

  KCal::Incidence *inc = NULL;
  KCal::CalendarLocal calendar(KDateTime::UTC);
  KCal::ICalFormat ical;

  if (ical.fromString(&calendar, p_proxy.itemData(id))) {
    KCal::Incidence::List raw = calendar.rawIncidences();
    if (raw.count() == 0) {
      kDebug() << "Parsed iCalendar has no event. Skipping.";
      cancelTask();
      return FALSE;
    }
    if (raw.count() > 1) {
      kDebug() << "Parsed iCalendar has more than one event. Taking only the first one.";
    }
    inc = (raw.front())->clone();
  } else {
    kDebug() << "Unable to parse iCalendar";
    cancelTask();
    return FALSE;
  }
  if (!inc) {
    kDebug() << "Unable to parse iCalendar";
    cancelTask();
    return FALSE;
  }

  Item newItem(item);
  newItem.setPayload(KCal::Incidence::Ptr(inc));

  itemRetrieved(newItem);

  return TRUE;

}

void GroupDAVResource::synchronize() {

  ResourceBase::synchronize();

}

void GroupDAVResource::proxyItemAdd(const quint32 id) {

  kDebug() << "item add detected:" << id;

  KCal::Incidence *inc = NULL;
  KCal::CalendarLocal calendar(KDateTime::UTC);
  KCal::ICalFormat ical;

  if (ical.fromString(&calendar, p_proxy.itemData(id))) {
    KCal::Incidence::List raw = calendar.rawIncidences();
    if (raw.count() == 0) {
      kDebug() << "Parsed iCalendar has no event. Skipping.";
      return;
    }
    if (raw.count() > 1) {
      kDebug() << "Parsed iCalendar has more than one event. Taking only the first one.";
    }
    inc = (raw.front())->clone();
  } else {
    kDebug() << "Unable to parse iCalendar";
    return;
  }
  if (!inc) {
    kDebug() << "Unable to parse iCalendar";
    return;
  }

  Item newItem;
  newItem.setMimeType(QLatin1String("text/calendar"));
  newItem.setRemoteId(QString("%1-%2").arg(id).arg(p_proxy.checksum(id)));
  newItem.setPayload(KCal::Incidence::Ptr(inc));

  Akonadi::Collection collection = currentCollection();
  Akonadi::ItemCreateJob *job = new Akonadi::ItemCreateJob(newItem, collection);

  if (job->exec()) {
    kDebug() << "item added:" << id;
  } else {
    kDebug() << "Error occurred";
    return;
  }

}

void GroupDAVResource::proxyItemMod(const quint32 id) {

  kDebug() << "item mod detected:" << id;

  Akonadi::Collection collection = currentCollection();
  Akonadi::Item item;

  Akonadi::ItemFetchJob *fjob = new Akonadi::ItemFetchJob(collection);
  fjob->fetchScope().fetchFullPayload(FALSE);

  if (fjob->exec()) {

    bool found = FALSE;
    Akonadi::Item::List items = fjob->items();
    foreach (const Akonadi::Item &nitem, items) {
      bool ok;
      int dPos = nitem.remoteId().indexOf('-');
      quint32 nid = nitem.remoteId().left(dPos).toUInt(&ok);
      if (!ok) continue;
      if (nid==id) {
        item = nitem;
        found = TRUE;
        break;
      }
    }

    if (!found) {
      kDebug() << "item to mod not found";
      return;
    }

    KCal::Incidence *inc = NULL;
    KCal::CalendarLocal calendar(KDateTime::UTC);
    KCal::ICalFormat ical;

    if (ical.fromString(&calendar, p_proxy.itemData(id))) {
      KCal::Incidence::List raw = calendar.rawIncidences();
      if (raw.count() == 0) {
        kDebug() << "Parsed iCalendar has no event. Skipping.";
        return;
      }
      if (raw.count() > 1) {
        kDebug() << "Parsed iCalendar has more than one event. Taking only the first one.";
      }
      inc = (raw.front())->clone();
    } else {
      kDebug() << "Unable to parse iCalendar";
      return;
    }
    if (!inc) {
      kDebug() << "Unable to parse iCalendar";
      return;
    }

    item.setRemoteId(QString("%1-%2").arg(id).arg(p_proxy.checksum(id)));
    item.setPayload(KCal::Incidence::Ptr(inc));

    Akonadi::ItemModifyJob *mjob = new Akonadi::ItemModifyJob(item);

    if (mjob->exec()) {
      kDebug() << "item moded:" << id;
    } else {
      kDebug() << "Error occurred";
    }

  } else {

    kDebug() << "Error occurred";

  }

}

void GroupDAVResource::proxyItemDel(const quint32 id) {

  kDebug() << "item del detected:" << id;

  Akonadi::Collection collection = currentCollection();
  Akonadi::Item item;

  Akonadi::ItemFetchJob *job = new Akonadi::ItemFetchJob(collection);
  job->fetchScope().fetchFullPayload(FALSE);

  if (job->exec()) {

    bool found = FALSE;
    Akonadi::Item::List items = job->items();
    foreach (const Akonadi::Item &nitem, items) {
      bool ok;
      int dPos = nitem.remoteId().indexOf('-');
      quint32 nid = nitem.remoteId().left(dPos).toUInt(&ok);
      if (!ok) continue;
      if (nid==id) {
        item = nitem;
        found = TRUE;
        break;
      }
    }

    if (!found) {
      kDebug() << "item to del not found";
      return;
    }

    ItemDeleteJob *job = new ItemDeleteJob(item);

    if (job->exec()) {
      kDebug() << "item deleted:" << id;
    } else {
      kDebug() << "Error occurred";
    }

  } else {

    kDebug() << "Error occurred";

  }

}

// any cleanup you need to do while there is still an active
// event loop. The resource will terminate after this method returns
void GroupDAVResource::aboutToQuit() {

}

// this method is usually called when a new resource is being
// added to the Akonadi setup. You can do any kind of user interaction here,
// e.g. showing dialogs.
// The given window ID is usually useful to get the correct
// "on top of parent" behavior if the running window manager applies any kind
// of focus stealing prevention technique
void GroupDAVResource::configure(WId windowId) {

  ConfigDialog dlg(windowId);

  if (dlg.exec()) {
    p_load_configuration();
    emit configurationDialogAccepted();
  } else {
    emit configurationDialogRejected();
  }

  //synchronize();
  p_proxy.sync();

}

// this method is called when somebody else, e.g. a client application,
// has created an item in a collection managed by your resource.
void GroupDAVResource::itemAdded(const Akonadi::Item &item, const Akonadi::Collection &collection) {

  kDebug() << "item added";

  Q_UNUSED(collection);

  KCal::Incidence *inc = NULL;
  if (item.hasPayload<KCal::Incidence::Ptr>()) {
    inc = item.payload<KCal::Incidence::Ptr>().get();
  }
  if (!inc) {
    kDebug() << "Item has no proper payload.";
    cancelTask();
    return;
  }

  KCal::ICalFormat format;
  format.setTimeSpec(KDateTime::UTC);
  quint32 id = p_proxy.add(format.toICalString(inc));

  Item newItem(item);
  newItem.setRemoteId(QString("%1-%2").arg(id).arg(p_proxy.checksum(id)));
  newItem.setPayload(KCal::Incidence::Ptr(inc->clone()));

  changeCommitted(newItem);

}

// this method is called when somebody else, e.g. a client application,
// has changed an item managed by your resource.
void GroupDAVResource::itemChanged(const Akonadi::Item &item, const QSet<QByteArray> &parts) {

  kDebug() << "item moded";

  Q_UNUSED(parts);

  bool ok;
  int dPos = item.remoteId().indexOf('-');
  quint32 id = item.remoteId().left(dPos).toUInt(&ok);
  if (!ok) {
    kDebug() << "Invalid remote id.";
    cancelTask();
    return;
  }

  KCal::Incidence *inc = NULL;
  if (item.hasPayload<KCal::Incidence::Ptr>()) {
    inc = item.payload<KCal::Incidence::Ptr>().get();
  }
  if (!inc) {
    kDebug() << "Item has no proper payload.";
    cancelTask();
    return;
  }

  KCal::ICalFormat format;
  if (!p_proxy.mod(id, format.toICalString(inc))) {
    kDebug() << "Item modification failed.";
    cancelTask();
    return;
  }

  Item newItem(item);
  newItem.setRemoteId(QString("%1-%2").arg(id).arg(p_proxy.checksum(id)));
  newItem.setPayload(KCal::Incidence::Ptr(inc->clone()));

  changeCommitted(newItem);

}

// this method is called when somebody else, e.g. a client application,
// has deleted an item managed by your resource.
void GroupDAVResource::itemRemoved(const Akonadi::Item &item) {

  kDebug() << "item deleted";

  bool ok;
  int dPos = item.remoteId().indexOf('-');
  quint32 id = item.remoteId().left(dPos).toUInt(&ok);
  if (!ok) {
    kDebug() << "Invalid remote id.";
    cancelTask();
    return;
  }

  if (!p_proxy.del(id)) {
    kDebug() << "Item delete failed.";
    cancelTask();
    return;
  }

  changeCommitted(item);

}

void GroupDAVResource::doSetOnline(bool online) {

  kDebug() << "online: " << online;

  ResourceBase::doSetOnline(online);
  if (online) {
    synchronize();
    p_load_configuration();
  } else {
    p_shutdown();
  }

}

bool GroupDAVResource::p_load_configuration() {

  if (Settings::self()->hostname().isEmpty()) {
    kDebug() << "Not configured yet";
    return FALSE;
  }

  p_proxy.setHost(Settings::self()->hostname());
  p_proxy.setPort(Settings::self()->port());
  p_proxy.setUsername(Settings::self()->username());
  p_proxy.setPassword(Settings::self()->password());
  p_proxy.setPath(Settings::self()->path());
  p_proxy.setSyncInterval(Settings::self()->syncFreq());
  p_proxy.setSyncOnChange(TRUE);

  p_proxy.sync();

  return TRUE;

}

void GroupDAVResource::p_shutdown() {

  p_proxy.setSyncInterval(0);

}

AKONADI_RESOURCE_MAIN(GroupDAVResource)

#include "groupdavresource.moc"
