/* KGroupDAVProxy - Implementation of GroupDAV protocol
 * Copyright (C) 2009 Marco Nelles (marco.nelles@credativ.de)
 * <http://www.credativ.de/>
 * <http://opensource.maniatek.com/kgroupdav/>
 *
 * credativ GmbH
 * Hohenzollernstr. 133
 * 41061 Mönchengladbach
 * E-Mail: info@credativ.de
 *
 * RFC 2445 (iCalendar)
 * RFC 2518 (WebDAV)
 * RFC 2616 (HTTP)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "kgroupdavproxy.h"
#include <QHttpResponseHeader>

KGroupDAVProxyDomParser:: KGroupDAVProxyDomParser() {

}

KGroupDAVProxyDomParser::~ KGroupDAVProxyDomParser() {

}

void KGroupDAVProxyDomParser::parse(const QDomDocument& doc, ItemHashByString *items) {

  this->doc = doc;
  this->items = items;

  QDomElement root = doc.documentElement();
  if (root.tagName() != TAG_MULTISTATUS) {
    p_last_error_message = QString("No %1 tag.").arg(TAG_MULTISTATUS);
    return;
  }

  QDomNode node = root.firstChild();
  while (!node.isNull()) {
    if (node.toElement().tagName() == TAG_RESPONSE) if (!p_parse_response(node.toElement())) return;
    node = node.nextSibling();
  }

}

bool KGroupDAVProxyDomParser::p_parse_response(const QDomElement& element) {

  href.clear();
  etag.clear();

  QDomNode node = element.firstChild();

  bool found = FALSE;
  while (!node.isNull()) {
    if (node.toElement().tagName() == TAG_HREF) {
      found = TRUE;
      href = node.firstChild().toText().data().trimmed();
      while (href.endsWith('/') && (href.length() > 0)) href.chop(1);
    } else if (node.toElement().tagName() == TAG_PROPSTAT) {
      if (!p_parse_propstat(node.toElement())) return FALSE;
    }
    node = node.nextSibling();
  }

  if (!found) {
    p_last_error_message = "Empty response.";
    return FALSE;
  }

  if ((!etag.isEmpty()) && (!href.isEmpty())) {
    GroupDAVItem *item = new GroupDAVItem();
    item->setEtag(etag);
    (*items)[href] = item;
  }

  return TRUE;

}

bool KGroupDAVProxyDomParser::p_parse_propstat(const QDomElement& element) {

  QDomNode node = element.firstChild();

  bool found = FALSE;
  while (!node.isNull()) {
    if (node.toElement().tagName() == TAG_PROP) {
      found = TRUE;
      if (!p_parse_prop(node.toElement())) return FALSE;
    }
    node = node.nextSibling();
  }

  if (!found) {
    p_last_error_message = "Empty propstat.";
    return FALSE;
  }

  return TRUE;

}

bool KGroupDAVProxyDomParser::p_parse_prop(const QDomElement& element) {

  QDomNode node = element.firstChild();

  while (!node.isNull()) {
    if (node.toElement().tagName() == TAG_GETETAG) {
      etag = node.firstChild().toText().data().trimmed();
      if (etag.startsWith('"')) etag = etag.mid(1);
      if (etag.endsWith('"')) etag.chop(1);
    }
    node = node.nextSibling();
  }

  return TRUE;

}

KGroupDAVProxyData::KGroupDAVProxyData() {
  p_id_peak = 0;
}

KGroupDAVProxyData::~KGroupDAVProxyData() {
  clear();
}

const GroupDAVItem* KGroupDAVProxyData::remoteItem(const QString& location) {

  p_remote_items_mutex.lock();
  const GroupDAVItem *item = p_remote_items.value(location, NULL);
  p_remote_items_mutex.unlock();

  return item;

}

void KGroupDAVProxyData::addRemoteItem(const QString& location, const QString& etag) {

  GroupDAVItem *item = new GroupDAVItem();
  item->setEtag(etag);
  p_remote_items_mutex.lock();
  p_remote_items[location] = item;
  p_remote_items_mutex.lock();

}

bool KGroupDAVProxyData::delRemoteItem(const QString& location) {

  p_remote_items_mutex.lock();
  GroupDAVItem *item = p_remote_items.value(location, NULL);
  bool ok = FALSE;
  if (item) {
    delete item;
    p_remote_items.remove(location);
    ok = TRUE;
  }
  p_remote_items_mutex.unlock();

  return ok;

}

const GroupDAVItem* KGroupDAVProxyData::item(const quint32 id) {

  p_items_mutex.lock();
  const GroupDAVItem *item = p_items.value(id, NULL);
  if ((item) && (item->isDel())) item = NULL;
  p_items_mutex.unlock();

  return item;

}

const QString KGroupDAVProxyData::itemData(const quint32 id) {

  QString data;
  p_items_mutex.lock();
  const GroupDAVItem *item = p_items.value(id, NULL);
  if (item) data = item->data();
  p_items_mutex.unlock();

  return data;

}

bool KGroupDAVProxyData::itemIsDeleted(const quint32 id) {

  p_items_mutex.lock();
  bool d = FALSE;
  const GroupDAVItem *item = p_items.value(id, NULL);
  if (item) d = item->isDel();
  p_items_mutex.unlock();

  return d;

}

quint32 KGroupDAVProxyData::addItem(const QString& data) {

  p_id_peak_mutex.lock();
  quint32 p = p_id_peak++;
  p_id_peak_mutex.unlock();

  GroupDAVItem *item = new GroupDAVItem(p);
  item->setData(data);
  item->setNew();
  p_items_mutex.lock();
  p_items[p] = item;
  p_items_mutex.unlock();
  return p;

}

bool KGroupDAVProxyData::modItem(const quint32 id, const QString& data) {

  p_items_mutex.lock();
  GroupDAVItem *item = p_items.value(id, NULL);
  bool ok = FALSE;
  if ((item) && (!item->isDel())) {
    if (item->isDel()) {
      p_items_mutex.unlock();
      return FALSE;
    }
    item->setData(data);
    item->setMod();
    ok = TRUE;
  }
  p_items_mutex.unlock();

  return ok;

}

bool KGroupDAVProxyData::delItem(const quint32 id) {

  p_items_mutex.lock();
  GroupDAVItem *item = p_items.value(id, NULL);
  bool ok = FALSE;
  if (item) {
    if (item->isNew()) {
      delete item;
      p_items.remove(id);
    } else {
      item->setDel();
    }
    ok = TRUE;
  }
  p_items_mutex.unlock();

  return ok;

}

const QByteArray KGroupDAVProxyData::md5sum(const quint32 id) {

  QByteArray result;
  p_items_mutex.lock();
  GroupDAVItem *item = p_items.value(id, NULL);
  if (item) {
    result = item->md5sum();
  }
  p_items_mutex.unlock();

  return result;

}

quint16 KGroupDAVProxyData::checksum(const quint32 id) {

  quint16 result = 0;
  p_items_mutex.lock();
  GroupDAVItem *item = p_items.value(id, NULL);
  if (item) {
    result = item->checksum();
  }
  p_items_mutex.unlock();

  return result;

}

const IdList KGroupDAVProxyData::ids() {

  IdList result;
  p_items_mutex.lock();
  ItemHash::const_iterator i = p_items.constBegin();
  while (i != p_items.constEnd()) {
    if (!i.value()->isDel()) result.append(i.value()->id());
    ++i;
  }
  p_items_mutex.unlock();
  return result;

}

int KGroupDAVProxyData::count() {

  int c = 0;
  p_items_mutex.lock();
  ItemHash::const_iterator i = p_items.constBegin();
  while (i != p_items.constEnd()) {
    if (!i.value()->isDel()) ++c;
    ++i;
  }
  p_items_mutex.unlock();
  return c;

}

void KGroupDAVProxyData::clear() {

  kDebug() << "complete clear.";
  clearItems();
  clearRemoteItems();

}

void KGroupDAVProxyData::clearItems() {

  p_items_mutex.lock();
  ItemHash::const_iterator i = p_items.constBegin();
  while (i != p_items.constEnd()) {
    if (i.value()) delete i.value();
    ++i;
  }
  p_items.clear();
  p_items_mutex.unlock();

}

void KGroupDAVProxyData::clearRemoteItems() {

  p_remote_items_mutex.lock();
  ItemHashByString::const_iterator i = p_remote_items.constBegin();
  while (i != p_remote_items.constEnd()) {
    if (i.value()) delete i.value();
    ++i;
  }
  p_remote_items.clear();
  p_remote_items_mutex.unlock();

}

void KGroupDAVProxyData::undelete() {

  p_items_mutex.lock();
  ItemHash::const_iterator i = p_items.constBegin();
  while (i != p_items.constEnd()) {
    if ((i.value()) && (!i.value()->isDel())) i.value()->setDel(FALSE);
    ++i;
  }
  p_items_mutex.unlock();

}

void KGroupDAVProxyData::purge() {

  p_items_mutex.lock();
  ItemHash::const_iterator i = p_items.constBegin();
  while (i != p_items.constEnd()) {
    if ((i.value()) && (i.value()->isDel())) {
      p_items.remove(i.key());
      delete i.value();
    }
    ++i;
  }
  p_items_mutex.unlock();

}

KGroupDAVProxySyncThread::KGroupDAVProxySyncThread(KGroupDAVProxyData *data, QObject *parent) : QThread(parent) {

  Q_UNUSED(parent);

  this->p_data = data;

  p_status = NOS;

  p_job = NULL;

  connect(this, SIGNAL(retRemoteItemsFinished()), this, SLOT(p_initiate_sync()));

}

KGroupDAVProxySyncThread::~KGroupDAVProxySyncThread() {

}

void KGroupDAVProxySyncThread::run() {

  QTimer::singleShot(0, this, SLOT(p_initiate_ret_remote_items()));

  exec();

}

void KGroupDAVProxySyncThread::p_initiate_ret_remote_items() {

  p_ret_remote_items();

}

void KGroupDAVProxySyncThread::p_initiate_sync() {

  p_sync();

  exit(0);

}

void KGroupDAVProxySyncThread::p_finished_ret_remote_items(KJob* job) {

  QDomDocument doc;

  if (job && job->error()) {

    kDebug() << "There was an error communicating with the server:" << job->error() << "," << job->errorString();
    p_set_status(NOS);
    return;

  }

  if (job) {

    KIO::DavJob* const davJob = static_cast<KIO::DavJob*>(job);
    p_parse_xml_remote_item_response(davJob->response(), p_data->remoteItems());

  }

  p_set_status(NOS);

  emit retRemoteItemsFinished();

}

void KGroupDAVProxySyncThread::p_finished_ret_item(KJob* job) {

  if (job && job->error()) {

    kDebug() << "There was an error communicating with the server:" << job->error() << "," << job->errorString();
    p_set_status(NOS);

  } else if (job) {

    KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>(job);

    QHttpResponseHeader header(storedJob->metaData().value("HTTP-Headers"));
    QString location = storedJob->outgoingMetaData().value("Location");
    QString etag = header.value("etag");
    if (etag.startsWith('"')) etag = etag.mid(1);
    if (etag.endsWith('"')) etag.chop(1);

    quint32 id;
    GroupDAVItem *item;
    bool mod = FALSE;
    if (p_resolver.locationExists(location)) {
      id = p_resolver.idByLocation(location);
      p_data->lockItems();
      item = p_data->items()->value(id);
      p_data->unlockItems();
      mod = TRUE;
    } else {
      id = p_data->peak();
      item = new GroupDAVItem(id);
      p_resolver.add(id, location);
    }
    item->setEtag(etag);
    item->setData(storedJob->data());
    item->setLocation(location);
    p_data->lockItems();
    (*p_data->items())[id] = item;
    p_data->unlockItems();

    if (mod) emit itemModed(id); else emit itemAdded(id);

    p_set_status(NOS);

    if (!p_ret_blocking) {
      if (p_locations_to_ret.length() > 0) {
        p_ret_item(p_locations_to_ret.takeFirst());
      } else {
        //emit p_items_retrieved();
        kDebug() << "all items retrieved";
      }
    }

  }

}

void KGroupDAVProxySyncThread::p_finished_del_item(KJob* job) {

  if (job && job->error()) {

    //FIXME: This seems to be a KDE error. Error code for delete job on groupdav server is always x49 (server time out)
    if (job->error() != 149) {
      kDebug() << "There was an error communicating with the server:" << job->error() << "," << job->errorString();
      p_set_status(NOS);
      return;
    }

  }

  quint32 id = p_last_deleted_item->id();

  if (p_last_deleted_item) {
    p_resolver.del(id);
    p_data->lockItems();
    p_data->items()->remove(id);
    p_data->unlockItems();
    delete p_last_deleted_item;
  }

  p_set_status(NOS);

  p_last_deleted_item = NULL;

  if (!p_del_blocking) {
    if (p_locations_to_del.length() > 0) {
      p_del_item(p_locations_to_del.takeFirst());
    } else {
      //emit itemsDeleted();
      kDebug() << "all items deleted";
    }
  }

}

void KGroupDAVProxySyncThread::p_finished_send_item(KJob* job) {

  if (job && job->error()) {

    kDebug() << "There was an error communicating with the server:" << job->error() << "," << job->errorString();
    p_set_status(NOS);
    return;

  }

  KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>(job);

  if (p_last_send_item->isNew()) {

    QHttpResponseHeader header(storedJob->metaData().value("HTTP-Headers"));
    p_last_send_item->setLocation(header.value("location"));
    //important: update resolver!
    p_resolver.del(p_last_send_item->id());
    p_resolver.add(p_last_send_item->id(), header.value("location"));
    QString etag = header.value("etag");
    if (etag.startsWith('"')) etag = etag.mid(1);
    if (etag.endsWith('"')) etag.chop(1);
    p_last_send_item->setEtag(etag);

    p_last_send_item->setNew(FALSE);

  } else if (p_last_send_item->isMod()) {

    p_last_send_item->setMod(FALSE);

  }

  p_last_send_item = NULL;

  p_set_status(NOS);

  if (!p_send_blocking) {
    if (p_ids_to_send.length() > 0) {
      p_send_item(p_ids_to_send.takeFirst());
    } else {
      //emit itemsSent();
      kDebug() << "all items sent.";
    }
  }

}

void KGroupDAVProxySyncThread::p_ret_remote_items(const bool blocking) {

  if ((p_port==0) || (p_host.isEmpty()) || (status() != NOS)) return;

  p_set_status(RET_REMOTE_ITEMS);

  QString url;
  if (p_port != 80) {
    url = "http://"+p_host+":"+QString("%1").arg(p_port)+p_path+"/";
  } else {
    url = "http://"+p_host+p_path+"/";
  }
  kDebug() << "RETRIEVING REMOTE ITEMS:" << url;
  p_job = KIO::davPropFind(KUrl(url), p_xml_getetag, "1", KIO::HideProgressInfo);
  if (!blocking) connect(p_job, SIGNAL(result(KJob*)), SLOT(p_finished_ret_remote_items(KJob*)));

  QString header;
  p_job->addMetaData("PropagateHttpHeader", "true");
  p_build_job_header(header);

  p_job->addMetaData("customHTTPHeader", header);

  if (blocking) {
    p_job->exec();
    p_finished_ret_remote_items(p_job);
  } else {
    p_job->start();
  }

}

void KGroupDAVProxySyncThread::p_parse_xml_remote_item_response(const QDomDocument& xml_response, ItemHashByString *items) {

  p_xml_parser.parse(xml_response, items);

}

void KGroupDAVProxySyncThread::p_sync() {

  p_data->lockItems();
  p_data->lockRemoteItems();

  if ((p_data->remoteItems()->size() == 0) && (p_data->items()->count() == 0)) {
    kDebug() << "please update remote items first or just no remote items found.";
    p_data->unlockItems();
    p_data->unlockRemoteItems();
    return;
  }

  //IMPORTANT!! we assume, that remote items has just been updated!

  LocationList locations_to_ret;
  IdList ids_to_send;
  LocationList locations_to_del_remote;
  IdList ids_to_del_local;

  //iterate remote items and mark them to retrieve if needed
  ItemHashByString::const_iterator i = p_data->remoteItems()->constBegin();
  while (i != p_data->remoteItems()->constEnd()) {
    if (p_resolver.locationExists(i.key())) {
      GroupDAVItem *item = p_data->items()->value(p_resolver.idByLocation(i.key()), NULL);
      if (item->isDel()) {
        locations_to_del_remote << i.key();
      }
    } else {
      locations_to_ret << i.key();
    }
    ++i;
  }

  //iterate local items mark them to send or delete
  ItemHash::const_iterator j = p_data->items()->constBegin();
  while (j != p_data->items()->constEnd()) {
    if (!p_data->remoteItems()->contains(j.value()->location())) {
      if (j.value()->isNew()) {
        ids_to_send << j.key();
      } else {
        ids_to_del_local << j.key();
      }
    } else {
      if (p_data->remoteItems()->value(j.value()->location())->etag() != j.value()->etag()) {
        if (j.value()->isMod()) {
          //conflict!
        } else {
          locations_to_ret << j.value()->location();
        }
      } else {
        if (j.value()->isMod()) {
          ids_to_send << j.key();
        }
      }
    }
    ++j;
  }

  p_data->unlockItems();
  p_data->unlockRemoteItems();

  kDebug() << "item locations to retrieve:" << locations_to_ret;
  kDebug() << "item ids to send:" << ids_to_send;
  kDebug() << "item locations to del remote:" << locations_to_del_remote;
  kDebug() << "item ids to del local:" << ids_to_del_local;

  //first we delete local items which can be deleted
  for (int k = 0; k < ids_to_del_local.length(); ++k) {

    quint32 id = ids_to_del_local.value(k);

    emit itemToDelete(id);

    p_resolver.del(id);
    p_data->lockItems();
    delete p_data->items()->value(id);
    p_data->items()->remove(id);
    p_data->unlockItems();

    emit itemDeleted(id);

  }

  //second we delete remote items
  if (locations_to_del_remote.length() > 0) {
    p_del_items(locations_to_del_remote);
  }

  //third we receive new or modified items from server
  if (locations_to_ret.length() > 0) {
    p_ret_items(locations_to_ret);
  }

  //fourth and last we send local new/modified items to the server
  if (ids_to_send.length() > 0) {
    p_send_items(ids_to_send);
  }

  p_data->clearRemoteItems();

}

void KGroupDAVProxySyncThread::p_ret_item(const QString& location, const bool blocking) {

  if (status() != NOS) return;

  if (location.isEmpty()) {
    kDebug() << "empty location.";
    return;
  }

  p_set_status(RET_ITEM);

  kDebug() << "RETRIEVING ITEM:" << location;

  p_job = KIO::storedGet(KUrl(location), KIO::NoReload, KIO::HideProgressInfo);
  if (!blocking) connect(p_job, SIGNAL(result(KJob*)), SLOT(p_finished_ret_item(KJob*)));

  QString header;
  p_job->addMetaData("PropagateHttpHeader", "true");
  p_build_job_header(header);

  p_job->addMetaData("Location", location);

  p_job->addMetaData("customHTTPHeader", header);

  if (blocking) {
    p_ret_blocking = TRUE;
    p_job->exec();
    p_finished_ret_item(p_job);
  } else {
    p_ret_blocking = FALSE;
    p_job->start();
  }

}

void KGroupDAVProxySyncThread::p_ret_items(const QStringList& locations, const bool blocking) {

  if (status() != NOS) return;

  if (locations.length()==0) {
    kDebug() << "empty location list.";
    return;
  }

  kDebug() << "RETRIEVING ITEMS: #" << locations.length();

  p_locations_to_ret.clear();

  if (!blocking) {

    p_locations_to_ret = locations;
    p_ret_item(p_locations_to_ret.takeFirst());

  } else {

    for (int i = 0; i < locations.length(); ++i) {
      p_ret_item(locations.at(i), TRUE);
    }

    //emit p_items_retrieved();
    kDebug() << "all items retrieved.";

  }

}

void KGroupDAVProxySyncThread::p_del_item(const QString& location, const bool blocking) {

  if (status() != NOS) return;

  if (!p_resolver.locationExists(location)) {
    kDebug() << "item not found:" << location;
    return;
  }

  quint32 id = p_resolver.idByLocation(location);
  p_data->lockItems();
  if (!p_data->items()->contains(id)) {
    p_data->unlockItems();
    kDebug() << "item id not found:" << id;
    return;
  }
  p_data->unlockItems();

  p_set_status(DEL_ITEM);

  kDebug() << "DELETE ITEM:" << location;

  p_job = KIO::file_delete(KUrl(location), KIO::HideProgressInfo);
  if (!blocking) connect(p_job, SIGNAL(result(KJob*)), SLOT(p_finished_del_item(KJob*)));

  QString header;
  p_job->addMetaData("PropagateHttpHeader", "true");
  p_build_job_header(header);

  p_data->lockItems();
  header += "\r\nIf-Match: "+p_data->items()->value(id)->etag();
  p_data->unlockItems();

  p_job->addMetaData("customHTTPHeader", header);

  p_data->lockItems();
  p_last_deleted_item = p_data->items()->value(id);
  p_data->unlockItems();

  if (blocking) {
    p_del_blocking = TRUE;
    p_job->exec();
    p_finished_del_item(p_job);
  } else {
    p_del_blocking = FALSE;
    p_job->start();
  }

}

void KGroupDAVProxySyncThread::p_del_items(const QStringList& locations, const bool blocking) {

  if (status() != NOS) return;

  if (locations.length()==0) {
    kDebug() << "empty location list.";
    return;
  }

  kDebug() << "DELETE ITEMS: #" << locations.length();

  p_locations_to_del.clear();

  if (!blocking) {

    p_locations_to_del = locations;
    p_del_item(p_locations_to_del.takeFirst());

  } else {

    for (int i = 0; i < locations.length(); ++i) {
      p_del_item(locations.at(i), TRUE);
    }

    //emit p_items_deleted();
    kDebug() << "all items deleted.";

  }

}

void KGroupDAVProxySyncThread::p_send_item(const quint32 id, const bool blocking) {

  if (status() != NOS) return;

  p_data->lockItems();
  if (!p_data->items()->contains(id)) {
    p_data->unlockItems();
    kDebug() << "item not found:" << id;
    return;
  }
  p_data->unlockItems();

  p_set_status(SEND_ITEM);

  QString url;
  p_data->lockItems();
  if (p_data->items()->value(id)->isNew()) {
    p_data->unlockItems();
    if (p_port != 80) {
      url = "http://"+p_host+":"+QString("%1").arg(p_port)+p_path+"/new.ics";
    } else {
      url = "http://"+p_host+p_path+"/new.ics";
    }
  } else {
    url = p_data->items()->value(id)->location();
    p_data->unlockItems();
  }

  kDebug() << "SEND ITEM:" << id << ", URL:" << url;

  p_data->lockItems();
  QByteArray data = p_data->items()->value(id)->data().toUtf8();
  p_data->unlockItems();
  p_job = KIO::storedPut(data, KUrl(url), -1, KIO::HideProgressInfo);
  if (!blocking) connect(p_job, SIGNAL(result(KJob*)), SLOT(finished_send_item(KJob*)));

  QString header;
  p_job->addMetaData("PropagateHttpHeader", "true");
  p_build_job_header(header);
  header += "\r\nContent-Length: "+QString("%1").arg(data.length());
  header += "\r\nContent-Type: text/calendar; charset=utf-8";

  p_data->lockItems();
  if (p_data->items()->value(id)->isNew()) {
    p_data->unlockItems();
    kDebug() << "new item";
    header += "\r\nIf-None-Match: *";
  } else if (p_data->items()->value(id)->isMod()) {
    p_data->unlockItems();
    kDebug() << "modified item";
    header += "\r\nIf-Match: "+QString("%1").arg(p_data->items()->value(id)->etag());
  } else {
    p_data->unlockItems();
  }

  p_job->addMetaData("customHTTPHeader", header);

  p_data->lockItems();
  p_last_send_item = p_data->items()->value(id);
  p_data->unlockItems();

  if (blocking) {
    p_send_blocking = TRUE;
    p_job->exec();
    p_finished_send_item(p_job);
  } else {
    p_send_blocking = FALSE;
    p_job->start();
  }

}

void KGroupDAVProxySyncThread::p_send_items(const IdList& ids, const bool blocking) {

  if (status() != NOS) return;

  if (ids.length()==0) {
    kDebug() << "empty id list.";
    return;
  }

  kDebug() << "SEND ITEMS: #" << ids.length();

  p_ids_to_send.clear();

  if (!blocking) {

    p_ids_to_send = ids;
    p_send_item(p_ids_to_send.takeFirst());

  } else {

    for (int i = 0; i < ids.length(); ++i) {
      p_send_item(ids.at(i), TRUE);
    }

    //emit p_items_sent();
    kDebug() << "all items sent.";

  }

}

void KGroupDAVProxySyncThread::p_build_job_header(QString& header) {

  if ((!p_username.isEmpty()) && (!p_password.isEmpty())) {
    QString id = QByteArray(p_username.toUtf8()+":"+p_password.toUtf8()).toBase64();
    header += "Authorization: "+QString("Basic %1").arg(id);
  }
  header += "\r\nHost: "+p_host;
  header += "\r\nUser-Agent: KGroupDAV/0.0";

}

KGroupDAVProxy::KGroupDAVProxy(const QString& host, const quint16 port, QObject *parent) : QObject(parent) {

  Q_UNUSED(parent);

  p_host = host;
  p_port = port;

  p_path = PATH;

  p_data = new KGroupDAVProxyData();

  p_sync_thread = new KGroupDAVProxySyncThread(p_data);

  connect(p_sync_thread, SIGNAL(statusChanged(int)), this, SLOT(p_change_status(int)));
  connect(p_sync_thread, SIGNAL(progress(int)), this, SLOT(p_progress(int)));
  connect(p_sync_thread, SIGNAL(finished()), this, SLOT(p_exit_thread()));
  connect(p_sync_thread, SIGNAL(terminated()), this, SLOT(p_exit_thread()));

  connect(p_sync_thread, SIGNAL(itemAdded(const quint32)), this, SLOT(p_item_added(const quint32)));
  connect(p_sync_thread, SIGNAL(itemModed(const quint32)), this, SLOT(p_item_moded(const quint32)));
  connect(p_sync_thread, SIGNAL(itemToDelete(const quint32)), this, SLOT(p_item_to_delete(const quint32)));
  connect(p_sync_thread, SIGNAL(itemDeleted(const quint32)), this, SLOT(p_item_deleted(const quint32)));

  connect(&p_sync_timer, SIGNAL(timeout()), this, SLOT(p_sync()));
  connect(&p_release_sync_on_change_timer, SIGNAL(timeout()), this, SLOT(p_sync()));

  p_release_sync_on_change_timer.setSingleShot(TRUE);

  p_sync_interval = 0;
  p_sync_on_change = FALSE;

  p_sync_for_change = FALSE;

}

KGroupDAVProxy::~KGroupDAVProxy() {

  delete p_sync_thread;

  delete p_data;

}

void KGroupDAVProxy::setSyncInterval(const int interval) {
  if (interval<10) {
    p_sync_timer.stop();
    return;
  }
  if (interval != p_sync_interval) {
    p_sync_timer.start(interval*1000);
    kDebug() << "sync timer started with interval" << interval;
    p_sync_interval = interval;
  }
}

quint32 KGroupDAVProxy::add(const QString& data) {

  //in progress? then we need to enqueue our job
  if (p_sync_thread->syncInProgress()) {

    KGroupDAVProxyJob job(KGroupDAVProxyJob::ADD, 0, data);
    p_job_queue.enqueue(job);

  } else {

    if (p_data->addItem(data)) {
      if (p_sync_on_change) {
        if (p_sync_timer.isActive()) p_sync_timer.stop();
        if (p_release_sync_on_change_timer.isActive()) p_release_sync_on_change_timer.stop();
        p_release_sync_on_change_timer.start(SYNC_DELAY_ON_MODIFICATION);
        p_sync_for_change = TRUE;
      }
    } else {
      return FALSE;
    }

  }

  return TRUE;

}

bool KGroupDAVProxy::del(const quint32 id) {

  //in progress? then we need to enqueue our job
  if (p_sync_thread->syncInProgress()) {

    KGroupDAVProxyJob job(KGroupDAVProxyJob::DEL, id, "");
    p_job_queue.enqueue(job);

  } else {

    if (p_data->delItem(id)) {
      if (p_sync_on_change) {
        if (p_sync_timer.isActive()) p_sync_timer.stop();
        if (p_release_sync_on_change_timer.isActive()) p_release_sync_on_change_timer.stop();
        p_release_sync_on_change_timer.start(SYNC_DELAY_ON_MODIFICATION);
        p_sync_for_change = TRUE;
      }
    } else {
      return FALSE;
    }

  }

  return TRUE;

}

bool KGroupDAVProxy::mod(const quint32 id, const QString& data) {

  //in progress? then we need to enqueue our job
  if (p_sync_thread->syncInProgress()) {

    KGroupDAVProxyJob job(KGroupDAVProxyJob::MOD, id, data);
    p_job_queue.enqueue(job);

  } else {

    if (p_data->modItem(id, data)) {
      if (p_sync_on_change) {
        if (p_sync_timer.isActive()) p_sync_timer.stop();
        if (p_release_sync_on_change_timer.isActive()) p_release_sync_on_change_timer.stop();
        p_release_sync_on_change_timer.start(SYNC_DELAY_ON_MODIFICATION);
        p_sync_for_change = TRUE;
      }
    } else {
      return FALSE;
    }

  }

  return TRUE;

}

void KGroupDAVProxy::sync() {

  if (p_sync_for_change) return;

  p_sync();

}

void KGroupDAVProxy::p_sync() {

  p_sync_thread->setHost(p_host);
  p_sync_thread->setPort(p_port);
  p_sync_thread->setUsername(p_username);
  p_sync_thread->setPassword(p_password);
  p_sync_thread->setPath(p_path);

  p_sync_thread->start();

}

void KGroupDAVProxy::p_exit_thread() {

  if (p_job_queue.size() > 0) {

    while (!p_job_queue.isEmpty()) {
      KGroupDAVProxyJob job = p_job_queue.dequeue();
      switch (job.type()) {
        case KGroupDAVProxyJob::ADD : add(job.data()); break;
        case KGroupDAVProxyJob::MOD : mod(job.id(), job.data()); break;
        case KGroupDAVProxyJob::DEL : del(job.id()); break;
      }
    }

  } else {

    if (p_sync_for_change) {
      if (p_sync_interval>MINIMUM_SYNC_INTERVAL-1) p_sync_timer.start(p_sync_interval*1000);
      p_sync_for_change = FALSE;
    }

  }

  emit syncFinished();

}
