/* KGroupDAVProxy - Implementation of GroupDAV protocol
 * Copyright (C) 2009 Marco Nelles (marco.nelles@credativ.de)
 * <http://www.credativ.de/>
 * <http://opensource.maniatek.com/kgroupdav/>
 *
 * credativ GmbH
 * Hohenzollernstr. 133
 * 41061 Mönchengladbach
 * E-Mail: info@credativ.de
 *
 * RFC 2445 (iCalendar)
 * RFC 2518 (WebDAV)
 * RFC 2616 (HTTP)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef KGROUPDAV_HEADER
#define KGROUPDAV_HEADER

#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/mman.h>


#include <QObject>
#include <QString>
#include <QByteArray>
#include <QThread>
#include <QCryptographicHash>
#include <QTimer>
#include <QMutex>
#include <QQueue>

#include <KDebug>
#include <KIO/Job>
#include <KIO/DavJob>
#include <KIO/DeleteJob>

#define XML_GETETAG "<?xml version=\"1.0\" encoding=\"utf-8\"?><propfind xmlns=\"DAV:\"><prop><getetag/></prop></propfind>"

#define PATH "/groupdav/Calendar"

#define TAG_MULTISTATUS				"multistatus"
#define TAG_RESPONSE				"response"
#define TAG_HREF				"href"
#define TAG_PROPSTAT				"propstat"
#define TAG_PROP				"prop"
#define TAG_GETETAG				"getetag"

#define MINIMUM_SYNC_INTERVAL			10
#define SYNC_DELAY_ON_MODIFICATION		7000

class GroupDAVItem;

//id == GroupDAVItem id
typedef QHash<QString,GroupDAVItem*> ItemHashByString; //location, GroupDAVItem
typedef QHash<quint32,GroupDAVItem*> ItemHash; //id, GroupDAVItem
typedef QList<quint32> IdList;
typedef QStringList LocationList;

class GroupDAVItem {

public:
  GroupDAVItem(const quint32 id = 0) {
    p_id = id;
    p_is_new = FALSE;
    p_is_mod = FALSE;
    p_is_del = FALSE;
    p_etag = -1;
  };
  ~GroupDAVItem() { };

  inline void setEtag(const QString& etag) { p_etag = etag; }
  inline const QString etag() const { return p_etag; }
  inline QString* pEtag() { return &p_etag; }
  inline void setLocation(const QString& location) { p_location = location; }
  inline const QString location() const { return p_location; }
  inline QString* pLocation() { return &p_location; }
  inline quint32 id() const { return p_id; }
  inline void setData(const QString& data) { p_data = data; }
  inline const QString data() const { return p_data; }
  inline QString* pData() { return &p_data; }
  inline void setNew(const bool isNew = TRUE) { p_is_new = isNew; }
  inline bool isNew() const { return p_is_new; }
  inline void setMod(const bool isMod = TRUE) { p_is_mod = isMod; }
  inline bool isMod() const { return p_is_mod; }
  inline void setDel(const bool isDel = TRUE) { p_is_del = isDel; }
  inline bool isDel() const { return p_is_del; }

  inline const QByteArray md5sum() const {
    QCryptographicHash md5(QCryptographicHash::Md5);
    md5.addData(p_data.toUtf8());
    return md5.result();
  }
  inline quint16 checksum() const {
    QByteArray ba = p_data.toUtf8();
    return qChecksum(ba.data(), ba.size());
  }

private:
  QString p_etag;
  QString p_location;
  quint32 p_id;
  QString p_data;
  bool p_is_new; //this is a new item
  bool p_is_mod; //this is a modified item
  bool p_is_del; //this is a deleted item

};

class KGroupDAVProxyDomParser {

public:
  KGroupDAVProxyDomParser();
  ~KGroupDAVProxyDomParser();

  void parse(const QDomDocument& doc, ItemHashByString *items);

  inline const QString& errorMessage() { return p_last_error_message; }

private:
  bool p_parse_response(const QDomElement& element);
  bool p_parse_propstat(const QDomElement& element);
  bool p_parse_prop(const QDomElement& element);

  QDomDocument doc;
  ItemHashByString *items;

  QString href;
  QString etag;

  QString p_last_error_message;

};

//class for resolving id <=> location
typedef QHash<quint32,QString> LocationHash; //id, location
typedef QHash<QString,quint32> IdHash; //location, id

//thread safe
class KGroupDAVProxyResolver {

public:
  KGroupDAVProxyResolver() { }
  ~KGroupDAVProxyResolver() { }

  inline const QString locationById(const quint32 id) {
    p_mutex.lock();
    QString result = p_locations.value(id, QString());
    p_mutex.unlock();
    return result;
  }
  inline quint32 idByLocation(const QString& location) {
    p_mutex.lock();
    quint32 result = p_ids.value(location, 0);
    p_mutex.unlock();
    return result;
  }

  inline bool locationExists(const QString& location) {
    p_mutex.lock();
    bool result = p_ids.contains(location);
    p_mutex.unlock();
    return result;
  }
  inline bool idExists(const quint32 id) {
    p_mutex.lock();
    bool result = p_locations.contains(id);
    p_mutex.unlock();
    return result;
  }

  inline void add(const quint32 id, const QString& location) {
    p_mutex.lock();
    p_ids[location] = id;
    p_locations[id] = location;
    p_mutex.unlock();
  }

  inline void del(const quint32 id) {
    p_mutex.lock();
    QString location = p_locations.value(id);
    p_locations.remove(id);
    p_ids.remove(location);
    p_mutex.unlock();
  }

  inline void del(const QString& location) {
    p_mutex.lock();
    quint32 id = p_ids.value(location);
    p_ids.remove(location);
    p_locations.remove(id);
    p_mutex.unlock();
  }

private:
  LocationHash p_locations;
  IdHash p_ids;
  QMutex p_mutex;

};

//thread-safe
class KGroupDAVProxyData {

public:
  KGroupDAVProxyData();
  ~KGroupDAVProxyData();

  const GroupDAVItem* remoteItem(const QString& location);
  void addRemoteItem(const QString& location, const QString& etag);
  bool delRemoteItem(const QString& location);

  const GroupDAVItem* item(const quint32 id);
  const QString itemData(const quint32 id);
  bool itemIsDeleted(const quint32 id);
  quint32 addItem(const QString& data);
  bool modItem(const quint32 id, const QString& data);
  bool delItem(const quint32 id);

  const QByteArray md5sum(const quint32 id);
  quint16 checksum(const quint32 id);

  const IdList ids();
  int count();

  void clear();
  void clearItems();
  void clearRemoteItems();

  void undelete();
  void purge();

  //internal for performance
  inline ItemHash* items() { return &p_items; }
  inline ItemHashByString* remoteItems() { return &p_remote_items; }
  inline void lockItems() { p_items_mutex.lock(); }
  inline void unlockItems() { p_items_mutex.unlock(); }
  inline void lockRemoteItems() { p_remote_items_mutex.lock(); }
  inline void unlockRemoteItems() { p_remote_items_mutex.unlock(); }

  inline quint32 peak() {
    p_id_peak_mutex.lock();
    quint32 p = p_id_peak++;
    p_id_peak_mutex.unlock();
    return p;
  }

private:

  ItemHashByString p_remote_items;
  QMutex p_remote_items_mutex;
  ItemHash p_items;
  QMutex p_items_mutex;

  quint32 p_id_peak;
  QMutex p_id_peak_mutex;

};

class KGroupDAVProxySyncThread : public QThread {

  Q_OBJECT

public:
  KGroupDAVProxySyncThread(KGroupDAVProxyData *data, QObject *parent = 0);
  ~KGroupDAVProxySyncThread();

  inline void setHost(const QString& host) {
    if (isRunning()) return;
    p_host = host;
  }
  inline void setPort(const quint16 port) {
    if (isRunning()) return;
    p_port = port;
  }
  inline void setUsername(const QString& username) {
    if (isRunning()) return;
    p_username = username;
  }
  inline void setPassword(const QString& password) {
    if (isRunning()) return;
    p_password = password;
  }
  inline void setPath(const QString& path) {
    if (isRunning()) return;
    p_path = path;
    if (p_path.endsWith('/')) p_path.chop(1);
  }

  enum Status {
    NOS = 0,
    RET_REMOTE_ITEMS,
    RET_ITEM,
    DEL_ITEM,
    SEND_ITEM
  };
  inline Status status() {
    p_status_mutex.lock();
    Status status = p_status;
    p_status_mutex.unlock();
    return status;
  }

  inline bool syncInProgress() { return isRunning(); }

signals:
  void statusChanged(int status);
  void retRemoteItemsFinished();
  void progress(int percent);

  //remote
  void itemAdded(const quint32 id);
  void itemModed(const quint32 id);
  void itemToDelete(const quint32 id);
  void itemDeleted(const quint32 id);

protected:
  void run();

private slots:
  void p_initiate_ret_remote_items();
  void p_initiate_sync();

  void p_finished_ret_remote_items(KJob *job);
  void p_finished_ret_item(KJob *job);
  void p_finished_del_item(KJob *job);
  void p_finished_send_item(KJob *job);

private:
  KGroupDAVProxyData *p_data;

  Status p_status;
  QMutex p_status_mutex;
  inline void p_set_status(Status status) {
    p_status_mutex.lock();
    p_status = status;
    p_status_mutex.unlock();
    emit statusChanged(status);
  }

  KGroupDAVProxyResolver p_resolver;

  //no mutex needed, since we can only set these if thread isn't running
  QString p_host;
  quint16 p_port;
  QString p_username;
  QString p_password;
  QString p_path;


  KIO::Job* p_job;


  /**BEGIN: REMOTE ITEM HANDLING*******************/
  KGroupDAVProxyDomParser p_xml_parser;
  QDomDocument p_xml_getetag;
  void p_ret_remote_items(const bool blocking = TRUE);
  void p_parse_xml_remote_item_response(const QDomDocument& xml_response, ItemHashByString *items);
  /**END: REMOTE ITEM HANDLING*********************/

  void p_sync();

  /**BEGIN: ITEM HANDLING*******************/
  void p_ret_item(const QString& location, const bool blocking = TRUE);
  void p_ret_items(const QStringList& locations, const bool blocking = TRUE);
  bool p_ret_blocking;
  LocationList p_locations_to_ret;
  void p_del_item(const QString& location, const bool blocking = TRUE);
  void p_del_items(const QStringList& locations, const bool blocking = TRUE);
  GroupDAVItem *p_last_deleted_item;
  bool p_del_blocking;
  LocationList p_locations_to_del;
  void p_send_item(const quint32 id, const bool blocking = TRUE);
  void p_send_items(const IdList& ids, const bool blocking = TRUE);
  GroupDAVItem *p_last_send_item;
  bool p_send_blocking;
  IdList p_ids_to_send;
  /**END: ITEM HANDLING*********************/


  /**BEGIN: HELPER FUNCTIONS**/
  void p_build_job_header(QString& header);
  /**END: HELPER FUNCTIONS**/

};

class KGroupDAVProxyJob {

public:

  enum Type {
    ADD = 0,
    MOD,
    DEL
  };

  KGroupDAVProxyJob(const Type type, const quint32 id, const QString& data) {
    p_type = type;
    p_id = id;
    p_data = data;
  }
  KGroupDAVProxyJob(const KGroupDAVProxyJob& other) {
    p_type = other.p_type;
    p_id = other.p_id;
    p_data = other.p_data;
  }
  ~KGroupDAVProxyJob() {
  }

  KGroupDAVProxyJob& operator=(const KGroupDAVProxyJob& other) {
    p_type = other.p_type;
    p_id = other.p_id;
    p_data = other.p_data;
    return *this;
  }
  bool operator==(const KGroupDAVProxyJob& other) const {
    if (p_type != other.p_type) return FALSE;
    if (p_id != other.p_id) return FALSE;
    if (p_data != other.p_data) return FALSE;
    return TRUE;
  }
  bool operator!=(const KGroupDAVProxyJob& other) const {
    if (p_type != other.p_type) return TRUE;
    if (p_id != other.p_id) return TRUE;
    if (p_data != other.p_data) return TRUE;
    return FALSE;
  }

  inline Type type() const {
    return p_type;
  }
  inline const QString data() const {
    return p_data;
  }
  inline quint32 id() const {
    return p_id;
  }

private:
  Type p_type;
  quint32 p_id;
  QString p_data;

};

class KGroupDAVProxy : public QObject {

  Q_OBJECT

public:
  KGroupDAVProxy(const QString& host = "", const quint16 port = 0, QObject *parent = 0);
  ~KGroupDAVProxy();

  inline const QString host() const { return p_host; }
  inline void setHost(const QString& host) { p_host = host; }
  inline quint16 port() const { return p_port; }
  inline void setPort(const quint16 port) { p_port = port; }

  inline const QString username() const { return p_username; }
  inline void setUsername(const QString& username) { p_username = username; }
  inline const QString password() const { return p_password; }
  inline void setPassword(const QString& password) { p_password = password; }

  inline const QString path() const { return p_path; }
  inline void setPath(const QString& path) { p_path = path; }

  inline KGroupDAVProxySyncThread::Status status() { return p_sync_thread->status(); }

  inline bool syncInProgress() { return p_sync_thread->syncInProgress(); }

  inline int syncInterval() const { return p_sync_interval; }
  void setSyncInterval(const int interval);

  inline bool syncOnChange() const { return p_sync_on_change; }
  inline void setSyncOnChange(const bool syncOnChange) { p_sync_on_change = syncOnChange; }

  quint32 add(const QString& data);
  bool del(const quint32 id);
  bool mod(const quint32 id, const QString& data);

  inline const QString itemData(const quint32 id) { return p_data->itemData(id); }

  inline const QByteArray md5sum(const quint32 id) { return p_data->md5sum(id); }
  inline quint16 checksum(const quint32 id) { return p_data->checksum(id); }

  inline const IdList ids() { return p_data->ids(); }
  inline int count() { return p_data->count(); }

public slots:
  void sync();

signals:
  void statusChanged(int status);
  void syncFinished();
  void progress(int percent);

  //remote
  void itemAdded(const quint32 id);
  void itemModed(const quint32 id);
  void itemToDelete(const quint32 id);
  void itemDeleted(const quint32 id);

private slots:
  inline void p_change_status(int status) { emit statusChanged(status); }
  inline void p_progress(int percent) { emit progress(percent); }
  void p_exit_thread();

  inline void p_item_added(const quint32 id) { emit itemAdded(id); }
  inline void p_item_moded(const quint32 id) { emit itemModed(id); }
  inline void p_item_to_delete(const quint32 id) { emit itemToDelete(id); }
  inline void p_item_deleted(const quint32 id) { emit itemDeleted(id); }

  void p_sync();

private:
  QString p_host;
  quint16 p_port;
  QString p_username;
  QString p_password;
  QString p_path;

  QTimer p_sync_timer;
  int p_sync_interval;
  bool p_sync_on_change;

  QTimer p_release_sync_on_change_timer;

  KGroupDAVProxyData *p_data;
  QQueue<KGroupDAVProxyJob> p_job_queue;
  bool p_sync_for_change;

  KGroupDAVProxySyncThread *p_sync_thread;

  bool p_wait_for_next_sync;
  QMutex p_wait_for_next_sync_mutex;

};

#endif
