/*
    Copyright 2007 Robert Knight <robertknight@gmail.com> 

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

// Own
#include "deliciousclient.h"

// Qt
#include <QXmlStreamReader>
#include <QtDebug>

// KDE
#include <KIO/Job>
#include <KIO/NetAccess>
#include <KUrl>

using namespace Delicious;

Post::Post()
    : mShared(false)
{
}

void Post::setUrl(const QUrl& url) { mUrl = url; }
QUrl Post::url() const { return mUrl; }
void Post::setDescription(const QString& description) { mDescription = description; }
QString Post::description() const { return mDescription; }
void Post::setTags(const QStringList& tags) { mTags = tags; }
QStringList Post::tags() const { return mTags; }
void Post::setDateAndTime(const QDateTime& date) { mDateTime = date; }
QDateTime Post::dateAndTime() const { return mDateTime; }
void Post::setShared(bool shared) { mShared = shared; }
bool Post::isShared() const { return mShared; }

void Bundle::setName(const QString& name) { mName = name; }
QString Bundle::name() const { return mName; }
void Bundle::setTags(const QStringList& tags) { mTags = tags; }
QStringList Bundle::tags() const { return mTags; }

Client::Client(QObject *parent)
    : QObject(parent)
{

}
void Client::setLogin(const QString& user,const QString& password) 
{ 
    mUserName = user;
    mPassword = password;
}

QDateTime Client::fetchLastUpdateTime() 
{
    const QByteArray buffer = request("v1/posts/update");

    QXmlStreamReader reader(buffer);
    while (!reader.atEnd()) {
        QXmlStreamReader::TokenType token = reader.readNext();
        if ( token == QXmlStreamReader::StartElement && reader.name() == "update" ) {
            QStringRef dateTimeString = reader.attributes().value("time");
            return QDateTime::fromString(dateTimeString.toString(),Qt::ISODate);
        }
    }

    return QDateTime();
}

QList<Client::TagUsage> Client::fetchTags()
{
    QList<TagUsage> result;
    const QByteArray buffer = request("v1/tags/get",ArgList());

    QXmlStreamReader reader(buffer);
    while (!reader.atEnd()) {
        QXmlStreamReader::TokenType token = reader.readNext();

        if ( token == QXmlStreamReader::StartElement && reader.name() == "tag" ) {
            QXmlStreamAttributes attributes = reader.attributes();

            TagUsage usage = { attributes.value("tag").toString() , 
                               attributes.value("count").toString().toInt() };
            result << usage;
        }
    }

    return result;
}

bool Client::renameTag(const QString& old , const QString& newName)
{
    RequestArgument oldArg = { "old" , old };
    RequestArgument newArg = { "new" , newName };
   
    return parseResult(request("v1/tags/rename",ArgList() << oldArg << newArg));
}

bool Client::sendPost(const Post& post, bool replace)
{
    RequestArgument url = { "url" , post.url().toString() };
    RequestArgument description = { "description" , post.description() };
    RequestArgument tags = { "tags" , post.tags().join(" ") };
    RequestArgument date = { "dt" , post.dateAndTime().toString(Qt::ISODate) };
    RequestArgument replaceArg = { "replace" , replace ? "yes" : "no" };
    RequestArgument shared = { "shared" , post.isShared() ? "yes" : "no" };

    const QByteArray buffer = request("v1/posts/add",ArgList() << url << description << tags 
                                                   << date << replaceArg << shared);

    return parseResult(buffer);
}

bool Client::deletePost(const QUrl& url)
{
    RequestArgument urlArg = { "url" , url.toString() };
    return parseResult(request("v1/posts/delete",ArgList() << urlArg));
}

bool Client::addBundle(const Bundle& bundle)
{
    RequestArgument nameArg = { "bundle" , bundle.name() };
    RequestArgument tagsArg = { "tags" , bundle.tags().join(" ") };

    return parseResult(request("v1/tags/bundles/set",ArgList() << nameArg << tagsArg));
}

bool Client::deleteBundle(const QString& bundleName)
{
    RequestArgument nameArg = { "bundle" , bundleName };

    return parseResult(request("v1/tags/bundles/delete",ArgList() << nameArg));
}

QList<Bundle> Client::fetchBundles() 
{
    QXmlStreamReader reader(request("v1/tags/bundles/all",ArgList()));

    QList<Bundle> results;
    while (!reader.atEnd()) {
        QXmlStreamReader::TokenType token = reader.readNext();
        if ( token == QXmlStreamReader::StartElement && reader.name() == "bundle" ) {
            QXmlStreamAttributes attributes = reader.attributes();

            Bundle bundle;
            bundle.setName(attributes.value("name").toString());
            bundle.setTags(attributes.value("tags").toString().
                                split(' ',QString::SkipEmptyParts));

            results << bundle;
        }
    }

    return results;
}

QList<Client::DateUsage> Client::fetchDates(const QString& tag)
{
    RequestArgument tagArg = { "tag" , tag };
    
    QXmlStreamReader reader(request("v1/posts/dates",ArgList() << tagArg));

    QList<DateUsage> results;
    while (!reader.atEnd()) {
        QXmlStreamReader::TokenType token = reader.readNext();
        if ( token == QXmlStreamReader::StartElement && reader.name() == "date" ) {
            QXmlStreamAttributes attributes = reader.attributes();

            DateUsage usage = { QDate::fromString(attributes.value("date").toString(),Qt::ISODate) ,
                                attributes.value("count").toString().toInt() };
            results << usage;
        }
    }


    return results;
}

QList<Post> Client::fetchMatchingPosts(const QString& tag,
                                       const QDateTime& date,
                                       const QUrl& url)
{
    RequestArgument tagArg = { "tag" , tag };
    RequestArgument dateArg = { "dt" , date.toString(Qt::ISODate) };
    RequestArgument urlArg = { "url" , url.toString() };

    return parsePosts(request("v1/posts/get",ArgList() << tagArg << dateArg << urlArg));
}
QList<Post> Client::fetchAllPosts(const QString& tag)
{
    RequestArgument tagArg = { "tag" , tag };

    return parsePosts(request("v1/posts/all",ArgList() << tagArg));
}

QList<Post> Client::fetchRecentPosts(int maximumCount , const QString& tag)
{
    RequestArgument countArg = { "count" , QString::number(maximumCount) };
    RequestArgument tagArg = { "tag" , tag };

    return parsePosts(request("v1/posts/recent",ArgList() << countArg << tagArg));
}

bool Client::parseResult(const QByteArray& xml)
{
    //
    // parse <result> tags which can be in one of two forms:
    //
    // <result code="errorCode"/>
    //
    // or
    //
    // <result>errorCode</result>
    //
    // If errorCode is "ok" or "done" this means the request was successful,
    // so clear errorText() and return true
    // otherwise set the Client's errorText() to errorCode and return false
    //
    QXmlStreamReader reader(xml);
    while (!reader.atEnd()) {
        QXmlStreamReader::TokenType token = reader.readNext();
        if ( token == QXmlStreamReader::StartElement && reader.name() == "result" ) {
            QStringRef code = reader.attributes().value("code");

            if ( code == "ok" || code == "done" ) {
                mErrorText.clear();
                return true;
            } else if ( code.isEmpty() ) {
                QString content = reader.readElementText();
                if ( content == "done" || content == "ok" ) {
                    mErrorText.clear();
                    return true;
                } else {
                    mErrorText = content;
                    return false;
                }
            } else {
                mErrorText = code.toString();
                return false;
            }

        }
    }

    return false;
}

QList<Post> Client::parsePosts(const QByteArray& xml)
{
    QList<Post> result;
    QXmlStreamReader reader(xml);

    while (!reader.atEnd()) {
        QXmlStreamReader::TokenType token = reader.readNext();

        if ( token == QXmlStreamReader::StartElement && reader.name() == "post" ) {
            Post post;
            QXmlStreamAttributes attributes = reader.attributes();

            post.setUrl(attributes.value("href").toString());
            post.setDescription(attributes.value("description").toString());
            post.setDateAndTime(QDateTime::fromString(attributes.value("time").toString(),
                                                      Qt::ISODate));
            post.setTags(attributes.value("tag").toString().split(' ',QString::SkipEmptyParts));

            result << post;
        }
    }

    return result;
}

QString Client::errorText() const
{
    return mErrorText;
}

QByteArray Client::request(const QString& path , const ArgList& args)
{
    QString pathWithArgs = path + '?';

    // format arguments
    foreach( const RequestArgument& arg , args ) {
        if ( !arg.value.isNull() ) {
           pathWithArgs += arg.name + '=' + arg.value + '&'; 
        }
    }

    //qDebug() << "Sending request" << pathWithArgs;

    // send request
    QByteArray buffer;
    KUrl url(QString("https://%1:%2@api.del.icio.us/%3").arg(mUserName)
                                                        .arg(mPassword)
                                                        .arg(pathWithArgs));
    KIO::Job *job = KIO::get(url,KIO::Reload,KIO::HideProgressInfo);

    if ( KIO::NetAccess::synchronousRun(job,0,&buffer) )
        return buffer;
    else
        return QByteArray();
}

