/*
    Copyright 2007 Robert Knight <robertknight@gmail.com> 

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef DELICIOUSCLIENT_H
#define DELICIOUSCLIENT_H

// Qt
#include <QUrl>
#include <QBuffer>
#include <QByteArray>
#include <QDateTime>
#include <QStringList>
//#include <QHttp>
#include <QObject>

//#include <QSslError>
//#include <QAuthenticator>


namespace Delicious
{

/** A del.icio.us post. */
class Post
{
public:
    Post();

    void setUrl(const QUrl& url);
    QUrl url() const;

    void setDescription(const QString& description);
    QString description() const;

    void setTags(const QStringList& tags);
    QStringList tags() const;

    void setDateAndTime(const QDateTime& date);
    QDateTime dateAndTime() const;

    /** Specifies whether to make this post private or not. */
    void setShared(bool shared);
    /** Returns true if the post is public or false if it is private. */
    bool isShared() const;

private:
    QUrl mUrl;
    QString mDescription;
    QStringList mTags;
    QDateTime mDateTime;
    bool mShared;
};

/** A del.icio.us bundle. */
class Bundle
{
public:
    void setName(const QString& name);
    QString name() const;

    void setTags(const QStringList& tags);
    QStringList tags() const;

private:
    QString mName;
    QStringList mTags;
};

/** 
 * Client to connect to the del.icio.us and fetch
 * information from and send new content to it.
 *
 * Client implements the del.icio.us API described here:
 * http://del.icio.us/help/api/
 *
 * 
 */
class Client : public QObject
{
Q_OBJECT

public:
   /** Construct a new client. */
   Client(QObject *parent = 0);

   /** Set the username and password used to connect to del.icio.us */
   void setLogin(const QString& userName , const QString& password);

   /** 
    * Queries the time when the last update to the users'
    * del.icio.us data was changed.
    */
   QDateTime fetchLastUpdateTime();

   struct TagUsage
   {
        /** Name of the tag */
        QString tagName;
        /** Number of posts with the tag */
        int usageCount;
   };

   /** 
    * Returns a list of tags and the number of posts that have
    * each tag.
    */
   QList<TagUsage> fetchTags();

   /** Renames an existing tag. */
   bool renameTag(const QString& oldName , const QString& newName);

   /** 
    * Fetches posts which match
    * particular criteria.
    *
    * @param tag Look for posts tagged with @p tag
    * @param date Look for posts created on @p date
    * @param url Look for posts which link to @p url
    *
    * Returns a list of posts found if successful or a null
    * list otherwise.
    */
   QList<Post> fetchMatchingPosts(const QString& tag = QString(),
                                  const QDateTime& date = QDateTime(),
                                  const QUrl& url = QUrl());
   /**
    * Fetches up to @p maximumCount of the
    * most recent posts, optionally restricted to only those posts
    * tagged with @p tag.
    */
   QList<Post> fetchRecentPosts(int maximumCount , const QString& tag = QString());
   /** 
    * Fetches all posts, optionally restricted
    * to only those posts tagged with @p tag.
    */
   QList<Post> fetchAllPosts(const QString& tag = QString());

   struct DateUsage
   {
        /** Date when the posts were created. */
        QDate date;
        /** Number of posts created on that date. */
        int count;
   };

   /**
    * Fetches a list of dates and the number of posts created
    * on each date.
    */
   QList<DateUsage> fetchDates(const QString& tag = QString());

   /**
    * Adds a new post to the user's del.icio.us service.
    * Returns true if the new post was added successfully.
    *
    * @param post The post to create
    * @param replace Specifies whether to replace existing posts
    * with the same URL (if any)
    */
   bool sendPost(const Post& post , bool replace=false);

   /**
    * Deletes a post matching the specified @p url
    *
    * Returns true if a post was deleted successfully
    */
   bool deletePost(const QUrl& url);

   /** Fetches all of the user's del.icio.us bundles */
   QList<Bundle> fetchBundles();
   /** 
    * Creates a new del.icio.us bundle, replacing
    * any existing bundle with the same name.
    *
    * Returns true if the new bundle was created.
    */
   bool addBundle(const Bundle& bundle);

   /** 
    * Deletes the bundle with the specified name.
    * Returns true if the bundle was deleted.
    */
   bool deleteBundle(const QString& bundleName);

   /** Returns the error message from the last query. */
   QString errorText() const;

protected:

   /** An argument to a web service request. */
   struct RequestArgument
   {
      /** The name of the argument, eg. "tag" */
      QString name;
      /** The argument value formatted as a string. */
      QString value;
   };
   typedef QList<RequestArgument> ArgList;

   /** 
    * Sends a request to the del.icio.us web service
    * @param path Absolute path on the web service, eg "v1/posts/update"
    * @param args List of argument names and values for the request
    */
   virtual QByteArray request(const QString& path , const ArgList& args = ArgList());

private:
   
   QList<Post> parsePosts(const QByteArray& xml);
   bool parseResult(const QByteArray& xml);

   QString mUserName;
   QString mPassword;
   QString mErrorText;
};

}

#endif // DELICIOUSCLIENT_H


