/*
    Copyright 2007 Robert Knight <robertknight@gmail.com>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

// Own
#include "deliciousresource.h"

// KDE
#include <KBookmark>

// Akonadi
#include <akonadi/itemcreatejob.h>

#include <QDebug>

using namespace Delicious;

Resource::Resource(const QString& id)
    : ResourceBase(id)
{
    // FIXME testing
    mClient.setLogin("YOUR-USER-NAME","YOUR-PASSWORD");
}

Resource::~Resource()
{
}

bool Resource::retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
    Q_UNUSED(parts);

    const QString remoteId = item.remoteId();
    QList<Post> posts = mClient.fetchMatchingPosts(QString(),QDateTime(),remoteId);

    if ( posts.isEmpty() ) {
        error( QString("Bookmark with URL %1 not found on del.icio.us").arg(remoteId) );
        return false;
    }
    if ( posts.count() > 1 ) {
        qDebug() << "More than one post matches the requested url" << remoteId;
    }

    const Post post = posts.first();

    Akonadi::Item i(item);
    i.setMimeType("application/x-xbel");
    i.setPayload<KBookmark>(KBookmark::standaloneBookmark(post.description(),
                                                             KUrl(post.url()),
                                                             QString()));
    itemRetrieved(i);
    return true;
}

void Resource::configure( WId windowId )
{

}

void Resource::aboutToQuit()
{
}

void Resource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
{
    if ( item.mimeType() != "application/x-xbel" ) {
        changeProcessed();
        return;
    }

    KBookmark bookmark = item.payload<KBookmark>();

    Post newPost;
    newPost.setDescription(bookmark.fullText());
    newPost.setUrl(bookmark.url());
    newPost.setDateAndTime(QDateTime::currentDateTime());

    mClient.sendPost(newPost);

    Akonadi::Item i( item );
    i.setRemoteId( newPost.url().toString() );
    changeCommitted( i );
}


void Resource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
    //TODO Problem: Bookmark URL is currently used as the remote id
    // this will break if the URL is changed
    KBookmark bookmark = item.payload<KBookmark>();

    Post newPost;
    newPost.setDescription(bookmark.fullText());
    newPost.setUrl(bookmark.url());
    newPost.setDateAndTime(QDateTime::currentDateTime());

    mClient.sendPost(newPost,true /* replace existing post with same URL */);

    Akonadi::Item i( item );
    i.setRemoteId( newPost.url().toString() );
    changeCommitted( i );
}

void Resource::itemRemoved( const Akonadi::Item &item )
{
    mClient.deletePost(QUrl(item.remoteId()));
    changeProcessed();
}


void Resource::retrieveCollections()
{
    Akonadi::Collection root;
    root.setParent( Akonadi::Collection::root() );
    root.setRemoteId( "delicious-root" ); //settings()->value("General/Path").toString() );
    root.setName( name() );

    QStringList mimeTypes;
    mimeTypes << "application/x-xbel" << Akonadi::Collection::mimeType();
    root.setContentMimeTypes( mimeTypes );

    Akonadi::Collection::List list;
    list << root;

    collectionsRetrieved(list);
}

void Resource::retrieveItems( const Akonadi::Collection &collection )
{
    qDebug() << "Akonadi del.icio.us synchronosing resource";

    if ( !collection.isValid() ) {
        qDebug() << "Collection not valid";
    }

    emit percent(0);

    QList<Post> posts = mClient.fetchRecentPosts(100);

    qDebug() << "Found" << posts.count() << "posts";

    foreach( const Post& post , posts ) {
        Akonadi::Item item;
        item.setRemoteId( post.url().toString() );
        item.setMimeType("application/x-xbel");

        KBookmark bookmark = KBookmark::standaloneBookmark(post.description(),
                                                           KUrl(post.url()),
                                                           QString());

        item.setPayload<KBookmark>(bookmark);
        Akonadi::ItemCreateJob *job = new Akonadi::ItemCreateJob(item,collection,this);

        if (!job->exec()) {
            qDebug() << "Error occurred while appending bookmark" << post.url().toString()
                     << "to storage:" << job->errorString();

        }
    }

    emit percent(100);
    itemsRetrievalDone();
}

AKONADI_RESOURCE_MAIN( Resource )

