/*
    KickPim's modular QListViewItems
    Based on kopetelistviewitem.cpp - Kopete's modular QListViewItems
    
    Copyright (c) 2004      by Richard Smith          <kde@metafoo.co.uk>
    
    Ported by Duncan Mac-Vicar P. <duncan@kde.org>
    
    Kopete    (c) 2002-2004 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "kickpimlistviewitem.h"

#include <kdebug.h>
#include <kiconloader.h>
#include <kstringhandler.h>

#include <qapplication.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qptrlist.h>
#include <qrect.h>
#include <qtimer.h>
#include <qheader.h>
#include <qstyle.h>

#ifdef HAVE_XRENDER
#  include <X11/Xlib.h>
#  include <X11/extensions/Xrender.h>
#endif

#include <limits.h>

// KickPimListViewComponentBase --------

class KickPimListViewComponentBase::Private
{
public:
	QPtrList<KickPimListViewComponent> components;
};

KickPimListViewComponentBase::KickPimListViewComponentBase()
 : d( new Private )
{
}

KickPimListViewComponentBase::~KickPimListViewComponentBase()
{
	d->components.setAutoDelete( true );
	delete d;
}

uint KickPimListViewComponentBase::components() { return d->components.count(); }
KickPimListViewComponent *KickPimListViewComponentBase::component( uint n ) { return d->components.at( n ); }

KickPimListViewComponent *KickPimListViewComponentBase::componentAt( const QPoint &pt )
{
	for ( uint n = 0; n < components(); ++n )
	{
		if ( component( n )->rect().contains( pt ) )
		{
			if ( KickPimListViewComponent *comp = component( n )->componentAt( pt ) )
				return comp;
			return component( n );
		}
	}
	return 0;
}

void KickPimListViewComponentBase::componentAdded( KickPimListViewComponent *component )
{
	d->components.append( component );
}

void KickPimListViewComponentBase::componentRemoved( KickPimListViewComponent *component )
{
	//TODO: make sure the component is in d->components once and only once.
	// if not, the situation is best referred to as 'very very broken indeed'.
	d->components.remove( component );
}

void KickPimListViewComponentBase::componentResized( KickPimListViewComponent * )
{
}

std::pair<QString,QRect> KickPimListViewComponentBase::toolTip( const QPoint &relativePos )
{
	for ( uint n = 0; n < components(); ++n )
		if ( component( n )->rect().contains( relativePos ) )
			return component( n )->toolTip( relativePos );

	return std::make_pair( QString::null, QRect() );
}

void KickPimListViewComponentBase::updateAnimationPosition( int p, int s )
{
	for ( uint n = 0; n < components(); ++n )
	{
		KickPimListViewComponent *comp = component( n );
		QRect start = comp->startRect();
		QRect target = comp->targetRect();
		QRect rc( start.left() + ((target.left() - start.left()) * p) / s,
		          start.top() + ((target.top() - start.top()) * p) / s,
		          start.width() + ((target.width() - start.width()) * p) / s,
		          start.height() + ((target.height() - start.height()) * p) / s );
		comp->setRect( rc );
		comp->updateAnimationPosition( p, s );
	}
}

// KickPimListViewComponent --------

class KickPimListViewComponent::Private
{
public:
	Private( KickPimListViewComponentBase *parent )
	 : parent( parent ), minWidth( 0 ), minHeight( 0 )
	 , growHoriz( false ), growVert( false )
	 , tipSource( 0 )
	{
	}
	KickPimListViewComponentBase *parent;
	QRect rect;
	QRect startRect, targetRect;
	int minWidth, minHeight;
	bool growHoriz, growVert;
	KickPimListViewToolTipSource *tipSource;
};

KickPimListViewComponent::KickPimListViewComponent( KickPimListViewComponentBase *parent )
 : d( new Private( parent ) )
{
	d->parent->componentAdded( this );
}

KickPimListViewComponent::~KickPimListViewComponent()
{
	d->parent->componentRemoved( this );
	delete d;
}

void KickPimListViewComponent::setToolTipSource( KickPimListViewToolTipSource *source )
{
	d->tipSource = source;
}

std::pair<QString,QRect> KickPimListViewComponent::toolTip( const QPoint &relativePos )
{
	if ( !d->tipSource )
		return KickPimListViewComponentBase::toolTip( relativePos );

	QRect rc = rect();
	QString result = (*d->tipSource)( this, relativePos, rc );
	return std::make_pair(result, rc);
}

QRect KickPimListViewComponent::rect() { return d->rect; }
QRect KickPimListViewComponent::startRect() { return d->startRect; }
QRect KickPimListViewComponent::targetRect() { return d->targetRect; }

int KickPimListViewComponent::minWidth() { return d->minWidth; }
int KickPimListViewComponent::minHeight() { return d->minHeight; }
int KickPimListViewComponent::widthForHeight( int ) { return minWidth(); }
int KickPimListViewComponent::heightForWidth( int ) { return minHeight(); }

bool KickPimListViewComponent::setMinWidth( int width )
{
	if ( d->minWidth == width ) return false;
	d->minWidth = width;

	d->parent->componentResized( this );
	return true;
}
bool KickPimListViewComponent::setMinHeight( int height )
{
	if ( d->minHeight == height ) return false;
	d->minHeight = height;

	d->parent->componentResized( this );
	return true;
}

void KickPimListViewComponent::layout( const QRect &newRect )
{
	if ( rect().isNull() )
		d->startRect = QRect( newRect.topLeft(), newRect.topLeft() );
	else
		d->startRect = rect();
	d->targetRect = newRect;
	//kdDebug(14000) << "At " << rect << endl;
}

void KickPimListViewComponent::setRect( const QRect &rect )
{
	d->rect = rect;
}

void KickPimListViewComponent::paint( QPainter *painter, const QColorGroup &cg )
{
	/*painter->setPen( Qt::red );
	painter->drawRect( rect() );*/
	for ( uint n = 0; n < components(); ++n )
		component( n )->paint( painter, cg );
}

void KickPimListViewComponent::repaint()
{
	d->parent->repaint();
}

void KickPimListViewComponent::relayout()
{
	d->parent->relayout();
}

void KickPimListViewComponent::componentAdded( KickPimListViewComponent *component )
{
	KickPimListViewComponentBase::componentAdded( component );
	//update( Relayout );
}

void KickPimListViewComponent::componentRemoved( KickPimListViewComponent *component )
{
	KickPimListViewComponentBase::componentRemoved( component );
	//update( Relayout );
}

// KickPimListViewBoxComponent --------

class KickPimListViewBoxComponent::Private
{
public:
	Private( KickPimListViewBoxComponent::Direction dir ) : direction( dir ) {}
	KickPimListViewBoxComponent::Direction direction;

	static const int padding = 2;
};

KickPimListViewBoxComponent::KickPimListViewBoxComponent( KickPimListViewComponentBase *parent, Direction dir )
 : KickPimListViewComponent( parent ), d( new Private( dir ) )
{
}
KickPimListViewBoxComponent::~KickPimListViewBoxComponent()
{
	delete d;
}

int KickPimListViewBoxComponent::widthForHeight( int height )
{
	if ( d->direction != Horizontal )
	{
		int width = 0;
		for ( uint n = 0; n < components(); ++n )
			width = QMAX( width, component( n )->widthForHeight( height ) );
		return width;
	}
	else
	{
		int width = (components() - 1) * Private::padding;
		for ( uint n = 0; n < components(); ++n )
			width += component( n )->widthForHeight( height );
		return width;
	}
}

int KickPimListViewBoxComponent::heightForWidth( int width )
{
	if ( d->direction == Horizontal )
	{
		int height = 0;
		for ( uint n = 0; n < components(); ++n )
			height = QMAX( height, component( n )->heightForWidth( width ) );
		return height;
	}
	else
	{
		int height = (components() - 1) * Private::padding;
		for ( uint n = 0; n < components(); ++n )
			height += component( n )->heightForWidth( width );
		return height;
	}
}

void KickPimListViewBoxComponent::calcMinSize()
{
	int sum = (components() - 1) * Private::padding, max = 0;
	for ( uint n = 0; n < components(); ++n )
	{
		KickPimListViewComponent *comp = component( n );
		if ( d->direction == Horizontal )
		{
			max = QMAX( max, comp->minHeight() );
			sum += comp->minWidth();
		}
		else
		{
			max = QMAX( max, comp->minWidth() );
			sum += comp->minHeight();
		}
	}

	bool sizeChanged = false;
	if ( d->direction == Horizontal )
	{
		if ( setMinWidth( sum ) ) sizeChanged = true;
		if ( setMinHeight( max ) ) sizeChanged = true;
	}
	else
	{
		if ( setMinWidth( max ) ) sizeChanged = true;
		if ( setMinHeight( sum ) ) sizeChanged = true;
	}

	if ( sizeChanged )
		repaint();
	else
		relayout();
}

void KickPimListViewBoxComponent::layout( const QRect &rect )
{
	KickPimListViewComponent::layout( rect );

	bool horiz = (d->direction == Horizontal);
	int fixedSize = 0;
	for ( uint n = 0; n < components(); ++n )
	{
		KickPimListViewComponent *comp = component( n );
		if ( horiz )
			fixedSize += comp->minWidth();
		else
			fixedSize += comp->minHeight();
	}

	// remaining space after all fixed items have been allocated
	int padding = Private::padding;

	// ensure total is at least minXXX. the only time the rect
	// will be smaller than that is when we don't fit, and in
	// that cases we should pretend that we're wide/high enough.
	int total;
	if ( horiz )
		total = QMAX( rect.width(), minWidth() );
	else
		total = QMAX( rect.height(), minHeight() );

	int remaining = total - fixedSize - padding * (components() - 1);

	// finally, lay everything out
	int pos = 0;
	for ( uint n = 0; n < components(); ++n )
	{
		KickPimListViewComponent *comp = component( n );
		QRect rc;
		if ( horiz )
		{
			rc.setLeft( rect.left() + pos );
			rc.setTop( rect.top() );
			rc.setHeight( rect.height() );
			int minWidth = comp->minWidth();
			int desiredWidth = comp->widthForHeight( rect.height() );
			rc.setWidth( QMIN( remaining + minWidth, desiredWidth ) );
			pos += rc.width();
			remaining -= rc.width() - minWidth;
		}
		else
		{
			rc.setLeft( rect.left() );
			rc.setTop( rect.top() + pos );
			rc.setWidth( rect.width() );
			int minHeight = comp->minHeight();
			int desiredHeight = comp->heightForWidth( rect.width() );
			rc.setHeight( QMIN( remaining + minHeight, desiredHeight ) );
			pos += rc.height();
			remaining -= rc.height() - minHeight;
		}
		comp->layout( rc & rect );
		pos += padding;
	}
}

void KickPimListViewBoxComponent::componentAdded( KickPimListViewComponent *component )
{
	KickPimListViewComponent::componentAdded( component );
	calcMinSize();
}

void KickPimListViewBoxComponent::componentRemoved( KickPimListViewComponent *component )
{
	KickPimListViewComponent::componentRemoved( component );
	calcMinSize();
}

void KickPimListViewBoxComponent::componentResized( KickPimListViewComponent *component )
{
	KickPimListViewComponent::componentResized( component );
	calcMinSize();
}

// KickPimListViewImageComponent --------

class KickPimListViewImageComponent::Private
{
public:
	QPixmap image;
};

KickPimListViewImageComponent::KickPimListViewImageComponent( KickPimListViewComponentBase *parent )
 : KickPimListViewComponent( parent ), d( new Private )
{
}

KickPimListViewImageComponent::KickPimListViewImageComponent( KickPimListViewComponentBase *parent, int minW, int minH )
 : KickPimListViewComponent( parent ), d( new Private )
{
	setMinWidth( minW );
	setMinHeight( minH );
	repaint();
}

KickPimListViewImageComponent::~KickPimListViewImageComponent()
{
	delete d;
}

void KickPimListViewImageComponent::setPixmap( const QPixmap &img, bool adjustSize)
{
	d->image = img;
	if ( adjustSize )
	{
		setMinWidth( img.width() );
		setMinHeight( img.height() );
	}
	repaint();
}

static QPoint operator+( const QPoint &pt, const QSize &sz )
{
	return QPoint( pt.x() + sz.width(), pt.y() + sz.height() );
}

/*static QPoint operator+( const QSize &sz, const QPoint &pt )
{
	return pt + sz;
}*/

void KickPimListViewImageComponent::paint( QPainter *painter, const QColorGroup & )
{
	QRect ourRc = rect();
	QRect rc = d->image.rect();
	// center rc within our rect
	rc.moveTopLeft( ourRc.topLeft() + (ourRc.size() - rc.size()) / 2 );
	// paint, shrunk to be within our rect
	painter->drawPixmap( rc & ourRc, d->image );
}

// KickPimListViewTextComponent

class KickPimListViewTextComponent::Private
{
public:
	Private() : customColor( false ) {}
	QString text;
	bool customColor;
	QColor color;
	QFont font;
};

KickPimListViewTextComponent::KickPimListViewTextComponent( KickPimListViewComponentBase *parent, const QFont &font, const QString &text )
 : KickPimListViewComponent( parent ), d( new Private )
{
	setFont( font );
	setText( text );
}

KickPimListViewTextComponent::~KickPimListViewTextComponent()
{
	delete d;
}

QString KickPimListViewTextComponent::text()
{
	return d->text;
}

void KickPimListViewTextComponent::setText( const QString &text )
{
	if ( text == d->text ) return;
	d->text = text;
	relayout();
	calcMinSize();
}

QFont KickPimListViewTextComponent::font()
{
	return d->font;
}

void KickPimListViewTextComponent::setFont( const QFont &font )
{
	if ( font == d->font ) return;
	d->font = font;
	calcMinSize();
}

void KickPimListViewTextComponent::calcMinSize()
{
	if ( !d->text.isEmpty() )
	{
		setMinHeight( QFontMetrics( font() ).height() );
		setMinWidth( QFontMetrics( font() ).width(d->text) );
	}
	else
	{
		setMinHeight( 0 );
		setMinWidth( 0 );
	}
	repaint();
}

int KickPimListViewTextComponent::widthForHeight( int )
{
	// add 2 to place an extra gap between the text and things to its right.
	// allegedly if this is not done the protocol icons overlap the text.
	// i however have never seen this problem (which would almost certainly
	// be a bug somewhere else).
	return QFontMetrics( font() ).width( d->text ) + 2;
}

QColor KickPimListViewTextComponent::color()
{
	return d->color;
}

void KickPimListViewTextComponent::setColor( const QColor &color )
{
	d->color = color;
	d->customColor = true;
	repaint();
}

void KickPimListViewTextComponent::setDefaultColor()
{
	d->customColor = false;
	repaint();
}

void KickPimListViewTextComponent::paint( QPainter *painter, const QColorGroup &cg )
{
	if ( d->customColor )
		painter->setPen( d->color );
	else
		painter->setPen( cg.text() );
	QString dispStr = KStringHandler::rPixelSqueeze( d->text, QFontMetrics( font() ), rect().width() );
	painter->setFont( font() );
	painter->drawText( rect(), Qt::SingleLine, dispStr );
}

// KickPimListViewHSpacerComponent --------

KickPimListViewHSpacerComponent::KickPimListViewHSpacerComponent( KickPimListViewComponentBase *parent )
 : KickPimListViewComponent( parent )
{
	setMinWidth( 0 );
	setMinHeight( 0 );
}

int KickPimListViewHSpacerComponent::widthForHeight( int )
{
	return INT_MAX;
}

// KickPimListViewVSpacerComponent --------

KickPimListViewVSpacerComponent::KickPimListViewVSpacerComponent( KickPimListViewComponentBase *parent )
 : KickPimListViewComponent( parent )
{
	setMinWidth( 0 );
	setMinHeight( 0 );
}

int KickPimListViewVSpacerComponent::heightForWidth( int )
{
	return INT_MAX;
}

// Item --------

/**
 * A periodic timer intended to be shared amongst multiple objects. Will run only
 * if an object is attached to it.
 */
class SharedTimer : private QTimer
{
	int period;
	int users;
public:
	SharedTimer( int period ) : period(period), users(0) {}
	void attach( QObject *target, const char *slot )
	{
		connect( this, SIGNAL(timeout()), target, slot );
		if( users++ == 0 )
			start( period );
		//kdDebug(14000) << "SharedTimer::attach: users is now " << users << "\n";
	}
	void detach( QObject *target, const char *slot )
	{
		disconnect( this, SIGNAL(timeout()), target, slot );
		if( --users == 0 )
			stop();
		//kdDebug(14000) << "SharedTimer::detach: users is now " << users << "\n";
	}
};

class SharedTimerRef
{
	SharedTimer &timer;
	QObject * const object;
	const char * const slot;
	bool attached;
public:
	SharedTimerRef( SharedTimer &timer, QObject *obj, const char *slot )
	 : timer(timer), object(obj), slot(slot), attached(false)
	{
	}
	void start()
	{
		if( attached ) return;
		timer.attach( object, slot );
		attached = true;
	}
	void stop()
	{
		if( !attached ) return;
		timer.detach( object, slot );
		attached = false;
	}
	bool isActive()
	{
		return attached;
	}
};

class KickPimListViewItem::Private
{
public:
	Private( KickPimListViewItem *item )
	 : layoutAnimateTimer( theLayoutAnimateTimer(), item, SLOT( slotLayoutAnimateItems() ) )
	 , animateLayout( true ), opacity( 1.0 )
	 , visibilityTimer( theVisibilityTimer(), item, SLOT( slotUpdateVisibility() ) )
	 , visibilityLevel( 0 ), visibilityTarget( false ), searchMatch( true )
	{
	}

	QTimer layoutTimer;

	//QTimer layoutAnimateTimer;
	SharedTimerRef layoutAnimateTimer;
	SharedTimer &theLayoutAnimateTimer()
	{
		static SharedTimer timer( 10 );
		return timer;
	}
	
	bool animateLayout;
	int layoutAnimateSteps;
	static const int layoutAnimateStepsTotal = 10;

	float opacity;

	//QTimer visibilityTimer;
	SharedTimerRef visibilityTimer;
	SharedTimer &theVisibilityTimer()
	{
		static SharedTimer timer( 40 );
		return timer;
	}

	int visibilityLevel;
	bool visibilityTarget;
	static const int visibilityFoldSteps = 7;
#ifdef HAVE_XRENDER
	static const int visibilityFadeSteps = 7;
#else
	static const int visibilityFadeSteps = 0;
#endif
	static const int visibilityStepsTotal = visibilityFoldSteps + visibilityFadeSteps;
	
	bool searchMatch;
		
	static bool animateChanges;
	static bool fadeVisibility;
	static bool foldVisibility;
};

bool KickPimListViewItem::Private::animateChanges = true;
bool KickPimListViewItem::Private::fadeVisibility = true;
bool KickPimListViewItem::Private::foldVisibility = true;

KickPimListViewItem::KickPimListViewItem( QListViewItem *parent, QObject *owner, const char *name )
 : QObject( owner, name ), KListViewItem( parent ), d( new Private(this) )
{
	initLVI();
}

KickPimListViewItem::KickPimListViewItem( QListView *parent, QListViewItem *after, QObject *owner, const char *name )
 : QObject( owner, name ), KListViewItem( parent, after ), d( new Private(this) )
{
	initLVI();
}

KickPimListViewItem::~KickPimListViewItem()
{
	delete d;
}

void KickPimListViewItem::setEffects( bool animation, bool fading, bool folding )
{
	Private::animateChanges = animation;
	Private::fadeVisibility = fading;
	Private::foldVisibility = folding;
}

void KickPimListViewItem::initLVI()
{
	connect( listView()->header(), SIGNAL( sizeChange( int, int, int ) ), SLOT( slotColumnResized() ) );
	connect( &d->layoutTimer, SIGNAL( timeout() ), SLOT( slotLayoutItems() ) );
	//connect( &d->layoutAnimateTimer, SIGNAL( timeout() ), SLOT( slotLayoutAnimateItems() ) );
	//connect( &d->visibilityTimer, SIGNAL( timeout() ), SLOT( slotUpdateVisibility() ) );
	setVisible( false );
	setTargetVisibility( true );
}

void KickPimListViewItem::slotColumnResized()
{
	scheduleLayout();
	// if we've been resized, don't animate the layout
	d->animateLayout = false;
}

void KickPimListViewItem::scheduleLayout()
{
	// perform a delayed layout in order to speed it all up
	if ( ! d->layoutTimer.isActive() )
		d->layoutTimer.start( 30, true );
}

void KickPimListViewItem::slotLayoutItems()
{
	d->layoutTimer.stop();

	for ( uint n = 0; n < components(); ++n )
	{
		int width = listView()->columnWidth(n);
		if ( n == 0 )
		{
			int d = depth() + (listView()->rootIsDecorated() ? 1 : 0);
			width -= d * listView()->treeStepSize();
		}

		int height = component( n )->heightForWidth( width );
		component( n )->layout( QRect( 0, 0, width, height ) );
		//kdDebug(14000) << "KickPimListViewComponent " << n << " is " << width << " x " << height << endl;
	}

	if ( Private::animateChanges && d->animateLayout && !d->visibilityTimer.isActive() )
	{
		d->layoutAnimateTimer.start();
		//if ( !d->layoutAnimateTimer.isActive() )
		//	d->layoutAnimateTimer.start( 10 );
		d->layoutAnimateSteps = 0;
	}
	else
	{
		d->layoutAnimateSteps = Private::layoutAnimateStepsTotal;
		d->animateLayout = true;
	}
	slotLayoutAnimateItems();
}

void KickPimListViewItem::slotLayoutAnimateItems()
{
	if ( ++d->layoutAnimateSteps >= Private::layoutAnimateStepsTotal )
		d->layoutAnimateTimer.stop();

	const int s = Private::layoutAnimateStepsTotal;
	const int p = QMIN( d->layoutAnimateSteps, s );

	updateAnimationPosition( p, s );
	setHeight(0);
	repaint();
}

float KickPimListViewItem::opacity()
{
	return d->opacity;
}

void KickPimListViewItem::setOpacity( float opacity )
{
	if ( d->opacity == opacity ) return;
	d->opacity = opacity;
	repaint();
}

void KickPimListViewItem::setSearchMatch( bool match )
{
	d->searchMatch = match;
	
	if ( !match )
		setVisible( false );
	else
	{
		kdDebug(14000) << " match: " << match << ", vis timer active: " << d->visibilityTimer.isActive()
		               << ", target visibility: " << targetVisibility() << endl;
		if ( d->visibilityTimer.isActive() )
			setVisible( true );
		else
			setVisible( targetVisibility() );
	}
}

bool KickPimListViewItem::targetVisibility()
{
	return d->visibilityTarget;
}

void KickPimListViewItem::setTargetVisibility( bool vis )
{
	if ( d->visibilityTarget == vis )
	{
		// in case we're getting called because our parent was shown and
		// we need to be rehidden
		if ( !d->visibilityTimer.isActive() )
			setVisible( vis && d->searchMatch );
		return;
	}
	d->visibilityTarget = vis;
	d->visibilityTimer.start();
	//d->visibilityTimer.start( 40 );
	if ( targetVisibility() )
		setVisible( d->searchMatch );
	slotUpdateVisibility();
}

void KickPimListViewItem::slotUpdateVisibility()
{
	if ( targetVisibility() )
		++d->visibilityLevel;
	else
		--d->visibilityLevel;

	if ( !Private::foldVisibility && !Private::fadeVisibility )
		d->visibilityLevel = targetVisibility() ? Private::visibilityStepsTotal : 0;
	else if ( !Private::fadeVisibility && d->visibilityLevel >= Private::visibilityFoldSteps )
		d->visibilityLevel = targetVisibility() ? Private::visibilityStepsTotal : Private::visibilityFoldSteps - 1;
	else if ( !Private::foldVisibility && d->visibilityLevel <= Private::visibilityFoldSteps )
		d->visibilityLevel = targetVisibility() ? Private::visibilityFoldSteps + 1 : 0;

	if ( d->visibilityLevel >= Private::visibilityStepsTotal )
	{
		d->visibilityLevel = Private::visibilityStepsTotal;
		d->visibilityTimer.stop();
	}
	else if ( d->visibilityLevel <= 0 )
	{
		d->visibilityLevel = 0;
		d->visibilityTimer.stop();
		setVisible( false );
	}
	setHeight( 0 );
	repaint();
}

void KickPimListViewItem::repaint()
{
	// if we're about to relayout, don't bother painting yet.
	if ( d->layoutTimer.isActive() )
		return;
	listView()->repaintItem( this );
}

void KickPimListViewItem::relayout()
{
	scheduleLayout();
}

void KickPimListViewItem::setup()
{
	KListViewItem::setup();
	slotLayoutItems();
}

void KickPimListViewItem::setHeight( int )
{
	int minHeight = 0;
	for ( uint n = 0; n < components(); ++n )
		minHeight = QMAX( minHeight, component( n )->rect().height() );
	//kdDebug(14000) << "Height is " << minHeight << endl;
	if ( Private::foldVisibility && d->visibilityTimer.isActive() )
	{
		int vis = QMIN( d->visibilityLevel, Private::visibilityFoldSteps );
		minHeight = (minHeight * vis) / Private::visibilityFoldSteps;
	}
	KListViewItem::setHeight( minHeight );
}

int KickPimListViewItem::width( const QFontMetrics &, const QListView *lv, int c ) const
{
	// Qt computes the itemRect from this. we want the whole item to be
	// clickable, so we return the widest we could possibly be.
	return lv->header()->sectionSize( c );
}

void KickPimListViewItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align )
{
	QPixmap back( width, height() );
	QPainter paint( &back );
	//KListViewItem::paintCell( &paint, cg, column, width, align );
	// PASTED FROM KLISTVIEWITEM:
	// set the alternate cell background colour if necessary
	QColorGroup _cg = cg;
	if (isAlternate())
		if (listView()->viewport()->backgroundMode()==Qt::FixedColor)
			_cg.setColor(QColorGroup::Background, static_cast< KListView* >(listView())->alternateBackground());
		else
			_cg.setColor(QColorGroup::Base, static_cast< KListView* >(listView())->alternateBackground());
	// PASTED FROM QLISTVIEWITEM
	{
		QPainter *p = &paint;

		QListView *lv = listView();
		if ( !lv )
			return;
		QFontMetrics fm( p->fontMetrics() );

		// any text we render is done by the KickPimListViewComponents, not by this class, so make sure we've nothing to write
		QString t;

		// removed text truncating code from Qt - we do that differently, further on

		int marg = lv->itemMargin();
		int r = marg;
	//	const QPixmap * icon = pixmap( column );

		const BackgroundMode bgmode = lv->viewport()->backgroundMode();
		const QColorGroup::ColorRole crole = QPalette::backgroundRoleFromMode( bgmode );

		if ( _cg.brush( crole ) != lv->colorGroup().brush( crole ) )
			p->fillRect( 0, 0, width, height(), _cg.brush( crole ) );
		else
		{
			// all copied from QListView::paintEmptyArea

			//lv->paintEmptyArea( p, QRect( 0, 0, width, height() ) );
			QStyleOption opt( lv->sortColumn(), 0 ); // ### hack; in 3.1, add a property in QListView and QHeader
			QStyle::SFlags how = QStyle::Style_Default;
			if ( lv->isEnabled() )
				how |= QStyle::Style_Enabled;

			lv->style().drawComplexControl( QStyle::CC_ListView,
						p, lv, QRect( 0, 0, width, height() ), lv->colorGroup(),
						how, QStyle::SC_ListView, QStyle::SC_None,
						opt );
		}



		if ( isSelected() &&
		(column == 0 || lv->allColumnsShowFocus()) ) {
			p->fillRect( r - marg, 0, width - r + marg, height(),
					_cg.brush( QColorGroup::Highlight ) );
	// removed text pen setting code from Qt
		}

		// removed icon drawing code from Qt

		// draw the tree gubbins
		if ( multiLinesEnabled() && column == 0 && isOpen() && childCount() ) {
			int textheight = fm.size( align, t ).height() + 2 * lv->itemMargin();
			textheight = QMAX( textheight, QApplication::globalStrut().height() );
			if ( textheight % 2 > 0 )
				textheight++;
			if ( textheight < height() ) {
				int w = lv->treeStepSize() / 2;
				lv->style().drawComplexControl( QStyle::CC_ListView, p, lv,
								QRect( 0, textheight, w + 1, height() - textheight + 1 ), _cg,
								lv->isEnabled() ? QStyle::Style_Enabled : QStyle::Style_Default,
								QStyle::SC_ListViewExpand,
								(uint)QStyle::SC_All, QStyleOption( this ) );
			}
		}
	}
	// END OF PASTE
	
	
	//do you see a better way to tell the KickPimListViewTextComponent we are selected ?  - Olivier 2004-09-02
	if ( isSelected() ) 
		_cg.setColor(QColorGroup::Text , _cg.highlightedText() );
	
	if ( KickPimListViewComponent *comp = component( column ) )
		comp->paint( &paint, _cg );
	paint.end();

#ifdef HAVE_XRENDER
	QColor rgb = cg.base();//backgroundColor();
	float opac = 1.0;
	if ( d->visibilityTimer.isActive() && Private::fadeVisibility )
	{
		int vis = QMAX( d->visibilityLevel - Private::visibilityFoldSteps, 0 );
		opac = float(vis) / Private::visibilityFadeSteps;
	}
	opac *= opacity();
	const int alpha = 257 - int(opac * 257);
	if ( alpha != 0 )
	{
		XRenderColor clr = { alpha * rgb.red(), alpha * rgb.green(), alpha * rgb.blue(), alpha * 0xff };
		XRenderFillRectangle( back.x11Display(), PictOpOver, back.x11RenderHandle(),
		                      &clr, 0, 0, width, height() );
	}
#endif

	p->drawPixmap( 0, 0, back );
}

void KickPimListViewItem::componentAdded( KickPimListViewComponent *component )
{
	KickPimListViewComponentBase::componentAdded( component );
	scheduleLayout();
}

void KickPimListViewItem::componentRemoved( KickPimListViewComponent *component )
{
	KickPimListViewComponentBase::componentRemoved( component );
	scheduleLayout();
}

void KickPimListViewItem::componentResized( KickPimListViewComponent *component )
{
	KickPimListViewComponentBase::componentResized( component );
	scheduleLayout();
}

#include "kickpimlistviewitem.moc"

// vim: set noet ts=4 sts=4 sw=4:
