/*
    This file is part of libkdepim.

    Copyright (c) 2004 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <kabc/vcardconverter.h>

#include "addresseediffalgo.h"

using namespace KSync;

AddresseeDiffAlgo::AddresseeDiffAlgo( const KABC::Addressee &leftAddressee,
                                      const KABC::Addressee &rightAddressee )
  : mLeftAddressee( leftAddressee ), mRightAddressee( rightAddressee )
{
}

AddresseeDiffAlgo::AddresseeDiffAlgo( const QString &leftAddressee,
                                      const QString &rightAddressee )
{
  KABC::VCardConverter converter;

  mLeftAddressee = converter.parseVCard( leftAddressee.toUtf8() );
  mRightAddressee = converter.parseVCard( rightAddressee.toUtf8() );
}

void AddresseeDiffAlgo::run()
{
  begin();

  if ( mLeftAddressee.uid() != mRightAddressee.uid() ) {
    conflictField( KABC::Addressee::uidLabel(),
                   mLeftAddressee.uid(), mRightAddressee.uid() );
  }

  if ( mLeftAddressee.name() != mRightAddressee.name() ) {
    conflictField( KABC::Addressee::nameLabel(),
                   mLeftAddressee.name(), mRightAddressee.name() );
  }

  if ( mLeftAddressee.formattedName() != mRightAddressee.formattedName() ) {
    conflictField( KABC::Addressee::formattedNameLabel(),
                   mLeftAddressee.formattedName(),
                   mRightAddressee.formattedName() );
  }

  if ( mLeftAddressee.familyName() != mRightAddressee.familyName() ) {
    conflictField( KABC::Addressee::familyNameLabel(),
                   mLeftAddressee.familyName(), mRightAddressee.familyName() );
  }

  if ( mLeftAddressee.givenName() != mRightAddressee.givenName() ) {
    conflictField( KABC::Addressee::givenNameLabel(),
                   mLeftAddressee.givenName(), mRightAddressee.givenName() );
  }

  if ( mLeftAddressee.additionalName() != mRightAddressee.additionalName() ) {
    conflictField( KABC::Addressee::additionalNameLabel(),
                   mLeftAddressee.additionalName(), mRightAddressee.additionalName() );
  }

  if ( mLeftAddressee.prefix() != mRightAddressee.prefix() ) {
    conflictField( KABC::Addressee::prefixLabel(),
                   mLeftAddressee.prefix(), mRightAddressee.prefix() );
  }

  if ( mLeftAddressee.suffix() != mRightAddressee.suffix() ) {
    conflictField( KABC::Addressee::suffixLabel(),
                   mLeftAddressee.suffix(), mRightAddressee.suffix() );
  }

  if ( mLeftAddressee.nickName() != mRightAddressee.nickName() ) {
    conflictField( KABC::Addressee::nickNameLabel(),
                   mLeftAddressee.nickName(), mRightAddressee.nickName() );
  }

  if ( mLeftAddressee.birthday() != mRightAddressee.birthday() ) {
    conflictField( KABC::Addressee::birthdayLabel(),
                   mLeftAddressee.birthday().toString(), mRightAddressee.birthday().toString() );
  }

  if ( mLeftAddressee.mailer() != mRightAddressee.mailer() ) {
    conflictField( KABC::Addressee::mailerLabel(),
                   mLeftAddressee.mailer(), mRightAddressee.mailer() );
  }

  if ( mLeftAddressee.timeZone() != mRightAddressee.timeZone() ) {
    conflictField( KABC::Addressee::timeZoneLabel(),
                   mLeftAddressee.timeZone().toString(), mRightAddressee.timeZone().toString() );
  }

  if ( mLeftAddressee.geo() != mRightAddressee.geo() ) {
    conflictField( KABC::Addressee::geoLabel(),
                   mLeftAddressee.geo().toString(), mRightAddressee.geo().toString() );
  }

  if ( mLeftAddressee.title() != mRightAddressee.title() ) {
    conflictField( KABC::Addressee::titleLabel(),
                   mLeftAddressee.title(), mRightAddressee.title() );
  }

  if ( mLeftAddressee.role() != mRightAddressee.role() ) {
    conflictField( KABC::Addressee::roleLabel(),
                   mLeftAddressee.role(), mRightAddressee.role() );
  }

  if ( mLeftAddressee.organization() != mRightAddressee.organization() ) {
    conflictField( KABC::Addressee::organizationLabel(),
                   mLeftAddressee.organization(), mRightAddressee.organization() );
  }

  if ( mLeftAddressee.note() != mRightAddressee.note() ) {
    conflictField( KABC::Addressee::noteLabel(),
                   mLeftAddressee.note(), mRightAddressee.note() );
  }

  if ( mLeftAddressee.productId() != mRightAddressee.productId() ) {
    conflictField( KABC::Addressee::productIdLabel(),
                   mLeftAddressee.productId(), mRightAddressee.productId() );
  }

  if ( mLeftAddressee.sortString() != mRightAddressee.sortString() ) {
    conflictField( KABC::Addressee::sortStringLabel(),
                   mLeftAddressee.sortString(), mRightAddressee.sortString() );
  }

  if ( mLeftAddressee.secrecy() != mRightAddressee.secrecy() ) {
    conflictField( KABC::Addressee::secrecyLabel(),
                   mLeftAddressee.secrecy().toString(), mRightAddressee.secrecy().toString() );
  }

  if ( mLeftAddressee.url() != mRightAddressee.url() ) {
    conflictField( KABC::Addressee::urlLabel(),
                   mLeftAddressee.url().prettyUrl(), mRightAddressee.url().prettyUrl() );
  }

  if ( mLeftAddressee.logo() != mRightAddressee.logo() ) {
  }

  if ( mLeftAddressee.photo() != mRightAddressee.photo() ) {
  }

  diffList( "emails", mLeftAddressee.emails(), mRightAddressee.emails() );

  diffList( "Phone Numbers", mLeftAddressee.phoneNumbers(), mRightAddressee.phoneNumbers() );
  diffList( "Addresses", mLeftAddressee.addresses(), mRightAddressee.addresses() );

  end();
}

QString AddresseeDiffAlgo::toString( const KABC::PhoneNumber &number ) const
{
  return number.number();
}

QString AddresseeDiffAlgo::toString( const KABC::Address &addr ) const
{
  return addr.formattedAddress();
}

template <class L>
void AddresseeDiffAlgo::diffList( const QString &id,
                                  const QList<L> &left, const QList<L> &right )
{
  for ( int i = 0; i < left.count(); ++i ) {
    if ( !right.contains( left[ i ] ) ) {
      additionalLeftField( id, toString( left[ i ] ) );
    }
  }

  for ( int i = 0; i < right.count(); ++i ) {
    if ( !left.contains( right[ i ] ) ) {
      additionalRightField( id, toString( right[ i ] ) );
    }
  }
}
