/*
This file is part of KDE.

Copyright (c) 2008 Stephen Kelly <steveire@gmail.com>

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#ifndef CONTENTSTRATEGY_H
#define CONTENTSTRATEGY_H

#include <QObject>

namespace KMime
{
class Content;
}

using namespace KMime;


/**
    @brief A convenience base class for building mime messages

    This class can be subclassed to define a strategy for building part
    of a KMime::Content.

    Subclasses my be chained together to manipulate the content in multiple ways,
    for example, to sign and encrypt content.

    Subclasses should reimplement slotAddContent, and emit childContentsReady when
    the class has finished its processing of the content.

    @code
    ContentStrategy *cs = new GpgSigningStrategy( this, new GpgEncryptionStrategy(this) );

    connect( cs, SIGNAL( childContentsReady( Content* ) ), SLOT( sendMail( Content* ) ) );

    cs->addContent( c1 );

    @endcode

*/
class ContentStrategy : public QObject
{
    Q_OBJECT
public:
    /**
    Constructor
    */
    ContentStrategy ( QObject *parent = 0, ContentStrategy *s = 0 );

    /**
    Destructor
    */
    virtual ~ContentStrategy();

    /**
    Processes content by passing it through all strategies in the chain.
    */
    void processContent ( Content* content );

Q_SIGNALS:
    /**
    Emitted when the content is ready for further processing.
    */
    void contentsReady ( Content *content );

protected Q_SLOTS:
    /**
    Reimplement this to process the content. Emit contentsReady when finished.
    */
    virtual void slotProcessContent ( Content* content ) = 0;


private:
    ContentStrategy *m_successor;

};

#endif
