/***************************************************************************
   Copyright (C) 2007 by Matthias Lechner <matthias@lmme.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "gammuengine.h"

#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtCore/QMutexLocker>
#include <QtCore/QTimer>
#include <KDebug>
#include <kgenericfactory.h>

#include <jobs/fetchaddressbookjob.h>

GammuEngine::GammuEngine( QObject *parent, const QString& deviceName )
 : EngineXP( parent, deviceName )
{
    m_widget = new QWidget( 0 );
    QVBoxLayout* layout = new QVBoxLayout( m_widget );
    m_status = new QTextEdit();
    m_status->setReadOnly( true );
    layout->addWidget( m_status );

    m_widget->resize( 300, 300 );
    m_widget->setWindowTitle( "Gammu engine output" );
    m_widget->show();

    status( "Gammu Engine loaded." );

    m_statusInformationFetched = false;
    m_informationFetched = false;
    m_addressbookFetched = false;
    m_initialized = false;
    
    m_device = new Device( this->deviceName() );
    if (m_device)
      status( "Device loaded." );
    else
      status(" Error loading device");
}

void GammuEngine::status( const QString& statusInformation )
{
    QMutexLocker locker( &m_displayMutex );
    m_status->append( statusInformation );
}

GammuEngine::~GammuEngine()
{
    delete m_widget;
}

QList<QWizardPage*> GammuEngine::pageList() const {
    QList<QWizardPage*> wizardPages;

    QWizardPage* newPage = new QWizardPage;
    newPage->setTitle( QString( "Gammu Engine" ) );
    newPage->setSubTitle( QString( "Phone configuration running under the gammu engine" ) );
    QLabel *goOn = new QLabel( newPage );
    goOn->setWordWrap(1);
    goOn->setText( "The Gammu engine currently loads the default configuration located at ~/.gammurc. \n \
    Please edit this file manually or start the Wammu, the official Gammu GUI, for the initial configuration. \
    \n \n \
    The man page for the configuration of ~/.gammurc: \n http://wammu.eu/docs/man/en/gammurc.5/ \n\n \
    The Gammu phone database: \n http://wammu.eu/phones/" );

    wizardPages.append( newPage );

    return wizardPages;
}

void GammuEngine::connectDevice()
{
    m_device->initPhone();
    status( QString( "Initialized device %1" ).arg( deviceName() ) );
    m_initialized = true;
    emit deviceConnected();
}

void GammuEngine::disconnectDevice()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        emit deviceDisconnected();
        return;
    }
    m_device->terminatePhone();
    m_initialized = false;
    emit deviceDisconnected();
    status( QString( "Shut down succeeded ;-)" ));
}

int GammuEngine::signalStrength() const
{
    if( !m_statusInformationFetched )
        return -1;

    return m_device->signalQuality();
}

int GammuEngine::charge() const
{
    if( !m_statusInformationFetched )
        return -1;

    return m_device->battery();
}

KMobileTools::Status::PowerSupplyType GammuEngine::powerSupplyType() const
{
    if( !m_statusInformationFetched )
        return KMobileTools::Status::Unknown;

    return KMobileTools::Status::ACAdaptor;
}

bool GammuEngine::ringing() const
{
    return m_device->ringing();
}

QString GammuEngine::networkName() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return m_device->networkName();
}

QString GammuEngine::manufacturer() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "Not implemented! ->Fix!";
}

KMobileTools::Information::Manufacturer GammuEngine::manufacturerID() const
{
    if( !m_informationFetched )
        return KMobileTools::Information::Unknown;

    return KMobileTools::Information::Nokia;
}

QString GammuEngine::model() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return m_device->model();
}

QString GammuEngine::imei() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "FIXME!"/*m_device->imei()*/;
}

QString GammuEngine::revision() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return m_device->version();
}

void GammuEngine::fetchStatusInformation()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        return;
    }

    if( !m_statusInformationFetched ) {
        m_statusInformationFetched = true;

        emit signalStrengthChanged( signalStrength() );
        emit chargeChanged( charge() );
        emit chargeTypeChanged( powerSupplyType() );
        emit phoneRinging( ringing() );
    }

    status( "Status information fetched." );
    emit statusInformationFetched();
}

void GammuEngine::fetchInformation()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        return;
    }

    if( !m_informationFetched ) {
        m_informationFetched = true;

        emit networkNameChanged( networkName() );
    }

    status( "Mobile phone information fetched." );
    emit informationFetched();
}

void GammuEngine::populateAddressbook()
{
  
  //FIXME: this is probably a bad use of the fetchadressbookJOB, fix this!
    KMobileTools::AddressbookEntry s;
    m_addressbook =  m_device->phonebook(); //NOT NECESSARY?
    foreach( s, m_device->phonebook() ) {
      emit addresseeAdded(s);
    }

//     // building sim contacts
//     for( int i=1; i<=10; i++ ) {
//         KMobileTools::AddressbookEntry addressee;
//         addressee.setName( QString( "Sim contact %1" ).arg( QString::number( i ) ) );
//         addressee.insertEmail( QString( "dummy@kmobiletools.org" ) );
//         addressee.setMemorySlot( KMobileTools::AddressbookEntry::Sim );
//         addAddressee( addressee );
//     }
// 
//     // building phone contacts
//     for( int i=1; i<=5; i++ ) {
//         KMobileTools::AddressbookEntry addressee;
//         addressee.setName( QString( "Phone contact %1" ).arg( QString::number( i ) ) );
//         addressee.insertEmail( QString( "dummy@kmobiletools.org" ) );
//         addressee.setMemorySlot( KMobileTools::AddressbookEntry::Phone );
//         addAddressee( addressee );
//     }
}

void GammuEngine::fetchAddressbook()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        return;
    }

    // first fetch?
    if( !m_addressbookFetched )
        populateAddressbook();

    
    // contacts to add?
    for( int i=0; i<m_addedAddressees.size(); i++ ) {
        m_addressbook.append( m_addedAddressees.at( i ) );
        emit addresseeAdded( m_addedAddressees.at( i ) );
    }
    m_addedAddressees.clear();

    
    // contacts to remove?
    /* use former KABC object instead of the KMT::Addressbook to keep an abstract level 
      between gammuengine.cpp and device.cpp if we later want to port the engine as akonadi resource*/
    KABC::Addressee::List* removedAddressbook = new KABC::Addressee::List;
    for( int i=0; i<m_addressbook.size(); i++ ) {
        for( int j=0; j<m_removedAddressees.size(); j++ ) {
            if( m_addressbook.at( i ).uid() == m_removedAddressees.at( j ) ) {
	      removedAddressbook->append(m_addressbook.at( i ));
                emit addresseeRemoved( m_addressbook.at( i ) );
                m_addressbook.removeAt( i );
            }
        }
    }
    m_device->removeAddressee(removedAddressbook);
    m_removedAddressees.clear();
    
    emit jobCreated( new FetchAddressbookJob( this ) );
    
    m_addressbookFetched = true;
    status( "Address book fetched." );
    emit addressbookFetched();
}

KMobileTools::AddressbookEntry::MemorySlots GammuEngine::availableMemorySlots() const
{
    if( !m_addressbookFetched )
        return KMobileTools::AddressbookEntry::Unknown;

    return ( KMobileTools::AddressbookEntry::Phone | KMobileTools::AddressbookEntry::Sim );
}

KMobileTools::Addressbook GammuEngine::addressbook() const
{
    if( !m_addressbookFetched )
        return KMobileTools::Addressbook();

    return m_addressbook;
}

void GammuEngine::addAddressee( const KMobileTools::AddressbookEntry& addressee )
{
    QMutexLocker locker( &m_mutex );

    m_addedAddressees.append( addressee );

    status( QString( "Enqueued addressee \"%1\" to be added." ).arg( addressee.name() ) );
}

void GammuEngine::editAddressee( const KMobileTools::AddressbookEntry& oldAddressee,
                                const KMobileTools::AddressbookEntry& newAddressee )
{
    QMutexLocker locker( &m_mutex );

    m_addedAddressees.append( newAddressee );
    m_removedAddressees << oldAddressee.uid();

    status( QString( "Enqueued addressee \"%1\" to be edited." ).arg( newAddressee.name() ) );
}

void GammuEngine::removeAddressee( const KMobileTools::AddressbookEntry& addressee )
{
    QMutexLocker locker( &m_mutex );

    m_removedAddressees << addressee.uid();
    status( QString( "Enqueued addressee \"%1\" to be removed." ).arg( addressee.name() ) );
}

K_EXPORT_PLUGIN( GammuEngineFactory )

GammuEngineFactory::GammuEngineFactory()
 : KPluginFactory("kmobiletools_fake")
{
}

GammuEngineFactory::~GammuEngineFactory()
{
}

QObject *GammuEngineFactory::create(
  const char *iface, QWidget *parentWidget, QObject *parent, const QVariantList &args, const QString &keyword )
{
    Q_UNUSED(iface)
    Q_UNUSED(parentWidget)
    Q_UNUSED(keyword)
    QString deviceName = args.at( 0 ).toString();
    return new GammuEngine( parent, deviceName );
}

#include "gammuengine.moc"
