/*
 *  This file is part of kremail.
 *  Copyright (C) 2008 Thomas Gillespie <tomjamesgillespie@googlemail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "kremail.h"
#include "emailcollectionconfig.h"

#include <QBuffer>
#include <QLabel>
#include <QRegExp>
#include <QWidget>

#include <KApplication>
#include <KAction>
#include <KActionCollection>
#include <KMessageBox>
#include <KStandardAction>

#include <kmime/kmime_message.h>
#include <kmime/kmime_headers.h>
#include <kmimetype.h>

#include <akonadi/itemcreatejob.h>
#include <akonadi/item.h>
#include <akonadi/collection.h>
#include <akonadi/collectioncreatejob.h>

#include <kpimidentities/identity.h>
#include <kpimidentities/identitymanager.h>

#include <boost/shared_ptr.hpp>
typedef boost::shared_ptr<KMime::Message> MessagePtr;

KRemail::KRemail():
    KXmlGuiWindow(),
    mainWidget(new QWidget()),
    destination(None),
    scannerIface(new KSaneIface::KSaneWidget()),
    scannerReady(false)
{
    ui.setupUi(mainWidget);
    connect(ui.butGrabFrom, SIGNAL(pressed()), this, SLOT(butGrabFromPressed()));
    connect(ui.butGrabSubject, SIGNAL(pressed()), this, SLOT(butGrabSubjectPressed()));
    connect(ui.butGrabMessage, SIGNAL(pressed()), this, SLOT(butGrabMessagePressed()));
    connect(ui.butScan, SIGNAL(pressed()), this, SLOT(butScanPressed()));
    this->setCentralWidget(mainWidget);

    //Initialise pages
    QImage blank;
    KRemailPage * page = new KRemailPage(blank);
    pages.insert(0, page);
    pageWidgets.append(ui.pagesWidget->addPage(page, QString("No Pages")));

    setupActions();
 
    //Setup config
    config = KGlobal::config();
}

void KRemail::setupActions()
{
    KAction * newAction = new KAction(this);
    newAction->setText(i18n("New"));
    newAction->setIcon(KIcon("document-new"));
    newAction->setShortcut(Qt::CTRL + Qt::Key_N);
    actionCollection()->addAction("new", newAction);
    connect(newAction, SIGNAL(triggered(bool)), this, SLOT(newDocument()));

    KAction * saveAction = new KAction(this);
    saveAction->setText(i18n("Save"));
    saveAction->setIcon(KIcon("document-save"));
    saveAction->setShortcut(Qt::CTRL + Qt::Key_S);
    actionCollection()->addAction("save", saveAction);
    connect(saveAction, SIGNAL(triggered(bool)), this, SLOT(saveScan()));
    
    KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
    
    setupGUI();
}

KRemail::~KRemail()
{
}

void KRemail::butGrabFromPressed()
{
    destination = From;
}

void KRemail::butGrabSubjectPressed()
{
    destination = Subject;
}

void KRemail::butGrabMessagePressed()
{
    destination = Message;
}

void KRemail::butScanPressed()
{

    if(!scannerReady)
    {
        //Setup scanner
        connect(scannerIface, SIGNAL(imageReady(QByteArray &, int, int, int, int)),
                this, SLOT(imageReady(QByteArray &, int, int, int, int)));
        connect(scannerIface, SIGNAL(scanProgress(int)), ui.progressBar, SLOT(setValue(int)));

        QString device;

        // Check if a scanner has been selected
        KConfigGroup generalGroup(config, "General");
        if(!generalGroup.hasKey("ScannerId"))
        {
            device = scannerIface->selectDevice();
            if(device.isEmpty())
            {
                KMessageBox::error(this, i18n("No scanner found."));
                return;
            }
	    else
            {
                generalGroup.writeEntry("ScannerId", device);
            }
        }
        else
        {
            device = generalGroup.readEntry("ScannerId");
        }
    
        if (scannerIface->openDevice(device) == false)
        {
            // could not open a scanner
            KMessageBox::sorry(this, i18n("Opening the selected scanner (%1) failed. Retrying....", device));
            device = scannerIface->selectDevice();
            if(device.isEmpty())
            {
                KMessageBox::error(this, i18n("No scanner found."));
                return;
            }
	    else
            {
                generalGroup.writeEntry("ScannerId", device);
            }
            if (scannerIface->openDevice(device) == false)
            {
                // could not open a scanner
                KMessageBox::sorry(this, i18n("Opening the selected scanner (%1) failed again. Scanning failed.", device));
            }
        }
        scannerIface->setOptVal("resolution", "150");
        scannerIface->setOptVal("mode", "Lineart");
        scannerReady = true;
    }
    scannerIface->scanFinal();
}

void KRemail::imageReady(QByteArray &data, int w, int h, int bpl, int f)
{
    // Get rid of "no pages"
    if(ui.pagesWidget->currentPage()->name() == QString("No Pages"))
    {
        ui.pagesWidget->removePage(ui.pagesWidget->currentPage());
        pages.empty();
        pageWidgets.empty();
    }

    // copy the image data into img
    QImage img = scannerIface->toQImage(data, w, h, bpl, (KSaneIface::KSaneWidget::ImageFormat)f);
    KRemailPage * page = new KRemailPage(img);
    connect(page, SIGNAL(textReady(QString)), this, SLOT(textRecieved(QString)));
    pages.append(page);
    pageWidgets.append(ui.pagesWidget->addPage(page, QString("Page %1").arg(pages.size()-1)));
}

void KRemail::textRecieved(QString text)
{
    text = text.trimmed();
    text.append(" ");
    switch(destination)
    {
        case None:
            break;
        case From:
            ui.txtFrom->setPlainText(ui.txtFrom->toPlainText().append(text));
            break;
        case Subject:
            ui.txtSubject->setText(ui.txtSubject->text().append(text));
            break;
        case Message:
            ui.txtMessage->setPlainText(ui.txtMessage->toPlainText().append(text));
            break;
    }
}

void KRemail::newDocument()
{
    while(!pages.isEmpty())
    {
        pages.removeLast();
        ui.pagesWidget->removePage(dynamic_cast<KPageWidgetItem * > (pageWidgets.takeLast()));
    }
    KRemailPage * page = new KRemailPage(QImage());
    pages.append(page);
    pageWidgets.append(ui.pagesWidget->addPage(page, QString("No Pages")));
}

void KRemail::saveScan()
{
    KMime::Message * newMessage = new KMime::Message();
    newMessage->contentType()->setMimeType( "multipart/mixed" ); 
    newMessage->contentType()->setBoundary( KMime::multiPartBoundary() ); 

    // Do headers
    KMime::Headers::From * fromHeader = new KMime::Headers::From(newMessage);

    QRegExp emailRegex("/^(.+)@([^\\(\\);:,<>]+\\.[a-zA-Z]{2,4})/");
    emailRegex.indexIn(ui.txtFrom->toPlainText());
    QByteArray email = emailRegex.cap().toUtf8();
    if(email.isEmpty())
    {
        email = QByteArray("none@none.com");
    }


    fromHeader->addAddress(email, ui.txtFrom->toPlainText());
    newMessage->setHeader(fromHeader);

    KMime::Headers::To * toHeader = new KMime::Headers::To(newMessage);
    KPIMIdentities::IdentityManager man;
    KPIMIdentities::Identity id = man.defaultIdentity();
    toHeader->addAddress(id.emailAddr().toUtf8(), id.fullName());
    newMessage->setHeader(toHeader);

    KMime::Headers::Subject * toSubject = new KMime::Headers::Subject(newMessage);
    toSubject->fromUnicodeString(ui.txtSubject->text().toUtf8(), "utf8");
    newMessage->setHeader(toSubject);

    //Do body
    newMessage->contentType()->from7BitString( "text/plain" );
    newMessage->fromUnicodeString(ui.txtMessage->toPlainText());

    //Do attachments
    int i = 1;
    while(!pages.isEmpty())
    {
        KRemailPage * page = pages.takeLast();
        ui.pagesWidget->removePage(dynamic_cast<KPageWidgetItem * > (pageWidgets.takeLast()));

        QImage img = page->image();
        QByteArray ba;
        QBuffer buffer(&ba);
        buffer.open(QIODevice::WriteOnly);
        img.save(&buffer, "JPG");

        // Add this part 
        KMime::Content * c = new KMime::Content(); 
        c->setBody( ba+"\n\n" ); // is the /n/n still a bug?

        // Add a content type / set mime-type 
        KMime::Headers::ContentType * ctt = new KMime::Headers::ContentType( c ); 
        ctt->setMimeType("image/jpg"); 
        ctt->setName( QString("Page %1").arg(i), "" ); 
        c->setHeader( ctt ); 

        // Set the encoding. 
        KMime::Headers::ContentTransferEncoding * cte= new KMime::Headers::ContentTransferEncoding( c ); 
        cte->setEncoding( KMime::Headers::CEbase64 ); 
        cte->setDecoded( false ); 
        c->setHeader( cte ); 
        c->assemble();
        newMessage->addContent( c );

        i++;
    }

    using namespace Akonadi;

    Item item;
    item.setMimeType( "message/rfc822" );
    item.setPayload<MessagePtr>( MessagePtr(newMessage) );

    KConfigGroup generalGroup( config, "General" );

    if(!generalGroup.hasKey("CollectionId"))
    {
        EmailCollectionConfig * dlg = new EmailCollectionConfig(this);
        dlg->exec();
    }

    Collection collection((qint64) generalGroup.readEntry("CollectionId", 0));

    Akonadi::ItemCreateJob *job = new Akonadi::ItemCreateJob( item, collection );

    if ( job->exec() )
    {
        KMessageBox::information(this, i18n("Email item created successfully"));
    }
    else
    {
        KMessageBox::error(this, i18n("Error occurred"));
    }

    ui.txtFrom->clear();
    ui.txtSubject->clear();
    ui.txtMessage->clear();

    newDocument();
}
