/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwidget.h"
#include "importitemsjob.h"
#include "migratefeedscommand.h"

#include <krss/itemjobs.h>
#include <krss/itemlisting.h>
#include <krss/itemlistjob.h>
#include <krss/itemmodel.h>
#include <krss/ui/itemview.h>
#include <krss/feedlist.h>
#include <krss/feedlistmodel.h>
#include <krss/ui/feedlistview.h>
#include <krss/netfeed.h>
#include <krss/resource.h>
#include <krss/resourcemanager.h>
#include <krss/standardactionmanager.h>
#include <krss/tagprovider.h>
#include <krss/importopmljob.h>
#include <krss/treenodevisitor.h>

#include <akonadi/itemfetchscope.h>

#include <KXmlGuiWindow>
#include <KAction>
#include <KActionCollection>
#include <KLineEdit>
#include <KLocale>
#include <KDebug>
#include <KUrl>

#include <QtGui/QFileDialog>
#include <QtGui/QLabel>
#include <QtGui/QMessageBox>
#include <QtGui/QSplitter>
#include <QtGui/QHBoxLayout>
#include <QtGui/QSortFilterProxyModel>
#include <QtWebKit/QWebView>

#include <memory>

#include <cassert>

using namespace boost;
using namespace KRss;
using std::auto_ptr;

namespace {
static const struct {
    const char *name;
    const char *label;
    const char *icon;
    int shortcut;
    const char* slot;
} actionData[] = {
    { "krssreader_import_opml", I18N_NOOP( "&Import..." ), "document-import", 0, SLOT( importOpml() ) },
    { "krssreader_import_akregator_feeds", I18N_NOOP( "&Migrate Akregator Feeds..." ), "document-import", 0, SLOT( importAkregatorFeeds() ) },
    { "krssreader_import_items", I18N_NOOP( "&Import Items from Akregator" ), "document-import", 0, SLOT( importItems() ) }
};

static const int numActionData = sizeof (actionData) / sizeof (*actionData);

static void createAllActions( QWidget* receiver, KActionCollection* actionCollection ) {
    for ( int i = 0; i < numActionData; ++i ) {
        KAction *action = new KAction( receiver );
        if ( actionData[i].label )
            action->setText( i18n( actionData[i].label ) );

        if ( actionData[i].icon )
            action->setIcon( KIcon( QString::fromLatin1( actionData[i].icon ) ) );

        action->setShortcut( actionData[i].shortcut );

        if ( actionData[i].slot )
            receiver->connect( action, SIGNAL(triggered(bool)), receiver, actionData[i].slot );

        actionCollection->addAction( QString::fromLatin1( actionData[i].name ), action );
    }
}

}

void MainWidget::importOpml() {
    const QString fn = QFileDialog::getOpenFileName( this, i18nc("window title", "Import File"), QString(), i18n( "OPML files (*.opml)") );
    if ( fn.isEmpty() )
        return;
    const Resource* const resource = ResourceManager::self()->resource( ResourceManager::self()->identifiers().first() );
    ImportOpmlJob* job = resource->createImportOpmlJob( KUrl::fromPath( fn ) );
    connect( job, SIGNAL(finished(KJob*)), this, SLOT(importOpmlFinished(KJob*)) );
    job->start();
}

void MainWidget::importOpmlFinished( KJob* job ) {
    if ( !job->error() ) {
        QMessageBox::information( this, i18n("Import Successful"), i18n("The feed list was successfully imported.") );
        return;
    }

    QMessageBox::critical( this, i18n("Import Failed"), i18n("Import failed: %1 (%2)", job->errorText(), job->error() ) );
}

void MainWidget::importAkregatorFeeds() {
    Akregator::MigrateFeedsCommand* job = new Akregator::MigrateFeedsCommand( this );
    job->setParentWidget( this );
    job->start();
}

void MainWidget::importItems() {
    NetFeed* nf = static_cast<NetFeed*>( qobject_cast<Feed*>( m_currentFeed ) );

    //TODO assuming for now that the feed is from the first resource
    const Resource* const resource = ResourceManager::self()->resource( ResourceManager::self()->identifiers().first() );
    Akregator::ImportItemsJob* job = new Akregator::ImportItemsJob( resource );
    job->setXmlUrl( nf->xmlUrl() );
    connect( job, SIGNAL(finished(KJob*)), this, SLOT(importItemsFinished(KJob*)) );
    job->start();
}

void MainWidget::importItemsFinished( KJob* job ) {
    if ( !job->error() ) {
        QMessageBox::information( this, i18n("Item Import Successful"), i18n("Feed items from Akregator were successfully imported.") );
        return;
    }

    QMessageBox::critical( this, i18n("Import Failed"), i18n("Item import failed: %1", job->errorText() ) );
}

void MainWidget::feedListRetrieved( KJob* j )
{
    const RetrieveFeedListJob* const job = qobject_cast<const RetrieveFeedListJob*>( j );
    assert( job );

    if ( job->error() != KJob::NoError )
        return;

    kDebug() << "FeedList retrieved";
    m_feedList = job->feedList();
    if ( m_tagProvider ) {
        FeedListModel* const newModel = new FeedListModel( m_feedList, m_tagProvider, this );
        QAbstractItemModel* const oldModel = m_feedListView->model();
        m_feedListView->setModel( newModel );
        m_actionManager->setFeedSelectionModel( m_feedListView->selectionModel() );
        delete oldModel;
    }
}

void MainWidget::tagProviderRetrieved( KJob* j )
{
    const TagProviderRetrieveJob* const job = qobject_cast<const TagProviderRetrieveJob*>( j );
    assert( job );

    if ( job->error() != KJob::NoError ) {
        return;
    }

    kDebug() << "TagProvider retrieved";
    m_tagProvider = job->tagProvider();
    m_actionManager->setTagProvider( m_tagProvider );
    if ( m_feedList ) {
        FeedListModel* const newModel = new FeedListModel( m_feedList, m_tagProvider, this );
        QAbstractItemModel* const oldModel = m_feedListView->model();
        m_feedListView->setModel( newModel );
        m_actionManager->setFeedSelectionModel( m_feedListView->selectionModel() );
        delete oldModel;
    }
}

MainWidget::MainWidget( KXmlGuiWindow *parent )
    : QWidget( parent ), m_actionCollection( parent->actionCollection() )
{
    QHBoxLayout *mainLayout = new QHBoxLayout( this );
    setLayout( mainLayout );

    m_mainSplitter = new QSplitter( Qt::Horizontal, this );
    mainLayout->addWidget( m_mainSplitter );

    m_feedListView = new FeedListView( this );

    m_itemSplitter = new QSplitter( Qt::Vertical, this );

    m_searchEdit = new KLineEdit( this );
    m_searchEdit->setClearButtonShown( true );
    m_itemView = new ItemView( this );
    m_itemView->setSortingEnabled( true );
    m_itemViewer = new QWebView( this );
    m_itemViewer->setHtml( QString() );
    m_itemSplitter->addWidget( m_searchEdit );
    m_itemSplitter->addWidget( m_itemView );
    m_itemSplitter->addWidget( m_itemViewer );

    m_mainSplitter->addWidget( m_feedListView );
    m_mainSplitter->addWidget( m_itemSplitter );

    connect( m_feedListView, SIGNAL( clicked( const boost::shared_ptr<KRss::TreeNode>& ) ),
             this, SLOT( setCurrentNode( const boost::shared_ptr<KRss::TreeNode>& ) ) );
    connect( m_itemView, SIGNAL( clicked( const KRss::Item& ) ),
             this, SLOT( setCurrentItem( const KRss::Item& ) ) );

    m_actionManager = new StandardActionManager( parent->actionCollection(), this );
    m_actionManager->setSubscriptionLabel( "krssreader" );
    m_actionManager->createAllActions();

    createAllActions( this, parent->actionCollection() );

    std::auto_ptr<RetrieveFeedListJob> job( new RetrieveFeedListJob( this ) );
    job->setResourceIdentifiers( ResourceManager::self()->identifiers() );
    connect( job.get(), SIGNAL(finished(KJob*)),
             this, SLOT(feedListRetrieved(KJob*)) );
    job.release()->start();

    std::auto_ptr<TagProviderRetrieveJob> tjob( new TagProviderRetrieveJob( this ) );
    connect( tjob.get(), SIGNAL( result( KJob* ) ), this, SLOT( tagProviderRetrieved( KJob* ) ) );
    tjob.release()->start();
}

MainWidget::~MainWidget()
{
}

void MainWidget::setCurrentNode( const shared_ptr<TreeNode>& node )
{
    if ( node->tier() == TreeNode::FeedTier )
        m_currentFeed = m_feedList->feedById( dynamic_pointer_cast<FeedNode, TreeNode>( node )->feedId() ).get();

    m_currentNode = node;

    const NetFeed * const nf = static_cast<const NetFeed*>( qobject_cast<Feed*>( m_currentFeed ) );
    m_actionCollection->action( QString::fromLatin1( "krssreader_import_items" ) )->setEnabled( nf );

    Akonadi::ItemFetchScope fetchScope;
    fetchScope.fetchPayloadPart( Item::HeadersPart );
    fetchScope.fetchAllAttributes();
    ItemListJob * const job = m_currentNode->createItemListJob( m_feedList );
    job->setFetchScope( fetchScope );
    connect( job, SIGNAL(finished(KJob*)), this, SLOT(itemListingDone(KJob*)) );
    job->start();
}


void MainWidget::itemListingDone( KJob* j ) {
    ItemListJob* const job = qobject_cast<ItemListJob*>( j );
    assert( job );
    if ( job->error() ) {
        //TODO print error
        setCollection( shared_ptr<ItemListing>( new ItemListing( QList<Item>(), Akonadi::ItemFetchScope() ) ) );
        return;
    }

    Akonadi::ItemFetchScope fetchScope;
    fetchScope.fetchPayloadPart( Item::HeadersPart );
    fetchScope.fetchAllAttributes();
    const shared_ptr<ItemListing> coll( new ItemListing( job->items(), fetchScope ) );
    auto_ptr<ConnectToItemListingVisitor> visitor( new ConnectToItemListingVisitor( m_feedList, coll ) );
    m_currentNode->accept( visitor.get() );
    setCollection( coll );
}

void MainWidget::setCollection( const shared_ptr<ItemListing>& coll ) {
    QAbstractItemModel* const oldModel = m_itemView->model();
    ItemModel* newModel = new ItemModel( coll, this );
    QSortFilterProxyModel* proxy = new QSortFilterProxyModel( this );
    proxy->setSortRole( ItemModel::SortRole );
    proxy->setSourceModel( newModel );
    m_itemView->setModel( proxy );
    m_actionManager->setItemSelectionModel( m_itemView->selectionModel() );
    delete oldModel;
}

void MainWidget::setCurrentItem( const Item& item )
{
    ItemFetchJob * const job = new ItemFetchJob();
    job->setItem( item );
    job->fetchScope().fetchFullPayload();
    job->fetchScope().fetchAllAttributes();
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( fullItemFetched( KJob* ) ) );
    job->start();
}

void MainWidget::fullItemFetched( KJob *job )
{
    const ItemFetchJob * const fjob = qobject_cast<const ItemFetchJob*>( job );
    Q_ASSERT( fjob );

    m_itemViewer->setHtml( fjob->item().content() );
}

#include "mainwidget.moc"
