/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "configdialog.h"

#include "settings.h"

#include <kacceleratormanager.h>
#include <kcombobox.h>
#include <kldap/ldapconfigwidget.h>
#include <klineedit.h>
#include <klocale.h>
#include <kwindowsystem.h>

#include <QtGui/QCheckBox>
#include <QtGui/QFormLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QScrollArea>
#include <QtGui/QVBoxLayout>

ConfigDialog::ConfigDialog( WId windowId )
{
  if ( windowId )
    KWindowSystem::setMainWindow( this, windowId );

  setButtons( Ok | Cancel );

  setCaption( i18n( "LDAP Configuration" ) );

  QWidget *wdg = new QWidget( this );
  setMainWidget( wdg );

  QVBoxLayout *layout = new QVBoxLayout( wdg );
  layout->setMargin( 0 );

  mConfigWidget = new KLDAP::LdapConfigWidget( KLDAP::LdapConfigWidget::W_USER |
                                               KLDAP::LdapConfigWidget::W_PASS |
                                               KLDAP::LdapConfigWidget::W_BINDDN |
                                               KLDAP::LdapConfigWidget::W_REALM |
                                               KLDAP::LdapConfigWidget::W_HOST |
                                               KLDAP::LdapConfigWidget::W_PORT |
                                               KLDAP::LdapConfigWidget::W_VER |
                                               KLDAP::LdapConfigWidget::W_DN |
                                               KLDAP::LdapConfigWidget::W_FILTER |
                                               KLDAP::LdapConfigWidget::W_TIMELIMIT |
                                               KLDAP::LdapConfigWidget::W_SIZELIMIT |
                                               KLDAP::LdapConfigWidget::W_SECBOX |
                                               KLDAP::LdapConfigWidget::W_AUTHBOX );

  QHBoxLayout *hbox = new QHBoxLayout;

  mSubTree = new QCheckBox( i18n( "Sub-tree query" ) );
  mEditButton = new QPushButton( i18n( "Edit Attributes..." ) );

  hbox->addWidget( mSubTree, Qt::AlignLeft );
  hbox->addWidget( mEditButton, Qt::AlignRight );

  layout->addWidget( mConfigWidget );
  layout->addLayout( hbox );

  connect( this, SIGNAL( okClicked() ), SLOT( save() ) );
  connect( mEditButton, SIGNAL( clicked() ), SLOT( editAttributes() ) );

  load();

  setInitialSize( QSize( 450, 400 ) );
}

void ConfigDialog::load()
{
  mConfigWidget->setUser( Settings::self()->user() );
  mConfigWidget->setPassword( Settings::self()->password() );
  mConfigWidget->setRealm( Settings::self()->realm() );
  mConfigWidget->setBindDn( Settings::self()->bindDN() );
  mConfigWidget->setHost( Settings::self()->host() );
  mConfigWidget->setPort( Settings::self()->port() );
  mConfigWidget->setVersion( Settings::self()->version() );
  mConfigWidget->setTimeLimit( Settings::self()->timeLimit() );
  mConfigWidget->setSizeLimit( Settings::self()->sizeLimit() );
  mConfigWidget->setDn( KLDAP::LdapDN( Settings::self()->dn() ) );
  mConfigWidget->setFilter( Settings::self()->filter() );

  mConfigWidget->setMech( Settings::self()->mech() );

  if ( Settings::self()->isTLS() )
    mConfigWidget->setSecurity( KLDAP::LdapConfigWidget::TLS );
  else if ( Settings::self()->isSSL() )
    mConfigWidget->setSecurity( KLDAP::LdapConfigWidget::SSL );
  else
    mConfigWidget->setSecurity( KLDAP::LdapConfigWidget::None );

  if ( Settings::self()->isAnonymous() )
    mConfigWidget->setAuth( KLDAP::LdapConfigWidget::Anonymous );
  else if ( Settings::self()->isSASL() )
    mConfigWidget->setAuth( KLDAP::LdapConfigWidget::SASL );
  else
    mConfigWidget->setAuth( KLDAP::LdapConfigWidget::Simple );

  mSubTree->setChecked( Settings::self()->isSubTree() );
  mRdnPrefix = Settings::self()->rdnPrefix();
  mAttributes = ContactAttributes::fromString( Settings::self()->attributes() );
}

void ConfigDialog::save()
{
  Settings::self()->setUser( mConfigWidget->user() );
  Settings::self()->setPassword( mConfigWidget->password() );
  Settings::self()->setRealm( mConfigWidget->realm() );
  Settings::self()->setBindDN( mConfigWidget->bindDn() );
  Settings::self()->setHost( mConfigWidget->host() );
  Settings::self()->setPort( mConfigWidget->port() );
  Settings::self()->setVersion( mConfigWidget->version() );
  Settings::self()->setTimeLimit( mConfigWidget->timeLimit() );
  Settings::self()->setSizeLimit( mConfigWidget->sizeLimit() );
  Settings::self()->setDn( mConfigWidget->dn().toString() );
  Settings::self()->setFilter( mConfigWidget->filter() );

  Settings::self()->setIsAnonymous( mConfigWidget->auth() == KLDAP::LdapConfigWidget::Anonymous );
  Settings::self()->setIsSASL( mConfigWidget->auth() == KLDAP::LdapConfigWidget::SASL );
  Settings::self()->setMech( mConfigWidget->mech() );
  Settings::self()->setIsTLS( mConfigWidget->security() == KLDAP::LdapConfigWidget::TLS );
  Settings::self()->setIsSSL( mConfigWidget->security() == KLDAP::LdapConfigWidget::SSL );
  Settings::self()->setIsSubTree( mSubTree->isChecked() );
  Settings::self()->setRdnPrefix( mRdnPrefix );
  Settings::self()->setAttributes( mAttributes.toString() );

  Settings::self()->writeConfig();
}

void ConfigDialog::editAttributes()
{
  AttributesDialog dlg( mAttributes, mRdnPrefix, this );
  if ( dlg.exec() ) {
    mAttributes = dlg.attributes();
    mRdnPrefix = dlg.rdnPrefix();
  }
}


AttributesDialog::AttributesDialog( const ContactAttributes &attributes, int rdnPrefix, QWidget *parent )
  : KDialog( parent ), mAttributes( attributes )
{
  setCaption( i18n( "Attributes Configuration" ) );
  setButtons( Ok | Cancel );
  setDefaultButton( Ok );

  QWidget *page = new QWidget( this );
  setMainWidget( page );

  QVBoxLayout *layout = new QVBoxLayout( page );

  QHBoxLayout *schemaLayout = new QHBoxLayout;
  QLabel *label = new QLabel( i18n( "LDAP Schema:" ) );
  mSchemaCombo = new KComboBox;
  mSchemaCombo->addItem( i18n( "Internet Organization Person" ) );

  schemaLayout->addWidget( label );
  schemaLayout->addWidget( mSchemaCombo );

  connect( schemaLayout, SIGNAL( activated( int ) ), SLOT( schemaChanged( int ) ) );

  mScrollArea = new QScrollArea;

  QHBoxLayout *searchLayout = new QHBoxLayout;
  label = new QLabel( i18n( "Primary key attribute:" ) );
  mRdnCombo = new KComboBox;
  mRdnCombo->addItem( ContactAttributes::label( ContactAttributes::CommonName ) );
  mRdnCombo->addItem( ContactAttributes::label( ContactAttributes::Uid ) );

  searchLayout->addWidget( label );
  searchLayout->addWidget( mRdnCombo );

  layout->addLayout( schemaLayout );
  layout->addWidget( mScrollArea );
  layout->addLayout( searchLayout );

  mRdnCombo->setCurrentIndex( rdnPrefix );

  updateAttributeFields();

  KAcceleratorManager::manage( this );

  setInitialSize( QSize( 440, 470 ) );
}

AttributesDialog::~AttributesDialog()
{
}

ContactAttributes AttributesDialog::attributes() const
{
  ContactAttributes attributes;

  foreach ( const KLineEdit *lineEdit, mLineEdits ) {
    if ( !lineEdit->text().isEmpty() )
      attributes.setName( (ContactAttributes::Type)lineEdit->property( "attributeType" ).toInt(), lineEdit->text() );
  }

  return attributes;
}

int AttributesDialog::rdnPrefix() const
{
  return mRdnCombo->currentIndex();
}

void AttributesDialog::schemaChanged( int index )
{
  switch ( index ) {
    case 0: mAttributes = ContactAttributes::defaultAttributes(); break;
    default: mAttributes = ContactAttributes::defaultAttributes(); break;
  }

  updateAttributeFields();
}

void AttributesDialog::updateAttributeFields()
{
  mLineEdits.clear();
  delete mScrollArea->widget();

  QWidget *widget = new QWidget( mScrollArea );

  QFormLayout *layout = new QFormLayout( widget );

  const ContactAttributes::TypeList types = mAttributes.types();
  foreach ( const ContactAttributes::Type &type, types ) {
    KLineEdit *lineEdit = new KLineEdit;
    lineEdit->setProperty( "attributeType", (int)type );
    mLineEdits.append( lineEdit );

    layout->addRow( ContactAttributes::label( type ) + QLatin1Char( ':' ), lineEdit );
  }

  mScrollArea->setWidget( widget );
  widget->show();

  // load values
  foreach ( KLineEdit *lineEdit, mLineEdits )
    lineEdit->setText( mAttributes.name( (ContactAttributes::Type)lineEdit->property( "attributeType" ).toInt() ) );
}

#include "configdialog.moc"
