/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "contactattributes.h"

#include <klocale.h>

ContactAttributes::TypeList ContactAttributes::allTypes()
{
  TypeList list;
  list << ObjectClass
       << CommonName
       << Uid
       << FormattedName
       << Title
       << GivenName
       << FamilyName
       << Birthday
       << GpgKey
       << SMimeCertificate
       << WorkMail
       << HomeMail
       << Url
       << HomePhoneNumber
       << WorkPhoneNumber
       << MobilePhoneNumber
       << FaxNumber
       << PagerNumber
       << IsdnNumber
       << WorkAddressPostOfficeBox
       << WorkAddressStreet
       << WorkAddressPostalCode
       << WorkAddressCity
       << WorkAddressState
       << HomeAddressPostOfficeBox
       << HomeAddressStreet
       << HomeAddressPostalCode
       << HomeAddressCity
       << HomeAddressState
       << Profession
       << Organization
       << Department
       << Office
       << Photo
       << Sound
       << Note;

  return list;
}

QString ContactAttributes::label( Type type )
{
  QString label;

  switch ( type ) {
    case ObjectClass: label = i18n( "Object classes" ); break;
    case CommonName: label = i18n( "Common name" ); break;
    case Uid: label = i18n( "UID" ); break;
    case FormattedName: label = i18n( "Formatted name" ); break;
    case Title: label = i18nc( "job title", "Title" ); break;
    case GivenName: label = i18n( "Given name" ); break;
    case FamilyName: label = i18n( "Family name" ); break;
    case Birthday: label = i18n( "Birthday" ); break;
    case GpgKey: label = i18n( "GPG Key" ); break;
    case SMimeCertificate: label = i18n( "S/MIME Certificate" ); break;
    case WorkMail: label = i18nc( "email address", "Email (work)" ); break;
    case HomeMail: label = i18nc( "email address", "Email (home)" ); break;
    case Url: label = i18n( "Homepage" ); break;
    case HomePhoneNumber: label = i18n( "Home telephone number" ); break;
    case WorkPhoneNumber: label = i18n( "Work telephone number" ); break;
    case MobilePhoneNumber: label = i18n( "Mobile phone number" ); break;
    case FaxNumber: label = i18n( "Fax number" ); break;
    case PagerNumber: label = i18n( "Pager" ); break;
    case IsdnNumber: label = i18n( "ISDN" ); break;
    case WorkAddressPostOfficeBox: label = i18n( "Post Office Box (work)" ); break;
    case WorkAddressStreet: label = i18n( "Street (work)" ); break;
    case WorkAddressPostalCode: label = i18n( "Postal code (work)" ); break;
    case WorkAddressCity: label = i18n( "City (work)" ); break;
    case WorkAddressState: label = i18nc( "state/province", "State (work)" ); break;
    case HomeAddressPostOfficeBox: label = i18n( "Post Office Box (home)" ); break;
    case HomeAddressStreet: label = i18n( "Street (home)" ); break;
    case HomeAddressPostalCode: label = i18n( "Postal code (home)" ); break;
    case HomeAddressCity: label = i18n( "City (home)" ); break;
    case HomeAddressState: label = i18nc( "state/province", "State (home)" ); break;
    case Profession: label = i18n( "Profession" ); break;
    case Organization: label = i18n( "Organization" ); break;
    case Department: label = i18n( "Department" ); break;
    case Office: label = i18n( "Office" ); break;
    case Photo: label = i18n( "Photo" ); break;
    case Sound: label = i18n( "Sound" ); break;
    case Note: label = i18n( "Note" ); break;
  }

  return label;
}

ContactAttributes::TypeList ContactAttributes::types() const
{
  return mAttributes.keys();
}

QString ContactAttributes::name( Type type ) const
{
  return mAttributes.value( type );
}

void ContactAttributes::setName( Type type, const QString &name )
{
  mAttributes.insert( type, name );
}

ContactAttributes ContactAttributes::fromString( const QString &value )
{
  if ( value.isEmpty() )
    return defaultAttributes();

  const QStringList list = value.split( QLatin1String( "|" ), QString::KeepEmptyParts );

  ContactAttributes attributes;
  for ( int i = 0; i < list.count(); i += 2 )
    attributes.mAttributes.insert( (Type)list[ i ].toInt(), list[ i + 1 ] );

  return attributes;
}

QString ContactAttributes::toString() const
{
  QStringList list;

  QMapIterator<Type, QString> it( mAttributes );
  while ( it.hasNext() ) {
    it.next();
    list.append( QString::number( (int)it.key() ) );
    list.append( it.value() );
  }

  return list.join( QLatin1String( "|" ) );
}

ContactAttributes ContactAttributes::defaultAttributes()
{
  ContactAttributes attributes;

  attributes.mAttributes.insert( ObjectClass, QLatin1String( "top,person,organizationalPerson,inetOrgPerson" ) );
  attributes.mAttributes.insert( CommonName, QLatin1String( "cn" ) );
  attributes.mAttributes.insert( Uid, QLatin1String( "uid" ) );
  attributes.mAttributes.insert( FormattedName, QLatin1String( "displayName" ) );
  attributes.mAttributes.insert( Title, QLatin1String( "title" ) );
  attributes.mAttributes.insert( FamilyName, QLatin1String( "sn" ) );
  attributes.mAttributes.insert( GivenName, QLatin1String( "givenName" ) );
  attributes.mAttributes.insert( WorkMail, QLatin1String( "mail" ) );
  attributes.mAttributes.insert( HomePhoneNumber, QLatin1String( "homePhone" ) );
  attributes.mAttributes.insert( WorkPhoneNumber, QLatin1String( "telephoneNumber" ) );
  attributes.mAttributes.insert( FaxNumber, QLatin1String( "facsimileTelephoneNumber" ) );
  attributes.mAttributes.insert( MobilePhoneNumber, QLatin1String( "mobile" ) );
  attributes.mAttributes.insert( PagerNumber, QLatin1String( "pager" ) );
  attributes.mAttributes.insert( IsdnNumber, QLatin1String( "internationaliSDNNumber" ) );
  attributes.mAttributes.insert( WorkAddressStreet, QLatin1String( "postOfficeBox" ) );
  attributes.mAttributes.insert( WorkAddressStreet, QLatin1String( "street" ) );
  attributes.mAttributes.insert( WorkAddressPostalCode, QLatin1String( "postalCode" ) );
  attributes.mAttributes.insert( WorkAddressCity, QLatin1String( "l" ) );
  attributes.mAttributes.insert( WorkAddressState, QLatin1String( "st" ) );
  attributes.mAttributes.insert( Profession, QLatin1String( "employeeType" ) );
  attributes.mAttributes.insert( Organization, QLatin1String( "o" ) );
  attributes.mAttributes.insert( Department, QLatin1String( "departmentNumber" ) );
  attributes.mAttributes.insert( Office, QLatin1String( "roomNumber" ) );
  attributes.mAttributes.insert( Photo, QLatin1String( "jpegPhoto" ) );
  attributes.mAttributes.insert( Sound, QLatin1String( "audio" ) );
  attributes.mAttributes.insert( Note, QLatin1String( "description" ) );

  return attributes;
}
