/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "ldapresource.h"

#include "configdialog.h"
#include "settingsadaptor.h"

#include <akonadi/changerecorder.h>
#include <akonadi/collectionfetchscope.h>
#include <akonadi/entitydisplayattribute.h>
#include <akonadi/itemfetchscope.h>

#include <kabc/addressee.h>
#include <kio/deletejob.h>
#include <kio/job.h>
#include <kldap/ldif.h>
#include <klocale.h>

using namespace Akonadi;

static QByteArray addEntry( const QString &attribute, const QString &value, bool modifyContact )
{
  QByteArray tmp;
  if ( !attribute.isEmpty() ) {
    if ( modifyContact )
      tmp += KLDAP::Ldif::assembleLine( QLatin1String( "replace" ), attribute ) + '\n';

    if ( !value.isEmpty() )
      tmp += KLDAP::Ldif::assembleLine( attribute, value ) + '\n';

    if ( modifyContact )
      tmp += "-\n";

  }

  return tmp;
}

static QString contactToLdif( const KABC::Addressee &contact, const QString &origDn, QByteArray &ldif )
{
  QByteArray tmp;
  QString dn;
  QByteArray data;
  bool modifyContact = false;

  const ContactAttributes attributes = ContactAttributes::fromString( Settings::self()->attributes() );

  if ( origDn.isEmpty() ) {
    // create new contact
    switch ( Settings::self()->rdnPrefix() ) {
      case 0:
      default:
        dn = attributes.name( ContactAttributes::CommonName ) + QLatin1Char( '=' ) + contact.assembledName() +
             QLatin1Char( ',' ) + Settings::self()->dn();
        break;
      case 1:
        dn = attributes.name( ContactAttributes::Uid ) + QLatin1Char( '=' ) + contact.uid() +
             QLatin1Char( ',' ) + Settings::self()->dn();
        break;
    }
  } else {
    // modify existing contact
    modifyContact = true;

    if ( origDn.startsWith( attributes.name( ContactAttributes::Uid ) ) ) {
      dn = attributes.name( ContactAttributes::Uid ) + QLatin1Char( '=' ) + contact.uid() +
           QLatin1Char( ',' ) + origDn.section( QLatin1Char( ',' ), 1 );
    } else if ( origDn.startsWith( attributes.name( ContactAttributes::CommonName ) ) ) {
      dn = attributes.name( ContactAttributes::CommonName ) + QLatin1Char( '=' ) + contact.assembledName() +
           QLatin1Char( ',' ) + origDn.section( QLatin1Char( ',' ), 1 );
    } else {
      dn = origDn;
    }

    if ( origDn.toLower() != dn.toLower() ) {
      tmp = KLDAP::Ldif::assembleLine( QLatin1String( "dn" ), origDn ) + '\n';
      tmp += "changetype: modrdn\n";
      tmp += KLDAP::Ldif::assembleLine( QLatin1String( "newrdn" ),
                                        dn.section( QLatin1Char( ',' ), 0, 0 ) ) + '\n';
      tmp += "deleteoldrdn: 1\n\n";
    }
  }

  tmp += KLDAP::Ldif::assembleLine( QLatin1String( "dn" ), dn ) + '\n';
  if ( modifyContact ) {
    tmp += "changetype: modify\n";
  } else {
    tmp += "objectClass: top\n";
    const QStringList objectClasses = attributes.name( ContactAttributes::ObjectClass ).split( QLatin1Char( ',' ),
                                                                                               QString::SkipEmptyParts );

    foreach ( const QString objectClass, objectClasses )
      tmp += KLDAP::Ldif::assembleLine( QLatin1String( "objectClass" ), objectClass ) + '\n';
  }

  tmp += addEntry( attributes.name( ContactAttributes::CommonName ), contact.assembledName(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::FormattedName ), contact.formattedName(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::GivenName ), contact.givenName(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::FamilyName ), contact.familyName(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::Uid ), contact.uid(), modifyContact );

  tmp += addEntry( attributes.name( ContactAttributes::Birthday ), contact.birthday().date().toString( "yyyy-MM-dd" ), modifyContact );

  KABC::PhoneNumber number;
  number = contact.phoneNumber( KABC::PhoneNumber::Home );
  tmp += addEntry( attributes.name( ContactAttributes::HomePhoneNumber ), number.number(), modifyContact );
  number = contact.phoneNumber( KABC::PhoneNumber::Work );
  tmp += addEntry( attributes.name( ContactAttributes::WorkPhoneNumber ), number.number(), modifyContact );
  number = contact.phoneNumber( KABC::PhoneNumber::Fax );
  tmp += addEntry( attributes.name( ContactAttributes::MobilePhoneNumber ), number.number(), modifyContact );
  number = contact.phoneNumber( KABC::PhoneNumber::Pager );
  tmp += addEntry( attributes.name( ContactAttributes::FaxNumber ), number.number(), modifyContact );
  number = contact.phoneNumber( KABC::PhoneNumber::Cell );
  tmp += addEntry( attributes.name( ContactAttributes::PagerNumber ), number.number(), modifyContact );
  number = contact.phoneNumber( KABC::PhoneNumber::Isdn );
  tmp += addEntry( attributes.name( ContactAttributes::IsdnNumber ), number.number(), modifyContact );

  tmp += addEntry( attributes.name( ContactAttributes::Note ), contact.note(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::Title ), contact.title(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::Profession ), contact.custom( QLatin1String( "KADDRESSBOOK" ), QLatin1String( "X-Profession" ) ), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::Organization ), contact.organization(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::Department ), contact.department(), modifyContact );
  tmp += addEntry( attributes.name( ContactAttributes::Office ), contact.custom( QLatin1String( "KADDRESSBOOK" ), QLatin1String( "X-Office" ) ), modifyContact );

  KABC::Address address = contact.address( KABC::Address::Work );
  if ( !address.isEmpty() ) {
    tmp += addEntry( attributes.name( ContactAttributes::WorkAddressPostOfficeBox ), address.postOfficeBox(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::WorkAddressStreet ), address.street(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::WorkAddressState ), address.region(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::WorkAddressCity ), address.locality(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::WorkAddressPostalCode ), address.postalCode(), modifyContact );
  }

  address = contact.address( KABC::Address::Home );
  if ( !address.isEmpty() ) {
    tmp += addEntry( attributes.name( ContactAttributes::HomeAddressPostOfficeBox ), address.postOfficeBox(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::HomeAddressStreet ), address.street(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::HomeAddressState ), address.region(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::HomeAddressCity ), address.locality(), modifyContact );
    tmp += addEntry( attributes.name( ContactAttributes::HomeAddressPostalCode ), address.postalCode(), modifyContact );
  }

  const QStringList emails = contact.emails();

  QStringList::ConstIterator mailIt = emails.constBegin();

  if ( !attributes.name( ContactAttributes::WorkMail ).isEmpty() ) {
    if ( modifyContact )
      tmp += KLDAP::Ldif::assembleLine( QLatin1String( "replace" ), attributes.name( ContactAttributes::WorkMail ) ) + '\n';

    if ( mailIt != emails.constEnd() ) {
      tmp += KLDAP::Ldif::assembleLine( attributes.name( ContactAttributes::WorkMail ), *mailIt ) + '\n';
      mailIt ++;
    }

    if ( modifyContact &&
         attributes.name( ContactAttributes::WorkMail ) != attributes.name( ContactAttributes::HomeMail ) ) {
      tmp += "-\n";
    }
  }

  if ( !attributes.name( ContactAttributes::HomeMail ).isEmpty() ) {
    if ( modifyContact && attributes.name( ContactAttributes::WorkMail ) != attributes.name( ContactAttributes::HomeMail ) ) {
      tmp += KLDAP::Ldif::assembleLine( QLatin1String( "replace" ),
                                        attributes.name( ContactAttributes::HomeMail ) ) + '\n';
    }

    for ( ; mailIt != emails.constEnd(); ++mailIt )
      tmp += KLDAP::Ldif::assembleLine( attributes.name( ContactAttributes::HomeMail ), *mailIt ) + '\n';

    if ( modifyContact )
      tmp += "-\n";
  }

  tmp += addEntry( attributes.name( ContactAttributes::Url ), contact.url().url(), modifyContact );

  if ( !attributes.name( ContactAttributes::Photo ).isEmpty() ) {
    QByteArray data;
    QBuffer buffer( &data );
    buffer.open( QIODevice::WriteOnly );

    contact.photo().data().save( &buffer, "JPEG" );

    if ( modifyContact )
      tmp += KLDAP::Ldif::assembleLine( QLatin1String( "replace" ), attributes.name( ContactAttributes::Photo ) ) + '\n';

    tmp += KLDAP::Ldif::assembleLine( attributes.name( ContactAttributes::Photo ), data, 76 ) + '\n';

    if ( modifyContact )
      tmp += "-\n";
  }

  tmp += '\n';

  ldif = tmp;

  return dn;
}

static QMap<QString, KABC::Addressee> ldifToContacts( const QByteArray &data )
{
  QMap<QString, KABC::Addressee> contacts;
  KLDAP::Ldif parser;

  parser.setLdif( data );
  parser.startParsing();

  QString dn;
  KABC::Addressee contact;
  KABC::Address workAddress( KABC::Address::Work );
  KABC::Address homeAddress( KABC::Address::Home );

  const ContactAttributes contactAttributes = ContactAttributes::fromString( Settings::self()->attributes() );

  KLDAP::Ldif::ParseValue parseValue;
  do {
    parseValue = parser.nextItem();
    switch ( parseValue ) {
      case KLDAP::Ldif::NewEntry:
        dn = parser.dn().toString();
        break;
      case KLDAP::Ldif::Item:
        {
          const QString name = parser.attr().toLower();
          const QByteArray value = parser.value();
          if ( name == contactAttributes.name( ContactAttributes::CommonName ).toLower() ) {
            if ( !contact.formattedName().isEmpty() ) {
              const QString formattedName = contact.formattedName();
              contact.setNameFromString( QString::fromUtf8( value, value.size() ) );
              contact.setFormattedName( formattedName );
            } else {
              contact.setNameFromString( QString::fromUtf8( value, value.size() ) );
            }
          } else if ( name == contactAttributes.name( ContactAttributes::FormattedName ).toLower() ) {
            contact.setFormattedName( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::GivenName ).toLower() ) {
            contact.setGivenName( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::FamilyName ).toLower() ) {
            contact.setFamilyName( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::Birthday ).toLower() ) {
            const QString dateString = QString::fromUtf8( value, value.size() );
            contact.setBirthday( QDateTime( QDate::fromString( dateString, "yyyy-MM-dd" ) ) );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkMail ).toLower() ) {
            contact.insertEmail( QString::fromUtf8( value, value.size() ), true );
          } else if ( name == contactAttributes.name( ContactAttributes::HomeMail ).toLower() ) {
            contact.insertEmail( QString::fromUtf8( value, value.size() ), false );
          } else if ( name == contactAttributes.name( ContactAttributes::Url ).toLower() ) {
            contact.setUrl( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::HomePhoneNumber ).toLower() ) {
            const KABC::PhoneNumber number( QString::fromUtf8( value, value.size() ), KABC::PhoneNumber::Home );
            contact.insertPhoneNumber( number );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkPhoneNumber ).toLower() ) {
            const KABC::PhoneNumber number( QString::fromUtf8( value, value.size() ), KABC::PhoneNumber::Work );
            contact.insertPhoneNumber( number );
          } else if ( name == contactAttributes.name( ContactAttributes::MobilePhoneNumber ).toLower() ) {
            const KABC::PhoneNumber number( QString::fromUtf8( value, value.size() ), KABC::PhoneNumber::Cell );
            contact.insertPhoneNumber( number );
          } else if ( name == contactAttributes.name( ContactAttributes::FaxNumber ).toLower() ) {
            const KABC::PhoneNumber number( QString::fromUtf8( value, value.size() ), KABC::PhoneNumber::Fax );
            contact.insertPhoneNumber( number );
          } else if ( name == contactAttributes.name( ContactAttributes::PagerNumber ).toLower() ) {
            const KABC::PhoneNumber number( QString::fromUtf8( value, value.size() ), KABC::PhoneNumber::Pager );
            contact.insertPhoneNumber( number );
          } else if ( name == contactAttributes.name( ContactAttributes::IsdnNumber ).toLower() ) {
            const KABC::PhoneNumber number( QString::fromUtf8( value, value.size() ), KABC::PhoneNumber::Isdn );
            contact.insertPhoneNumber( number );
          } else if ( name == contactAttributes.name( ContactAttributes::Note ).toLower() ) {
            contact.setNote( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::Title ).toLower() ) {
            contact.setTitle( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkAddressPostOfficeBox ).toLower() ) {
            workAddress.setPostOfficeBox( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkAddressStreet ).toLower() ) {
            workAddress.setStreet( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkAddressState ).toLower() ) {
            workAddress.setRegion( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkAddressCity ).toLower() ) {
            workAddress.setLocality( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::WorkAddressPostalCode ).toLower() ) {
            workAddress.setPostalCode( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::HomeAddressPostOfficeBox ).toLower() ) {
            homeAddress.setPostOfficeBox( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::HomeAddressStreet ).toLower() ) {
            homeAddress.setStreet( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::HomeAddressState ).toLower() ) {
            homeAddress.setRegion( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::HomeAddressCity ).toLower() ) {
            homeAddress.setLocality( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::HomeAddressPostalCode ).toLower() ) {
            homeAddress.setPostalCode( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::Organization ).toLower() ) {
            contact.setOrganization( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::Uid ).toLower() ) {
            contact.setUid( QString::fromUtf8( value, value.size() ) );
          } else if ( name == contactAttributes.name( ContactAttributes::Photo ).toLower() ) {
            KABC::Picture photo;
            const QImage image = QImage::fromData( value );
            if ( !image.isNull() ) {
              photo.setData( image );
              photo.setType( QLatin1String( "image/jpeg" ) );
              contact.setPhoto( photo );
            }
          }
        }

        break;
      case KLDAP::Ldif::EndEntry:
        {
          if ( !workAddress.isEmpty() )
            contact.insertAddress( workAddress );

          if ( !homeAddress.isEmpty() )
            contact.insertAddress( homeAddress );

          contacts.insert( dn, contact );

          // prepare for next round
          dn.clear();
          contact = KABC::Addressee();
          workAddress = KABC::Address( KABC::Address::Work );
          homeAddress = KABC::Address( KABC::Address::Home );
        }
        break;
      default:
        break;
    }
  } while ( parseValue != KLDAP::Ldif::MoreData );

  // handle last entry
  if ( !dn.isEmpty() ) {
    if ( !workAddress.isEmpty() )
      contact.insertAddress( workAddress );

    if ( !homeAddress.isEmpty() )
      contact.insertAddress( homeAddress );

    contacts.insert( dn, contact );
  }

  return contacts;
}

LdapResource::LdapResource( const QString &id )
  : ResourceBase( id )
{
  // setup the resource
  new SettingsAdaptor( Settings::self() );
  QDBusConnection::sessionBus().registerObject( QLatin1String( "/Settings" ),
                            Settings::self(), QDBusConnection::ExportAdaptors );

  changeRecorder()->fetchCollection( true );
  changeRecorder()->itemFetchScope().fetchFullPayload( true );
  changeRecorder()->itemFetchScope().setAncestorRetrieval( ItemFetchScope::Parent );
  changeRecorder()->collectionFetchScope().setAncestorRetrieval( CollectionFetchScope::Parent );

  setName( i18n( "LDAP" ) );

  initLdapUrl();
}

LdapResource::~LdapResource()
{
}

void LdapResource::aboutToQuit()
{
}

void LdapResource::configure( WId windowId )
{
  ConfigDialog dlg( windowId );
  if ( dlg.exec() ) {
    initLdapUrl();
    synchronize();
    emit configurationDialogAccepted();
  } else {
    emit configurationDialogRejected();
  }
}

void LdapResource::retrieveCollections()
{
  Collection resourceCollection;
  resourceCollection.setParentCollection( Collection::root() );
  resourceCollection.setRemoteId( QLatin1String( "akonadi_ldap_resource" ) );
  resourceCollection.setName( name() );
  resourceCollection.setContentMimeTypes( QStringList() << KABC::Addressee::mimeType() );
  //TODO: adapt rights to read-only mode
  resourceCollection.setRights( Collection::CanCreateItem | Collection::CanChangeItem | Collection::CanDeleteItem );

  collectionsRetrieved( Collection::List() << resourceCollection );
}

void LdapResource::retrieveItems( const Akonadi::Collection& )
{
  KIO::Job *job = KIO::storedGet( mLdapUrl, KIO::Reload, KIO::HideProgressInfo );
  connect( job, SIGNAL( result( KJob* ) ), SLOT( onRetrieveItemsFinished( KJob* ) ) );
}


bool LdapResource::retrieveItem( const Akonadi::Item &item, const QSet<QByteArray>& )
{
  itemRetrieved( item );
  return true;
}

void LdapResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection& )
{
  const KABC::Addressee contact = item.payload<KABC::Addressee>();

  QByteArray ldif;
  const QString dn = contactToLdif( contact, QString() /* new contact */, ldif );

  KJob *job = KIO::storedPut( ldif, mLdapUrl, -1 /* no special permissions */, KIO::HideProgressInfo );
  job->setProperty( "item", QVariant::fromValue( item ) );
  job->setProperty( "dn", dn );
  connect( job, SIGNAL( result( KJob* ) ), SLOT( onAddItemFinished( KJob* ) ) );
}

void LdapResource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray>& )
{
  const QString dn = item.remoteId();
  const KABC::Addressee contact = item.payload<KABC::Addressee>();

  QByteArray ldif;
  contactToLdif( contact, dn, ldif );

  KJob *job = KIO::storedPut( ldif, mLdapUrl, -1 /* no special permissions */, KIO::HideProgressInfo );
  job->setProperty( "item", QVariant::fromValue( item ) );
  connect( job, SIGNAL( result( KJob* ) ), SLOT( onAddItemFinished( KJob* ) ) );
}

void LdapResource::itemRemoved( const Akonadi::Item &item )
{
  const QString dn = item.remoteId();

  KLDAP::LdapUrl url( mLdapUrl );
  url.setPath( QLatin1Char( '/' ) + dn );
  url.setExtension( QLatin1String( "x-dir" ), QLatin1String( "base" ) );
  url.setScope( KLDAP::LdapUrl::Base );

  KJob *job = KIO::del( url, KIO::HideProgressInfo );
  connect( job, SIGNAL( result( KJob* ) ), SLOT( onRemoveItemFinished( KJob* ) ) );
}

void LdapResource::onRetrieveItemsFinished( KJob *job )
{
  if ( job->error() ) {
    cancelTask( job->errorText() );
    return;
  }

  KIO::StoredTransferJob *transferJob = qobject_cast<KIO::StoredTransferJob*>( job );

  const QByteArray data = transferJob->data();
  const QMap<QString, KABC::Addressee> contacts = ldifToContacts( data );

  Item::List items;

  QMapIterator<QString, KABC::Addressee> it( contacts );
  while ( it.hasNext() ) {
    it.next();

    Item item;
    item.setMimeType( KABC::Addressee::mimeType() );
    item.setPayload<KABC::Addressee>( it.value() );
    item.setRemoteId( it.key() );

    items.append( item );
  }

  itemsRetrieved( items );
}

void LdapResource::onAddItemFinished( KJob *job )
{
  if ( job->error() ) {
    cancelTask( job->errorText() );
    return;
  }

  Item item = job->property( "item" ).value<Item>();
  item.setRemoteId( job->property( "dn" ).toString() );

  changeCommitted( item );
}

void LdapResource::onModifyItemFinished( KJob *job )
{
  if ( job->error() ) {
    cancelTask( job->errorText() );
    return;
  }

  const Item item = job->property( "item" ).value<Item>();

  changeCommitted( item );
}

void LdapResource::onRemoveItemFinished( KJob *job )
{
  if ( job->error() ) {
    cancelTask( job->errorText() );
    return;
  }

  changeProcessed();
}

void LdapResource::initLdapUrl()
{
  mLdapUrl = KLDAP::LdapUrl();

  if ( !Settings::self()->isAnonymous() ) {
    mLdapUrl.setExtension( QLatin1String( "bindname" ), Settings::self()->bindDN() );
    mLdapUrl.setUser( Settings::self()->user() );
    mLdapUrl.setPass( Settings::self()->password() );
  }

  mLdapUrl.setProtocol( Settings::self()->isSSL() ? QLatin1String( "ldaps" ) : QLatin1String( "ldap" ) );
  mLdapUrl.setHost( Settings::self()->host() );
  mLdapUrl.setPort( Settings::self()->port() );
  mLdapUrl.setDn( KLDAP::LdapDN( Settings::self()->dn() ) );

  QStringList attributes;
  const ContactAttributes contactAttributes = ContactAttributes::fromString( Settings::self()->attributes() );
  const ContactAttributes::TypeList types = contactAttributes.types();
  foreach ( const ContactAttributes::Type &type, types ) {
    if ( type != ContactAttributes::ObjectClass ) {
      const QString attribute = contactAttributes.name( type );
      if ( !attribute.isEmpty() )
        attributes.append( attribute );
    }
  }

  if ( !attributes.isEmpty() )
    mLdapUrl.setAttributes( attributes );

  mLdapUrl.setScope( Settings::self()->isSubTree() ? KLDAP::LdapUrl::Sub : KLDAP::LdapUrl::One );

  if ( !Settings::self()->filter().isEmpty() &&
       Settings::self()->filter() != QLatin1String( "(objectClass=*)" ) ) {
    mLdapUrl.setFilter( Settings::self()->filter() );
  }

  mLdapUrl.setExtension( QLatin1String( "x-dir" ), QLatin1String( "base" ) );

  if ( Settings::self()->isTLS() )
    mLdapUrl.setExtension( QLatin1String( "x-tls" ), QString() );

  mLdapUrl.setExtension( QLatin1String( "x-ver" ), QString::number( Settings::self()->version() ) );

  if ( Settings::self()->sizeLimit() )
    mLdapUrl.setExtension( QLatin1String( "x-sizelimit" ), QString::number( Settings::self()->sizeLimit() ) );

  if ( Settings::self()->timeLimit() )
    mLdapUrl.setExtension( QLatin1String( "x-timelimit" ), QString::number( Settings::self()->timeLimit() ) );

  if ( Settings::self()->isSASL() ) {
    mLdapUrl.setExtension( QLatin1String( "x-sasl" ), QString() );

    if ( !Settings::self()->mech().isEmpty() )
      mLdapUrl.setExtension( QLatin1String( "x-mech" ), Settings::self()->mech() );

    if ( !Settings::self()->realm().isEmpty() )
      mLdapUrl.setExtension( QLatin1String( "x-realm" ), Settings::self()->realm() );
  }
}

AKONADI_RESOURCE_MAIN( LdapResource )

#include "ldapresource.moc"
