/*
    This file is part of KDE playground.
    Copyright (c) 2007 Mathias Soeken <msoeken@tzi.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "mainwindow.h"

#include "persontreeview.h"
#include "personmodel.h"
#include "personfilterproxymodel.h"

#include <QAction>
#include <QCheckBox>
#include <QComboBox>
#include <QHBoxLayout>
#include <QItemDelegate>
#include <QKeyEvent>
#include <QLabel>
#include <QPainter>
#include <QPushButton>
#include <QRadioButton>
#include <QStandardItemModel>
#include <QToolBar>
#include <QVBoxLayout>

#include <KApplication>
#include <KLineEdit>
#include <KLocale>

class CategoryComboBoxDelegate : public QItemDelegate {
public:
  void paint( QPainter *painter, const QStyleOptionViewItem& option, const QModelIndex& index) const {
    QString value = index.model()->data( index, Qt::DisplayRole ).toString();
    
    if (value.startsWith( "-separator:" )) {
      painter->setPen( Qt::gray );
      painter->drawLine( option.rect.x(), option.rect.y() + (option.rect.height() / 2), option.rect.x() + 4, option.rect.y() + (option.rect.height() / 2) );
      
      QFont newFont = painter->font();
      newFont.setPointSize( 8 );
      painter->setFont( newFont );
      
      QRect boundingRect;
      painter->drawText( QRect(option.rect.x() + 5, option.rect.y(), option.rect.width() - 5, option.rect.height()), Qt::AlignLeft | Qt::AlignVCenter, value.mid(11), &boundingRect);
      
      painter->drawLine( boundingRect.right() + 1, option.rect.y() + (option.rect.height() / 2), option.rect.right(), option.rect.y() + (option.rect.height() / 2) );
    } else {
      QItemDelegate::paint( painter, option, index );
    }
  }    
};
 
MainWindow::MainWindow( QWidget *parent ) : KDialog( parent ), mBasicInterface( true )
{
  /* read basic Settings var from KConfig */
  // TODO use KMKernel if inserted in kmail
  // to store whether to use basic settings or not
  
  setWindowTitle( i18n("Select recipient") );
  setWindowIcon( KIcon("internet-mail") );
  showButtonSeparator( true );
  setWindowFlags( windowFlags() | Qt::WindowContextHelpButtonHint );
  setButtons( Help | Ok | Apply | Cancel | User1 );
  setButtonGuiItem( User1, KGuiItem(i18n("Advanced Interface")) );
  setButtonWhatsThis( User1, i18n("Toggles between a simple interface to add one or more persons as recipient or an advanced interface with more possibilities which are not required so often.") );
  showButton( Apply, !mBasicInterface );
  resize( 600, 450 );
  
  connect( this, SIGNAL(user1Clicked()), this, SLOT(toggleSettings()) );
  
//BEGIN TreeView
  mTreeWidget = new PersonTreeView( this );
  if (mBasicInterface) {
    mTreeWidget->setSelectionMode( QAbstractItemView::MultiSelection );
  } else {
    mTreeWidget->setSelectionMode( QAbstractItemView::ExtendedSelection );
  }
  mTreeWidget->setWhatsThis( i18n("This list displays persons who are filtered with the settings above.") );
  
  connect( mTreeWidget->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), this, SLOT(treeWidgetSelectionChanged()) );
  
  connect( mTreeWidget->proxyModel(), SIGNAL(modelReset()), this, SLOT(slotModelReset()) );
  
  mTreeWidget->setColumnHidden( PersonModel::StatusColumn, mBasicInterface );
//END
  
//BEGIN Toolbar
  mToolBar = new QToolBar( this );
  mToolBar->setMovable( false );
  
  mSelectMarked = new QCheckBox( i18n("Marked persons only ") );
  mSelectMarked->setWhatsThis( i18n("Show only persons who are marked with a status") );
  connect( mSelectMarked, SIGNAL(stateChanged(int)), this, SLOT(selectMarkedStateChanged(int)) );
  mAddressBooksComboBoxAction = mToolBar->addWidget( mSelectMarked );
  mAddressBooksComboBoxAction->setVisible( !mBasicInterface );
  
  mSearchLine = new KLineEdit;
  mSearchLine->setClickMessage( i18n("Search") );
  mSearchLine->setWhatsThis( i18n("Enter some text to filter the list of persons. You can also enter information that is not visible in the list, such as addresses or phone numbers.") );
  mSearchLine->setClearButtonShown( true );
  connect( mSearchLine, SIGNAL(textChanged(const QString&)), this, SLOT(editLineTextChanged(const QString&)) );
  mToolBar->addWidget( mSearchLine );
  
  mAddressBooksLabel = new QLabel( i18n(" Address book:") );
  mToolBar->addWidget( mAddressBooksLabel );
  
  mAddressBooksComboBox = new QComboBox;
  mAddressBooksComboBox->setItemDelegate( new CategoryComboBoxDelegate() );
  mAddressBooksComboBox->addItem( i18n("All") );
  mAddressBooksComboBox->addItem( i18n("Distribution Lists") );
  mAddressBooksComboBox->setWhatsThis( i18n("Select a address book or a category to display those persons only.") );
  connect( mAddressBooksComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(addressBooksComboBoxCurrentIndexChanged(int)) );
  mAddressBooksLabel->setBuddy( mAddressBooksComboBox );
  mToolBar->addWidget( mAddressBooksComboBox );
//END
  
//BEGIN Bottom Panel
  mBottomPanel = new QWidget( this );
  mBottomPanel->setVisible( !mBasicInterface );
  mBottomPanelLayout = new QHBoxLayout( mBottomPanel );
  mBottomPanelLayout->setMargin( 0 );
  
  mBtnNone = new QPushButton( KIcon("edit-clear-locationbar-rtl"), "Remove mark" );
  mBtnNone->setEnabled( false );
  connect( mBtnNone, SIGNAL(clicked()), this, SLOT(actionNone()) );
  mBottomPanelLayout->addWidget( mBtnNone );
  
  mBtnTo = new QPushButton( KIcon("mail-to"), "Add to To" );
  mBtnTo->setEnabled( false );
  mBtnTo->setWhatsThis( "Add this person as recipient" );
  connect( mBtnTo, SIGNAL(clicked()), this, SLOT(actionTo()) );
  mBottomPanelLayout->addWidget( mBtnTo );
  
  mBtnCC = new QPushButton( KIcon("mail-cc"), "Add to CC" );
  mBtnCC->setEnabled( false );
  mBtnCC->setWhatsThis( "Add this person as carbon copy recipient" );
  connect( mBtnCC, SIGNAL(clicked()), this, SLOT(actionCC()) );
  mBottomPanelLayout->addWidget( mBtnCC );
  
  mBtnBCC = new QPushButton( KIcon("mail-bcc"), "Add to BCC" );
  mBtnBCC->setEnabled( false );
  mBtnBCC->setWhatsThis( "Add this person as blind carbon copy recipient" );
  connect( mBtnBCC, SIGNAL(clicked()), this, SLOT(actionBCC()) );
  mBottomPanelLayout->addWidget( mBtnBCC );
//END
    
//BEGIN Radio Buttons
  mRadBCCMail = new QRadioButton( i18n("Do not show recipients' names to each other."), this );
  mRadCCMail = new QRadioButton( i18n("Display all recipients' names to each other."), this );
  mRadBCCMail->setChecked( true );
  /* These widgets are shown, if mode is basic interface and
   * more than one item is selected.
   */
  mRadBCCMail->setVisible( false );
  mRadCCMail->setVisible( false );
//END
  
  mMainLayout = new QVBoxLayout();
  mMainLayout->setSpacing( KDialog::spacingHint() );
  mMainLayout->setMargin( KDialog::spacingHint() / 2 );
  
  mMainLayout->addWidget( mToolBar );
  mMainLayout->addWidget( mTreeWidget ); 
  mMainLayout->addWidget( mRadBCCMail );
  mMainLayout->addWidget( mRadCCMail );
  mMainLayout->addWidget( mBottomPanel );
  
  mainWidget()->setLayout( mMainLayout );

  mTreeWidget->setFocus();
}

void MainWindow::treeWidgetSelectionChanged() 
{ 
  if (mBasicInterface) {
    bool visible = mTreeWidget->selectionModel()->selectedRows().count() > 1;
    mRadBCCMail->setVisible( visible );
    mRadCCMail->setVisible( visible );
  } else {
    bool enabled = mTreeWidget->selectionModel()->selectedRows().count() > 0;
    mBtnNone->setEnabled( enabled );
    mBtnTo->setEnabled( enabled );
    mBtnCC->setEnabled( enabled );
    mBtnBCC->setEnabled( enabled );
  }
}

void MainWindow::editLineTextChanged( const QString& text )
{
  mTreeWidget->proxyModel()->setFilterFixedString( text );
}

void MainWindow::selectMarkedStateChanged( int state )
{
  mTreeWidget->proxyModel()->setMarkedItemsOnly( state == Qt::Checked );
  mTreeWidget->proxyModel()->setFilterFixedString( mSearchLine->text() );
}

void MainWindow::toggleSettings()
{
  mBasicInterface = !mBasicInterface;
  
  mTreeWidget->toggleSettings( mBasicInterface );
  
  if (mBasicInterface) {
    setButtonText( User1, i18n("Advanced Interface") );
  } else {
    setButtonText( User1, i18n("Basic Interface") );
  }
  
  showButton( Apply, !mBasicInterface );
  mAddressBooksComboBoxAction->setVisible( !mBasicInterface );
  mRadBCCMail->setVisible( mBasicInterface );
  mRadCCMail->setVisible( mBasicInterface );
  mBottomPanel->setVisible( !mBasicInterface );
  
  /* some widgets has to be updated because of 
   * current selection
   */
  treeWidgetSelectionChanged();
  
  // store in KConfig
  // TODO use KMKernel if inserted in kmail
  // to store whether to use basic settings or not
}

void MainWindow::slotModelReset() 
{
  mAddressBooksComboBox->clear();
  mAddressBooksComboBox->addItem( i18n("All") );
  mAddressBooksComboBox->addItem( i18n("Distribution Lists") );
  mAddressBooksComboBox->addItem( "-separator:" + i18n("Category") );      
  
  QStandardItemModel *model = qobject_cast<QStandardItemModel*>( mAddressBooksComboBox->model() );
  if (model) {
    QStandardItem *item = model->item( 2, 0 );
    if (item) {
      item->setFlags( 0 );
    }
  }
  
  foreach (QString category, mTreeWidget->model()->categories()) {
    mAddressBooksComboBox->addItem( category );
  }
  
  // seems unnecessary, but it the effect is not the same
  // when just calling in constructor since Qt 4.3.0
  mTreeWidget->setColumnHidden( PersonModel::StatusColumn, mBasicInterface );
}

void MainWindow::addressBooksComboBoxCurrentIndexChanged( int index )
{
  // FIXME create static const for the 2
  if (index > 2) {
    mTreeWidget->proxyModel()->setCategoryFilter( mAddressBooksComboBox->itemText(index) ); 
  } else {
    mTreeWidget->proxyModel()->setCategoryFilter( QString() ); 
  }
  
  mTreeWidget->proxyModel()->setFilterFixedString( mSearchLine->text() );
}

void MainWindow::actionNone() 
{
  for (int i = 0; i < mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn ).size(); ++i) {
    QModelIndex index = mTreeWidget->proxyModel()->mapToSource( mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn )[i] );
    mTreeWidget->model()->setData( index, "" );
  }
}

void MainWindow::actionTo() 
{
  for (int i = 0; i < mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn ).size(); ++i) {
    QModelIndex index = mTreeWidget->proxyModel()->mapToSource( mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn )[i] );
    mTreeWidget->model()->setData( index, "To" );
  }
}

void MainWindow::actionCC () 
{
  for (int i = 0; i < mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn ).size(); ++i) {
    QModelIndex index = mTreeWidget->proxyModel()->mapToSource( mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn )[i] );
    mTreeWidget->model()->setData( index, "CC" );
  }
}

void MainWindow::actionBCC () 
{
  for (int i = 0; i < mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn ).size(); ++i) {
    QModelIndex index = mTreeWidget->proxyModel()->mapToSource( mTreeWidget->selectionModel()->selectedRows( PersonModel::StatusColumn )[i] );
    mTreeWidget->model()->setData( index, "BCC" );
  }
}

void MainWindow::keyPressEvent( QKeyEvent* e )
{
  if ((e->modifiers() & Qt::ControlModifier) && e->key() == Qt::Key_F) {
    mSearchLine->setFocus();
  }
}

#include "mainwindow.moc"



