/*
    This file is part of KDE playground.
    Copyright (c) 2007 Mathias Soeken <msoeken@tzi.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "personmodel.h"

#include <iostream>

#include <KDebug>
#include <KGlobalSettings>
#include <KIcon>
#include <KIconLoader>
#include <KTemporaryFile>

#include <QModelIndex>
#include <QVariant>

#include <kabc/addressbook.h>
#include <kabc/phonenumber.h>
#include <kabc/stdaddressbook.h>

//BEGIN PersonModelItem data structure
struct PersonModelItem {
  PersonModelItem( Addressee *addressee, uint email, PersonModelItem *parent = 0 ) : mAddressee( addressee ), mEmail( email ), mParent( parent ), mSendingType( None ), mChecked( false ), mTempFile() {
    mTempFile.setSuffix( ".png" );
    mTempFile.open();
  }
  
  ~PersonModelItem() {
    mTempFile.close();
    qDeleteAll( mChildren );
  }
  
  enum SendingType { None, To, CC, BCC };
  
private:
  Addressee *mAddressee;
  uint mEmail;
  PersonModelItem *mParent;
  
  int mSendingType;
  bool mChecked;
  
  QList<PersonModelItem*> mChildren;
  
  /**
   * This is for storing the image of the 
   * photo data of a contact. The image is 
   * always saved in the same file and deleted
   * when the instance of this class is deleted.
   */
  KTemporaryFile mTempFile;
  
private:
  QString abcData( int column ) const {
    switch (column) {
      case 0:
        return mAddressee->assembledName();
        
      case 1:
        // The preffered email is always at the first index
        return mAddressee->emails()[mEmail];
        
      case 3:
        return mAddressee->phoneNumber( PhoneNumber::Home ).number();
      
      case 4:
        return mAddressee->phoneNumber( PhoneNumber::Work ).number();
      
      case 5:
        return mAddressee->phoneNumber( PhoneNumber::Cell ).number();
        
      default:
        return QString();
    }
  }    
  
public:
  void appendChild( PersonModelItem* child ) {
    mChildren.append( child );
  }
  
  void clear() {
    mChildren.clear();
  }
  
  PersonModelItem* child( int row ) const {
    return mChildren.value( row );
  }
  
  int row() const {
    if (mParent) {
      return mParent->mChildren.indexOf( const_cast<PersonModelItem*>(this) );
    }
    
    return 0;
  }
  
  int childCount() const {
    return mChildren.count(); 
  }
  
  QVariant data( int column, int role = Qt::DisplayRole ) const {
    if (!mAddressee) {
      return QVariant();
    }
    
    switch (role) {
      case Qt::DisplayRole:
        switch (column) {
          case PersonModel::StatusColumn:
            return sendingTypeToString();
            
          case 0:
            return mAddressee->realName();
              
          default:
            return abcData( column );
        }
        break;
        
      case Qt::UserRole:
        if (column <= PersonModel::FieldsCount) {
          /** when filtering data more informations are given,
           *  e.g. the assembledName instead of the realName
           */
          return abcData( column );
        } else if (column == PersonModel::FieldsCount + 1) {
          /**
           * return the categories for category search 
           */
          return mAddressee->categories();
        }
        break;
        
      case Qt::ToolTipRole:
        if (column != PersonModel::StatusColumn) {
          QString toolTip = "<qt>";
    
          QImage photo = mAddressee->photo().data();
          if (!photo.isNull() && !mTempFile.fileName().isEmpty()) {
            photo.save( mTempFile.fileName() );
            toolTip += "<table><tr><td valign=\"top\"><img src=\"" + mTempFile.fileName() + "\"></td><td valign=\"top\">";
          } 
          
          toolTip += "<font size=\"+1\"><b>" + mAddressee->assembledName() + "</b></font>";
          
          QString fieldNames[] = { "eMail", "", "Phone (Home)", "Phone (Work)", "Phone (Mobile)" };
          
          for (int i = 0; i < (PersonModel::FieldsCount); ++i) {
            if (!abcData( i + 1 ).isEmpty() && !fieldNames[i].isEmpty()) {
              toolTip += "<br><b>" + fieldNames[i] + ":</b> " + abcData( i + 1 );
            }
          }
              
          if (!photo.isNull() && !mTempFile.fileName().isEmpty()) {
            toolTip += "</td></tr></table>";
          }
              
          return toolTip;
        } else {
          return QVariant();
        }
        break;
        
      case Qt::DecorationRole:
        if (column == PersonModel::StatusColumn) {
          if (mSendingType != None) {
            KIcon engine( "mail-" + sendingTypeToString().toLower(), KIconLoader::global() );
            return engine.pixmap( QSize(16, 16), QIcon::Normal, QIcon::On );
          } else {
            return QVariant();
          }
        } else if (column == 0) {
          QImage photo = mAddressee->photo().data();
          if (!photo.isNull()) {
            return QPixmap::fromImage( photo.scaled(20, 20, Qt::KeepAspectRatio, Qt::SmoothTransformation) );
          } else {
            // just because KIcon( "x-office-contact" ) does not work
            KIcon engine( "x-office-contact", KIconLoader::global() );
            return engine.pixmap( QSize(16, 16), QIcon::Normal, QIcon::On );
          }
        } else {
          return QVariant();
        }
        break;
        
      case Qt::CheckStateRole:
        if (column == 0) {
          return mChecked ? Qt::Checked : Qt::Unchecked;
        } else {
          return QVariant();
        }
        
      default:
        return QVariant();
    }
    
    return QVariant();
  }
  
  Addressee* addressee() const { 
    return mAddressee; 
  }
  
  uint email() const { 
    return mEmail; 
  }
  
  PersonModelItem* parent() const {
    return mParent; 
  }
  
  int sendingType() const {
    return mSendingType;
  }
  
  void setSendingType( int sendingType ) {
    mSendingType = sendingType;
  }
  
  bool checked() const {
    return mChecked;
  }
  
  void setChecked( int checked ) {
    mChecked = checked;
  }
  
  void setSendingTypeFromString( const QString& str ) {
    if (str == "To") {
      mSendingType = To;
    } else if (str == "CC") {
      mSendingType = CC;
    } else if (str == "BCC") {
      mSendingType = BCC;
    } else {
      mSendingType = None;
    }
  }
  
  QString sendingTypeToString() const {
    switch (mSendingType) {
      case To:
        return "To";
      case CC:
        return "CC";
      case BCC:
        return "BCC";
      default:
        return "";
    }
  }
  
  bool operator==(const PersonModelItem& otherPerson) const {
    return (otherPerson.mAddressee == mAddressee && otherPerson.mEmail == mEmail);
  }   
};
//END

PersonModel::PersonModel( QObject *parent ) 
  : QAbstractItemModel( parent ), mBasicInterface( true )
{
  mRootItem = new PersonModelItem( 0, -1 );
  
  mStdAddressBook = KABC::StdAddressBook::self( true );
  
  {
    using namespace KABC;
    connect( mStdAddressBook, SIGNAL( addressBookChanged(AddressBook*)),
              this, SLOT(addressBookChanged(AddressBook*)) );
  }
}

PersonModel::~PersonModel() {  
  delete mStdAddressBook;  
  delete mRootItem;
}

void PersonModel::addressBookChanged( KABC::AddressBook *addressBook )
{  
  if (addressBook) {        
    mRootItem->clear();
    mCategories.clear();
    
    KABC::AddressBook::Iterator it;
    for (it = addressBook->begin(); it != addressBook->end(); ++it) {
      if ((*it).emails().size() > 0) {
        PersonModelItem *pi = new PersonModelItem( &(*it), 0, mRootItem );
      
        for (int i = 1; i < (*it).emails().size(); ++i) {
          pi->appendChild( new PersonModelItem(&(*it), i, pi) );
        }
        
        mRootItem->appendChild( pi );
        
        // categories
        QStringList categories = (*it).categories();
        foreach (QString category, categories) {
          if (!mCategories.contains( category )) {
            mCategories << category;
          }
        }
      }
    }
  }
  
  qSort( mCategories );
    
  reset();
}

int PersonModel::columnCount( const QModelIndex& parent ) const 
{
  return (parent.column() > 0 ? 0 : 3);
}

QVariant PersonModel::headerData( int section, Qt::Orientation orientation, int role ) const 
{
  if (orientation == Qt::Horizontal) {
    if (role == Qt::DisplayRole) {
      switch (section) {
      case 0:
        return "Name";
      case 1:
        return "Email";
      case 2:
        return "Status";
      default:
	return QVariant();
      }
    } else if (role == Qt::TextAlignmentRole) {
      return Qt::AlignLeft;
    } else {
      return QVariant();
    }
  }
  
  return QAbstractItemModel::headerData( section, orientation, role );
}

int PersonModel::rowCount( const QModelIndex& parent ) const {
  PersonModelItem *parentItem;
  
  if (!parent.isValid()) {
    parentItem = mRootItem;
  } else {
    parentItem = static_cast<PersonModelItem*>( parent.internalPointer() );
  }
  
  return parentItem->childCount();    
}

QVariant PersonModel::data (const QModelIndex& index, int role) const {
  if (!index.isValid()) {
    return QVariant();
  }
  
  if (role == Qt::CheckStateRole && index.column() == 0 && !mBasicInterface) {
    return QVariant();
  }
  
  PersonModelItem *item = static_cast<PersonModelItem*>( index.internalPointer() );
  return item->data( index.column(), role );
}

QModelIndex PersonModel::index( int row, int column, const QModelIndex& parent ) const 
{
  if (row < 0 || row >= mRootItem->childCount()) {
    return QModelIndex();
  }
  
  /** There are three columns but a forth item in the data,
   *  so we can check "hidden" data.
   */
  if (column < 0 || column > (FieldsCount + SpecialFieldsCount)) {
    return QModelIndex();
  }
  
  PersonModelItem *parentItem;
  
  if (!parent.isValid()) {
    parentItem = mRootItem;
  } else {
    parentItem = static_cast<PersonModelItem*>( parent.internalPointer() );
  }
  
  PersonModelItem *childItem = parentItem->child( row );
  if (childItem) {
    return createIndex( row, column, childItem );
  } else {
    return QModelIndex();
  }
}

QModelIndex PersonModel::parent( const QModelIndex& index ) const 
{
  if (!index.isValid()) {
    return QModelIndex();
  }
  
  PersonModelItem *childItem = static_cast<PersonModelItem*>( index.internalPointer() );
  PersonModelItem *parentItem = childItem->parent();
  
  if (parentItem == mRootItem) {
    return QModelIndex();
  }
  
  return createIndex( parentItem->row(), 0, parentItem );
}

bool PersonModel::setData( const QModelIndex& index, const QVariant& value, int role ) 
{
  if (role == Qt::EditRole && index.column() == PersonModel::StatusColumn) {
    PersonModelItem *item = static_cast<PersonModelItem*>( index.internalPointer() );
    if (item) {
      item->setSendingTypeFromString( value.toString() );
     
      emit dataChanged( index, index );
      return true;
    } else {
      return false;
    }
  } else if (role == Qt::CheckStateRole && index.column() == 0) {
    PersonModelItem *item = static_cast<PersonModelItem*>( index.internalPointer() );
    if (item) {
      item->setChecked( !item->checked() );
      
      emit dataChanged( index, index );
      return true;
    } else {
      return false;
    }
  } else {
    return false;
  }
}

Qt::ItemFlags PersonModel::flags( const QModelIndex& index ) const
{
  Qt::ItemFlags flags = QAbstractItemModel::flags( index );
  
  if (index.isValid() && index.column() == 0 && mBasicInterface) {
    flags |= Qt::ItemIsUserCheckable | Qt::ItemIsEditable;
  }
  
  return flags;
}

#include "personmodel.moc"
