/*
    This file is part of KDE playground.
    Copyright (c) 2007 Mathias Soeken <msoeken@tzi.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "persontreeview.h"

#include "personmodel.h"
#include "personfilterproxymodel.h"

#include <QHeaderView>
#include <QMouseEvent>
#include <QPainter>
#include <QToolButton>

#include <KDebug>
#include <KIcon>
#include <KIconLoader>
#include <KLocale>

PersonTreeView::PersonTreeView( QWidget *parent ) : QTreeView( parent ),
  mSelectedIndex( -1 ), mSelectedModelIndex ( QModelIndex() )
{
  setAlternatingRowColors( true );
  setRootIsDecorated( true );
  setSelectionMode( QAbstractItemView::MultiSelection );
  setMouseTracking( true );
  setItemsExpandable( true );
  setSortingEnabled( true );
  
  mPersonModel = new PersonModel( parent );
  
  mProxyModel = new PersonFilterProxyModel( parent );
  mProxyModel->setDynamicSortFilter( true );
  mProxyModel->setFilterCaseSensitivity( Qt::CaseInsensitive );
  mProxyModel->setSourceModel( mPersonModel );
  
  setModel( mProxyModel );
  connect( mProxyModel, SIGNAL(modelReset()), this, SLOT(slotModelReset()) );
  
  resizeColumnToContents( 1 );
  connect( this, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(slotDoubleClicked(const QModelIndex&)) );
  
  header()->setResizeMode( PersonModel::StatusColumn, QHeaderView::Fixed );
  
  mBtnTo = new QToolButton( this );
  mBtnTo->setText( "To" );
  mBtnTo->setCheckable( true );
  mBtnTo->hide();
  connect( mBtnTo, SIGNAL(clicked(bool)), this, SLOT(btnToClicked(bool)) );
  
  mBtnCC = new QToolButton( this );
  mBtnCC->setText( "CC" );
  mBtnCC->setCheckable( true );
  mBtnCC->hide();
  connect( mBtnCC, SIGNAL(clicked(bool)), this, SLOT(btnCCClicked(bool)) );
  
  mBtnBCC = new QToolButton( this );
  mBtnBCC->setText( "BCC" );
  mBtnBCC->setCheckable( true );
  mBtnBCC->hide();
  connect( mBtnBCC, SIGNAL(clicked(bool)), this, SLOT(btnBCCClicked(bool)) );
}

void PersonTreeView::toggleSettings( bool basicInterface ) 
{
  setColumnHidden( PersonModel::StatusColumn, basicInterface );
  
  if (basicInterface) {
    setSelectionMode( QAbstractItemView::MultiSelection );
  } else {
    setSelectionMode( QAbstractItemView::ExtendedSelection );
    setColumnWidth( 1, columnWidth(1) - 120 );
    setColumnWidth( PersonModel::StatusColumn, 120 );
  }
  
  mPersonModel->mBasicInterface = basicInterface;
}

void PersonTreeView::setButtonIndex( const QModelIndex& index ) 
{
  if (index.row() != mSelectedIndex) {
    int oldIndex = mSelectedIndex;
    mSelectedIndex = index.row();
    
    if (mSelectedIndex == -1) {
      mBtnTo->hide();
      mBtnCC->hide();
      mBtnBCC->hide();
    } else if (oldIndex == -1) {
      mBtnTo->show();
      mBtnCC->show();
      mBtnBCC->show();
    }
    
    if (mSelectedIndex != -1) {
      QRect rect = visualRect( index );
      
      int x = rect.x() + frameWidth();
      int y = rect.y() + header()->size().height() + frameWidth();
      int width = rect.width() / 3;
      int height = rect.height();
      
      mBtnTo->setGeometry( QRect(x, y, width, height) );
      mBtnCC->setGeometry( QRect(x + width, y, width, height) );
      mBtnBCC->setGeometry( QRect(x + 2 * width, y, width, height) );
      
      QModelIndex sourceIndex = mProxyModel->mapToSource( index );
      mBtnTo->setChecked( mPersonModel->data(sourceIndex) == "To" );
      mBtnCC->setChecked( mPersonModel->data(sourceIndex) == "CC" );
      mBtnBCC->setChecked( mPersonModel->data(sourceIndex) == "BCC" );
      
      mSelectedModelIndex = sourceIndex;
    } else {
      mSelectedModelIndex = QModelIndex();
    }
  }
}

void PersonTreeView::mouseMoveEvent( QMouseEvent* e ) 
{
  QModelIndex index = indexAt( e->pos() );
  
  if (!index.isValid() || index.column() != PersonModel::StatusColumn) {
    index = index.sibling(0, -1);
  }
  
  setButtonIndex( index );
}

void PersonTreeView::leaveEvent( QEvent* )
{
  setButtonIndex (mPersonModel->index(-1, -1) );
}

void PersonTreeView::keyPressEvent( QKeyEvent* e) 
{
  // if one item is selected, the space bar can be clicked
  // to toggle the status None -> To -> CC -> BCC -> None -> ...
  if (selectionModel()->selectedRows().count() == 1 && e->key() == Qt::Key_Space) {
    QModelIndex index = selectionModel()->selectedRows( PersonModel::StatusColumn )[0];
    QModelIndex sourceIndex = mProxyModel->mapToSource( index );
    
    QString data = mPersonModel->data( sourceIndex ).toString();
    if (data == "") {
      mPersonModel->setData( sourceIndex, "To" );
    } else if (data == "To") {
      mPersonModel->setData( sourceIndex, "CC" );
    } else if (data == "CC") {
      mPersonModel->setData( sourceIndex, "BCC" );
    } else if (data == "BCC") {
      mPersonModel->setData( sourceIndex, "" );
    }
  }
  
  QTreeView::keyPressEvent( e );
}

void PersonTreeView::paintEvent( QPaintEvent* e) 
{
  QTreeView::paintEvent( e );
  
  QPainter p( viewport() );
  
  KIcon icon( "system-users" );
  p.setOpacity( 0.07 );
  p.drawPixmap( width() - frameWidth() - 200,
                height() - header()->height() - frameWidth() - 200,
                icon.pixmap(QSize(196, 196), QIcon::Normal, QIcon::On) );
  p.setOpacity( 1.0 );
  
  // FIXME replace with model()
  if (proxyModel()->rowCount() == 0) {
    int space = 55;
    
    int w = width() - (2 * space) - (2 * frameWidth());
    int h = height() - (2 * space) - header()->height() - (2 * frameWidth());
    
    p.setBrush( palette().brush(QPalette::Background) );
    p.drawRoundRect( space, space, w, h, (8*200)/w, (8*200)/h );
    
    QRect textRect( space + 10, space + 10, w - 20, h - 20 );
    p.drawText( textRect, i18n("You do not have any persons in your address book. You can insert persons with the program kaddressbook, which is also part of the kdepim package."), Qt::AlignCenter | Qt::AlignVCenter );
  }
}

void PersonTreeView::slotModelReset()
{
  resizeColumnToContents( 0 );   
}

void PersonTreeView::slotDoubleClicked( const QModelIndex& index ) 
{
  QModelIndex sourceIndex = mProxyModel->mapToSource( index.sibling(index.row(), PersonModel::StatusColumn) );
  if (index.isValid() && mPersonModel->data(sourceIndex) == "") {
    mPersonModel->setData( sourceIndex, "To" );
  }
}

void PersonTreeView::btnToClicked( bool checked )
{
  if (checked) {
    mBtnCC->setChecked( Qt::Unchecked );
    mBtnBCC->setChecked( Qt::Unchecked );
    mPersonModel->setData( mSelectedModelIndex, "To" );
  } else {
    mPersonModel->setData( mSelectedModelIndex, "" );
  }
}

void PersonTreeView::btnCCClicked( bool checked )
{
  if (checked) {
    mBtnTo->setChecked( Qt::Unchecked );
    mBtnBCC->setChecked( Qt::Unchecked );
    mPersonModel->setData( mSelectedModelIndex, "CC" );
  } else {
    mPersonModel->setData( mSelectedModelIndex, "" );
  }
}

void PersonTreeView::btnBCCClicked( bool checked )
{
  if (checked) {
    mBtnTo->setChecked( Qt::Unchecked );
    mBtnCC->setChecked( Qt::Unchecked );
    mPersonModel->setData( mSelectedModelIndex, "BCC" );
  } else {
    mPersonModel->setData( mSelectedModelIndex, "" );
  }
}

#include "persontreeview.moc"
