/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "clientagent.h"
#include "clientadaptor.h"              // D-Bus adaptor
#include "settings.h"                   // KConfigXT stuff
#include "settingsadaptor.h"            // D-Bus adaptor
#include "settingsdialog.h"
#include "funambol/contactssyncsource.h"
#include "funambol/eventssyncsource.h"
#include "funambol/notessyncsource.h"
#include "funambol/todossyncsource.h"
#include "timetrackingobserver.h"

#include <Akonadi/ChangeRecorder>
#include <Akonadi/CollectionFetchJob>

#include <KWindowSystem>

#include <QDBusConnection>

#include <funambol/common/spds/SyncManagerConfig.h>

using namespace Akonadi;

ClientAgent::ClientAgent(const QString &id)
    : AgentBase(id)
    , m_managerConfig(new SyncManagerConfig), m_observer(new TimeTrackingObserver)
{
    // Setup D-Bus related stuff
    new ClientAdaptor(this);
    new SettingsAdaptor(Settings::self());
    QDBusConnection::sessionBus().registerObject(QLatin1String("/"), this);
    QDBusConnection::sessionBus().registerObject(QLatin1String("/Settings"), Settings::self());

    //TODO: Investigate AgentManager, AgentManagerInstance and friends to
    //      also have a visual change in the AgentInstanceWidget
    setAgentName(Settings::self()->name());

    // Register and setup our time-tracking observer. We also want the change recorder
    // to instantly notify us about changes so that our observer can react accordingly
    // and update it's internal lists of added/changed/deleted items.
    registerObserver(m_observer);
    connect(changeRecorder(), SIGNAL(changesAdded()), changeRecorder(), SLOT(replayNext()));

    // Tear up all the fancy Funambol stuff
    init();
}

ClientAgent::~ClientAgent()
{
    // Unregister D-Bus objects
    QDBusConnection::sessionBus().unregisterObject("/", QDBusConnection::UnregisterTree);
    QDBusConnection::sessionBus().unregisterObject("/Settings", QDBusConnection::UnregisterTree);

    // Make sure that all custom settings are also saved
    Settings::self()->writeConfig();

    delete m_managerConfig;
    delete m_observer;
}

void ClientAgent::configure(WId windowId)
{
    SettingsDialog dialog;

    if (windowId) {
        KWindowSystem::setMainWindow(&dialog, windowId);
    }
    if (dialog.exec() == QDialog::Accepted) {
        init();
    } else {
        if (Settings::self()->firstRun()) {
            const QString message = i18nc("@info:status", "Please configure the agent");
            emit status(Broken, message);
        }
    }
    Settings::self()->setFirstRun(false);
}

bool ClientAgent::sync()
{
    int index = 0;
    QString message;

    message = i18nc("@info:status", "Sync started");
    kDebug() << message;
    emit status(Running, message);

    // Funambol want's to have a null-terminated array of sync sources
    SyncSource *sources[] = {NULL, NULL, NULL, NULL, 0};

    // Add all sync sources that where enabled by the user in the config screen
    ContactsSyncSource *contactsSyncSource = NULL;
    if (Settings::self()->contactsEnabled()) {
        ContactsSyncSourceConfig *contactsConfig = new ContactsSyncSourceConfig;
        contactsSyncSource = new ContactsSyncSource(m_observer, contactsConfig, m_managerConfig);
        sources[index++] = contactsSyncSource;
        kDebug() << "Use contacts sync source";
    }
    EventsSyncSource *eventsSyncSource = NULL;
    if (Settings::self()->eventsEnabled()) {
        EventsSyncSourceConfig *eventsConfig = new EventsSyncSourceConfig;
        eventsSyncSource = new EventsSyncSource(m_observer, eventsConfig, m_managerConfig);
        sources[index++] = eventsSyncSource;
        kDebug() << "Use events sync source";
    }
    TodosSyncSource *todosSyncSource = NULL;
    if (Settings::self()->todosEnabled()) {
        TodosSyncSourceConfig *todosConfig = new TodosSyncSourceConfig;
        todosSyncSource = new TodosSyncSource(m_observer, todosConfig, m_managerConfig);
        sources[index++] = todosSyncSource;
        kDebug() << "Use todos sync source";
    }
    NotesSyncSource *notesSyncSource = NULL;
    if (Settings::self()->notesEnabled()) {
        NotesSyncSourceConfig *notesConfig = new NotesSyncSourceConfig;
        notesSyncSource = new NotesSyncSource(m_observer, notesConfig, m_managerConfig);
        sources[index++] = notesSyncSource;
        kDebug() << "Use notes sync source";
    }

    // Note: this can take some time depending on network speed and exchanged data
    if (SyncClient::sync(*m_managerConfig, sources) == 0) {
        if (Settings::self()->contactsEnabled()) {
            Settings::self()->setContactsLastSyncTime(contactsSyncSource->lastSyncTime());
        }
        if (Settings::self()->eventsEnabled()) {
            Settings::self()->setEventsLastSyncTime(eventsSyncSource->lastSyncTime());
        }
        if (Settings::self()->todosEnabled()) {
            Settings::self()->setTodosLastSyncTime(todosSyncSource->lastSyncTime());
        }
        if (Settings::self()->notesEnabled()) {
            Settings::self()->setNotesLastSyncTime(notesSyncSource->lastSyncTime());
        }
        Settings::self()->writeConfig();

        StringBuffer results;
        SyncClient::getSyncReport()->toString(results);
        kDebug() << "\n" << results;

        message = i18nc("@info:status", "Sync finished");
        kDebug() << message;
        emit status(Idle, message);
        return true;
    } else {
        StringBuffer results;
        SyncClient::getSyncReport()->toString(results);
        kDebug() << "\n" << results;

        message = i18nc("@info:status", "Sync failed (%1)", getSyncReport()->getLastErrorMsg());
        kDebug() << message;
        emit status(Idle, message);
        return false;
    }
}

QString ClientAgent::lastResults()
{
    QString str;

    for (unsigned int i = 0; getSyncReport()->getSyncSourceReport(i); i++) {
        SyncSourceReport *ssr = getSyncReport()->getSyncSourceReport(i);
        if (ssr->getState() == SOURCE_INACTIVE)
            continue;

        str += "Results for " + QString(ssr->getSourceName()) + ":\n";
        str += "  client   server\n";
        str += "  " + QString::number(ssr->getItemReportSuccessfulCount(CLIENT, COMMAND_ADD)) +
               " / " + QString::number(ssr->getItemReportCount(CLIENT, COMMAND_ADD)) + " | ";
        str += "  " + QString::number(ssr->getItemReportSuccessfulCount(SERVER, COMMAND_ADD)) +
               " / " + QString::number(ssr->getItemReportCount(SERVER, COMMAND_ADD)) + tr(" new\n");
        str += "  " + QString::number(ssr->getItemReportSuccessfulCount(CLIENT, COMMAND_REPLACE)) +
               " / " + QString::number(ssr->getItemReportCount(CLIENT, COMMAND_REPLACE)) + " | ";
        str += "  " + QString::number(ssr->getItemReportSuccessfulCount(SERVER, COMMAND_REPLACE)) +
               " / " + QString::number(ssr->getItemReportCount(SERVER, COMMAND_REPLACE)) + tr(" modified\n");
        str += "  " + QString::number(ssr->getItemReportSuccessfulCount(CLIENT, COMMAND_DELETE)) +
               " / " + QString::number(ssr->getItemReportCount(CLIENT, COMMAND_DELETE)) + " | ";
        str += "  " + QString::number(ssr->getItemReportSuccessfulCount(SERVER, COMMAND_DELETE)) +
               " / " + QString::number(ssr->getItemReportCount(SERVER, COMMAND_DELETE)) + tr(" deleted\n\n");
    }
    return str;
}

void ClientAgent::init()
{
    // Every SyncML clients needs a globally unique identifier.
    QUuid id = QUuid(Settings::self()->uuid());
    if (id.isNull()) {
        id = QUuid::createUuid();
        kDebug() << "Created UUID:" << id;
        Settings::self()->setUuid(id.toString());
    }

    kDebug() << "Initializing...";

    AccessConfig* ac = new AccessConfig();
    ac->setUsername(Settings::self()->transportUsername().toLatin1());
    ac->setPassword(Settings::self()->transportPassword().toLatin1());
    ac->setFirstTimeSyncMode(SYNC_SLOW);
    if (Settings::self()->useProxy()) {
        kDebug() << "Use proxy host" << Settings::self()->proxyHost();
    }
    ac->setUseProxy(Settings::self()->useProxy());
    ac->setProxyHost(Settings::self()->proxyHost().toLatin1());
    ac->setProxyPort(Settings::self()->proxyPort());
    ac->setProxyUsername(Settings::self()->proxyUsername().toLatin1());
    ac->setProxyPassword(Settings::self()->proxyPassword().toLatin1());
    kDebug() << "Use server URL" << Settings::self()->transportUrl();
    ac->setSyncURL(Settings::self()->transportUrl().toLatin1());
    ac->setBeginSync(0);
    ac->setEndSync(0);
    ac->setServerAuthRequired(FALSE);
    ac->setClientAuthType("syncml:auth-basic");
    ac->setServerAuthType("syncml:auth-basic");
    ac->setServerPWD("funambol");
    kDebug() << "Use server identifier" << Settings::self()->serverIdentifier();
    ac->setServerID(Settings::self()->serverIdentifier().toLatin1());
    ac->setServerNonce("");
    ac->setClientNonce("");
    ac->setMaxMsgSize(10000);
    ac->setReadBufferSize(0);
    ac->setUserAgent("");
    //ac->setUserAgent("Akonadi SyncML client agent");
    ac->setCheckConn(TRUE);
    ac->setResponseTimeout(0);
    m_managerConfig->setAccessConfig(*ac);
    delete ac;

    DeviceConfig* dc = new DeviceConfig();
    switch (Settings::self()->syncMLVersion()) {
        case 0:
            kDebug() << "Use SyncML version 1.2";
            dc->setVerDTD("1.2");
            break;
        case 1:
            kDebug() << "Use SyncML version 1.1";
            dc->setVerDTD("1.1");
            break;
        case 2:
            kDebug() << "Use SyncML version 1.0";
            dc->setVerDTD("1.0");
            break;
    }
    dc->setMan("KDE");                                          // Manufacturer
    dc->setMod("Akonadi");
    dc->setOem("KDE");                                          // OEM
    dc->setFwv("");                                             // Firmware version
    dc->setSwv("");                                             // Software version
    dc->setHwv("");                                             // Hardware version
    kDebug() << "Use client UUID" << Settings::self()->uuid().toLatin1();
    dc->setDevID(Settings::self()->uuid().toLatin1());          // This should be a GUID
    dc->setDevType("workstation");                              // Or pda,handheld,server,workstation,pager,phone
    dc->setDsV("");
    if (Settings::self()->useLocalTime()) {
        kDebug() << "Use local time";
    } else {
        kDebug() << "Use UTC";
    }
    dc->setUtc(!Settings::self()->useLocalTime());              // UTC support
    dc->setLoSupport(Settings::self()->supportLargeObjects());  // Large object support
    dc->setNocSupport(FALSE);
    dc->setLogLevel(LOG_LEVEL_NONE);
    //dc->setLogLevel(LOG_LEVEL_DEBUG);
    dc->setMaxObjSize(0);
    dc->setDevInfHash("");
    m_managerConfig->setDeviceConfig(*dc);
    delete dc;

    // Unset all previously monitored collections
    foreach (const Collection &collection, changeRecorder()->collectionsMonitored()) {
        changeRecorder()->setCollectionMonitored(collection, false);
    }

    // Set new monitored collections. It's easier here to fetch all collections and
    // to see which ones we need actually.
    CollectionFetchJob *job = new CollectionFetchJob(Collection::root(), CollectionFetchJob::Recursive);
    if (job->exec()) {
        foreach (const Collection &collection, job->collections()) {
            if (Settings::self()->contactsEnabled() &&
                collection.id() == Settings::self()->contactsCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
            if (Settings::self()->eventsEnabled() &&
                collection.id() == Settings::self()->eventsCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
            if (Settings::self()->todosEnabled() &&
                collection.id() == Settings::self()->todosCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
            if (Settings::self()->notesEnabled() &&
                collection.id() == Settings::self()->notesCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
        }
    }

    kDebug() << "Monitored collections:";
    foreach (const Collection &collection, changeRecorder()->collectionsMonitored()) {
        kDebug() << "\tid:" << collection.id() << "name:" << collection.name();
    }

    if (!Settings::self()->firstRun()) {
        const QString message = i18nc("@info:status", "Ready");
        emit status(Idle, message);
    }
}

AKONADI_AGENT_MAIN(ClientAgent)

#include "clientagent.moc"
