/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "timetrackingobserver.h"

#include <Akonadi/Collection>
#include <Akonadi/ItemFetchJob>

#include <QDateTime>

using namespace Akonadi;

Akonadi::Item::List TimeTrackingObserver::allItems(const QDateTime &timestamp, Entity::Id collectionId)
{
    if (!m_allItemsFetchedCollectionIds.contains(collectionId)) {
        kDebug() << "Try to initially fetch all items for collection" << collectionId;
        ItemFetchJob *fetchJob = new ItemFetchJob(Collection(collectionId));
        if (fetchJob->exec()) {
            foreach (const Item &item, fetchJob->items()) {
                m_allItems.insert(item.id(), item);
            }
            m_allItemsFetchedCollectionIds.append(collectionId);
            kDebug() << "Fetched" << fetchJob->items().size() << "items";
        } else {
            kDebug() << "Unable to initially fetch all items for collection" << collectionId;
        }
    }
    return items(m_allItems, timestamp, collectionId);
}

Akonadi::Item::List TimeTrackingObserver::addedItems(const QDateTime &timestamp, Entity::Id collectionId)
{
    return items(m_addedItems, timestamp, collectionId);
}

Akonadi::Item::List TimeTrackingObserver::changedItems(const QDateTime &timestamp, Entity::Id collectionId)
{
    return items(m_changedItems, timestamp, collectionId);
}

Akonadi::Item::List TimeTrackingObserver::removedItems(const QDateTime &timestamp, Entity::Id collectionId)
{
    return items(m_removedItems, timestamp, collectionId);;
}

void TimeTrackingObserver::itemAdded(const Item &item, const Collection &collection)
{
    kDebug() << "The item" << item.id() << "was added to collection" << collection.id();
    m_allItems.insert(item.id(), item);
    m_addedItems.insert(item.id(), item);

    AgentBase::Observer::itemAdded(item, collection);
}

void TimeTrackingObserver::itemChanged(const Item &item, const QSet<QByteArray> &partIdentifiers)
{
    //Q_UNUSED(partIdentifiers);

    kDebug() << "The item" << item.id() << "was changed in these parts:" << partIdentifiers;
    if (m_addedItems.contains(item.id())) {
        m_addedItems.remove(item.id());
    }
    m_changedItems.insert(item.id(), item);

    AgentBase::Observer::itemChanged(item, partIdentifiers);
}

void TimeTrackingObserver::itemRemoved(const Item &item)
{
    kDebug() << "The item" << item.id() << "was removed";
    if (m_addedItems.contains(item.id())) {
        m_addedItems.remove(item.id());
    }
    if (m_changedItems.contains(item.id())) {
        m_changedItems.remove(item.id());
    }
    m_removedItems.insert(item.id(), item);

    AgentBase::Observer::itemRemoved(item);
}

Akonadi::Item::List TimeTrackingObserver::items(const QHash<Entity::Id, Item> &items, const QDateTime &timestamp, Entity::Id collectionId)
{
    Item::List list;

    foreach (const Item &item, items) {
        if (item.modificationTime() >= timestamp) {
            if (collectionId >= 0) {
                if (collectionId == item.storageCollectionId()) {
                    list.append(item);
                }
            } else {
                list.append(item);
            }
        }
    }
    return list;
}

#include "moc_timetrackingobserver.cpp"
