/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "settingsdialog.h"
#include "settings.h"

#include <Akonadi/Collection>
#include <Akonadi/CollectionFetchJob>

#include <KConfigDialogManager>

using namespace Akonadi;

SettingsDialog::SettingsDialog(QWidget *parent)
    : KDialog(parent)
{
    m_ui.setupUi(mainWidget());
    setCaption(Settings::self()->name());

    m_manager = new KConfigDialogManager(this, Settings::self());
    m_manager->updateWidgets();

    // Set reasonable defaults when the client agent is run the first time
    // and the config dialog is shown. Otherwise use the user-configured values.
    if (Settings::self()->firstRun()) {
        kDebug() << "Try to set some default local collections";
        CollectionFetchJob *job = new CollectionFetchJob(Collection::root(), CollectionFetchJob::Recursive);
        if (job->exec()) {
            Collection::List collections = job->collections();
            foreach (const Collection &collection, collections) {
                if (collection.name() == "akonadi_vcard_resource_0") {
                    kDebug() << "Set default contacts collection";
                    m_ui.contactsRequester->setCollection(collection);
                } else if (collection.name() == "akonadi_ical_resource_0") {
                    kDebug() << "Set default events and todos collection";
                    m_ui.eventsRequester->setCollection(collection);
                    m_ui.todosRequester->setCollection(collection);
                } else if (collection.name() == "akonadi_notes_resource_0") {
                    kDebug() << "Set default notes collection";
                    m_ui.notesRequester->setCollection(collection);
                }
            }
        }
    } else {
        // Fetch all collections recursively, starting at the root collection
        CollectionFetchJob *job = new CollectionFetchJob(Collection::root(), CollectionFetchJob::Recursive);
        if (job->exec()) {
            Collection::List collections = job->collections();
            foreach (const Collection &collection, collections) {
                if (collection.id() == Settings::self()->contactsCollectionId()) {
                    m_ui.contactsRequester->setCollection(collection);
                } else if (collection.id() == Settings::self()->eventsCollectionId()) {
                    m_ui.eventsRequester->setCollection(collection);
                } else if (collection.id() == Settings::self()->todosCollectionId()) {
                    m_ui.todosRequester->setCollection(collection);
                } else if (collection.id() == Settings::self()->notesCollectionId()) {
                    m_ui.notesRequester->setCollection(collection);
                }
            }
        }
    }

    m_ui.contactsRequester->setMimeTypeFilter(QStringList() << "text/directory");
    m_ui.eventsRequester->setMimeTypeFilter(QStringList() << "application/x-vnd.akonadi.calendar.event");
    m_ui.todosRequester->setMimeTypeFilter(QStringList() << "application/x-vnd.akonadi.calendar.todo");
    m_ui.notesRequester->setMimeTypeFilter(QStringList() << "application/x-vnd.kde.notes");
}

SettingsDialog::~SettingsDialog()
{
    delete m_manager;
}

void SettingsDialog::accept()
{
    m_manager->updateSettings();

    // Check and save contacts settings
    Collection collection = m_ui.contactsRequester->collection();
    if (m_ui.kcfg_ContactsEnabled->isChecked() && (!collection.isValid() || m_ui.kcfg_ContactsRemoteDatabaseName->text().isEmpty())) {
        kDebug() << "Contacts disabled because of invalid settings";
        Settings::self()->setContactsEnabled(false);
    } else {
        Settings::self()->setContactsCollectionId(collection.id());
        Settings::self()->setContactsCollectionName(collection.name());
    }

    // Check and save events settings
    collection = m_ui.eventsRequester->collection();
    if (m_ui.kcfg_EventsEnabled->isChecked() && (!collection.isValid() || m_ui.kcfg_EventsRemoteDatabaseName->text().isEmpty())) {
        kDebug() << "Events disabled because of invalid settings";
        Settings::self()->setEventsEnabled(false);
    } else {
        Settings::self()->setEventsCollectionId(collection.id());
        Settings::self()->setEventsCollectionName(collection.name());
    }

    // Check and save todos settings
    collection = m_ui.todosRequester->collection();
    if (m_ui.kcfg_TodosEnabled->isChecked() && (!collection.isValid() || m_ui.kcfg_TodosRemoteDatabaseName->text().isEmpty())) {
        kDebug() << "Todos disabled because of invalid settings";
        Settings::self()->setTodosEnabled(false);
    } else {
        Settings::self()->setTodosCollectionId(collection.id());
        Settings::self()->setTodosCollectionName(collection.name());
    }

    // Check and save notes settings
    collection = m_ui.notesRequester->collection();
    if (m_ui.kcfg_NotesEnabled->isChecked() && (!collection.isValid() || m_ui.kcfg_NotesRemoteDatabaseName->text().isEmpty())) {
        kDebug() << "Notes disabled because of invalid settings";
        Settings::self()->setNotesEnabled(false);
    } else {
        Settings::self()->setNotesCollectionId(collection.id());
        Settings::self()->setNotesCollectionName(collection.name());
    }

    // Save settings and return from dialog
    Settings::self()->writeConfig();
    KDialog::accept();
}

#include "settingsdialog.moc"
