/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "kcmsyncmlclient.h"
#include "settings.h"                        // KConfigXT stuff
#include "clientsettingsinterface.h"         // D-Bus interface

#include <Akonadi/AgentInstance>
#include <Akonadi/AgentInstanceCreateJob>
#include <Akonadi/AgentManager>

#include <knewstuff2/engine.h>

#include <KAboutData>
#include <KLocale>
#include <KPluginFactory>
#include <KPluginLoader>

#include <QInputDialog>

K_PLUGIN_FACTORY(KCMSyncMLClientFactory, registerPlugin<KCMSyncMLClient>();)
K_EXPORT_PLUGIN(KCMSyncMLClientFactory("kcmsyncmlclient"))

KCMSyncMLClient::KCMSyncMLClient(QWidget *parent, const QVariantList &args)
    : KCModule(KCMSyncMLClientFactory::componentData(), parent, args)
{
    KAboutData *about = new KAboutData("kcmsyncmlclient", 0, ki18n("SyncML Remote Access"),
            KDE_VERSION_STRING, KLocalizedString(), KAboutData::License_GPL,
            ki18n("Copyright (c) 2009 Sascha Peilicke"));
    about->addAuthor(ki18n("Sascha Peilicke (saschpe)"), ki18n("Original author"),
                     "sasch.pe@gmx.de", "http://saschpe.wordpress.com");
    setAboutData(about);
    setButtons(KCModule::Help);

    KGlobal::locale()->insertCatalog("kcmsyncml");
    KGlobal::locale()->insertCatalog("libakonadi");

    // Setup user interface stuff
    m_ui.setupUi(this);
    m_ui.profileSearchLine->setListWidget(m_ui.profileListWidget);
    m_ui.addProfileButton->setIcon(KIcon("list-add"));
    m_ui.removeProfileButton->setIcon(KIcon("list-remove"));
    m_ui.renameProfileButton->setIcon(KIcon("edit-rename"));
    m_ui.syncButton->setIcon(KIcon("media-playback-start"));
    m_ui.getNewProfilesButton->setIcon(KIcon("get-hot-new-stuff"));
    m_ui.configureAgentButton->setIcon(KIcon("configure"));

    populateProfileTypeComboBox();

    connect(m_ui.profileListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
    connect(m_ui.profileListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(itemDoubleClicked(QListWidgetItem *)));
    connect(m_ui.profileListWidget, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem*)), this, SLOT(currentItemChanged(QListWidgetItem *, QListWidgetItem *)));

}

KCMSyncMLClient::~KCMSyncMLClient()
{
}

void KCMSyncMLClient::load()
{
    kDebug() << "Load settings";

    //TODO: setup profile stuff
}

void KCMSyncMLClient::save()
{
    kDebug() << "Save settings";

    //TODO: Specific stuff here

    Settings::self()->writeConfig();
}

void KCMSyncMLClient::defaults()
{
    kDebug() << "Restore default settings";
    Settings::self()->setDefaults();
}

void KCMSyncMLClient::on_addProfileButton_clicked()
{
    bool ok;
    QString text = QInputDialog::getText(this, i18n("Profile Name"),
                                         i18n("Profile name:"), QLineEdit::Normal,
                                         i18n("New profile"), &ok);
    if (ok && !text.isEmpty()) {
        QListWidgetItem *item = new QListWidgetItem();
        item->setText(text);
        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsEnabled);

        m_ui.profileListWidget->addItem(item);
        m_ui.profileListWidget->setCurrentItem(item);

        // Create a new KConfig group to store all related settings
        KConfigGroup group = Settings::self()->config()->group("profile-" + item->text());
        //TODO: Group not landed in settings
    }
}

void KCMSyncMLClient::on_renameProfileButton_clicked()
{
    m_ui.profileListWidget->editItem(m_ui.profileListWidget->currentItem());
}

void KCMSyncMLClient::on_removeProfileButton_clicked()
{
    QListWidgetItem *item = m_ui.profileListWidget->currentItem();

    if (item) {
        m_ui.profileListWidget->removeItemWidget(item);

        //TODO: Do D-Bus magic
        Settings::self()->config()->deleteGroup("profile-" + item->text());
    }
}

void KCMSyncMLClient::itemChanged(QListWidgetItem *item)
{
    kDebug() << "Current item text changed from" << m_lastTextBeforeEdit << "to" << item->text();

    // Fetch old config group and rename it, if it exists
    if (Settings::self()->config()->hasGroup("profile-" + m_lastTextBeforeEdit)) {
        kDebug() << "Rename previously existing config group";
        //TODO change config group only if config group previously existed
        // this is also called for new listwidgetitems
    }
}

void KCMSyncMLClient::itemDoubleClicked(QListWidgetItem *)
{
    m_lastTextBeforeEdit = m_ui.profileListWidget->currentItem()->text();
}

void KCMSyncMLClient::currentItemChanged(QListWidgetItem *current, QListWidgetItem *previous)
{
    if (current) {
        if (previous) {
            kDebug() << "Current item selection changed from" << previous->text() << "to" << current->text();
        }
        //TODO: Load corresponding conf values
        //KConfigGroup oldGroup = Settings::self()->config()->group("profile-" + item->text());
    } else {
    }
}

void KCMSyncMLClient::on_getNewProfilesButton_clicked()
{
    KNS::Engine engine(0);

    if (engine.init("kcmsyncmlclient.knsrc")) {
        KNS::Entry::List entries = engine.downloadDialogModal(this);
        if (entries.size() > 0) {
            // do something with the modified entries here if you want
            // such as rescaning your data folder or whatnot
            populateProfileTypeComboBox();
        }
    }
}

void KCMSyncMLClient::on_configureAgentButton_clicked()
{
    kDebug() << "Configure agent ...";
}

void KCMSyncMLClient::on_syncButton_clicked()
{
    kDebug() << "Sync ...";
}

void KCMSyncMLClient::populateProfileTypeComboBox()
{
    kDebug() << "Populate profile type combobox";
}
