/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "kcmsyncmlserver.h"
#include "settings.h"                       // KConfigXT stuff
#include "serversettingsinterface.h"        // D-Bus interface

#include <Akonadi/AgentInstance>
#include <Akonadi/AgentInstanceCreateJob>
#include <Akonadi/AgentManager>

#include <KAboutData>
#include <KConfigDialogManager>
#include <KLocale>
#include <KPluginFactory>
#include <KPluginLoader>
#include <KStandardGuiItem>

using namespace Akonadi;

K_PLUGIN_FACTORY(KCMSyncMLServerFactory, registerPlugin<KCMSyncMLServer>();)
K_EXPORT_PLUGIN(KCMSyncMLServerFactory("kcmsyncmlserver"))

KCMSyncMLServer::KCMSyncMLServer(QWidget *parent, const QVariantList &args)
    : KCModule(KCMSyncMLServerFactory::componentData(), parent, args)
{
    KAboutData *about = new KAboutData("kcmsyncmlserver", 0, ki18n("SyncML Server"),
            KDE_VERSION_STRING, KLocalizedString(), KAboutData::License_GPL,
            ki18n("Copyright (c) 2009 Sascha Peilicke"));
    about->addAuthor(ki18n("Sascha Peilicke (saschpe)"), ki18n("Original author"),
                     "sasch.pe@gmx.de", "http://saschpe.wordpress.com");
    setAboutData(about);

    // Insert i18n catalogs
    KGlobal::locale()->insertCatalog("kcmsyncml");
    KGlobal::locale()->insertCatalog("libakonadi");

    // Set standard gui item stuff for configure buttons
    m_ui.setupUi(this);
    KGuiItem item = KStandardGuiItem::configure();
    m_ui.httpConfigButton->setIcon(item.icon());
    m_ui.httpConfigButton->setToolTip(item.toolTip());
    m_ui.obexConfigButton->setIcon(item.icon());
    m_ui.obexConfigButton->setToolTip(item.toolTip());

    m_manager = new KConfigDialogManager(this, Settings::self());

    connect(m_ui.kcfg_HttpServerAgentEnabled, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(m_ui.kcfg_ObexServerAgentEnabled, SIGNAL(toggled(bool)), SLOT(changed()));


    //TODO: Implement corresponding display stuff!
    m_ui.httpDeviceSettingsGroupBox->setVisible(false);
    m_ui.obexDeviceSettingsGroupBox->setVisible(false);
}

KCMSyncMLServer::~KCMSyncMLServer()
{
    delete m_manager;
}

void KCMSyncMLServer::load()
{
    kDebug() << "Load settings";

    if (Settings::self()->httpServerAgentEnabled()) {
        AgentInstance httpInstance = AgentManager::self()->instance(Settings::self()->httpServerAgentIdentifier());
        if (httpInstance.isValid()) {
            m_ui.httpConfigButton->setEnabled(true);
            m_ui.httpDeviceSettingsGroupBox->setEnabled(true);
        } else {
            kDebug() << "The HTTP server agents seems to be gone.";
            Settings::self()->setHttpServerAgentEnabled(false);
            Settings::self()->setHttpServerAgentIdentifier("");
            Settings::self()->writeConfig();
        }
    }

    if (Settings::self()->obexServerAgentEnabled()) {
        AgentInstance obexInstance = AgentManager::self()->instance(Settings::self()->obexServerAgentIdentifier());
        if (obexInstance.isValid()) {
            m_ui.obexConfigButton->setEnabled(true);
            m_ui.obexDeviceSettingsGroupBox->setEnabled(true);
        } else {
            kDebug() << "The OBEX server agents seems to be gone.";
            Settings::self()->setObexServerAgentEnabled(false);
            Settings::self()->setObexServerAgentIdentifier("");
            Settings::self()->writeConfig();
        }
    }

    m_manager->updateWidgets();
}

void KCMSyncMLServer::save()
{
    kDebug() << "Save settings";
    AgentType type = Akonadi::AgentManager::self()->type("akonadi_syncmlserver_agent");
    m_manager->updateSettings();

    // Check if we shall create and initialize or remove the HTTP server agent
    if (Settings::self()->httpServerAgentEnabled()) {
        kDebug() << "User enabled HTTP server agent";
        if (Settings::self()->httpServerAgentIdentifier().isEmpty()) {
            kDebug() << "Create HTTP server agent instance";
            AgentInstanceCreateJob *createJob = new AgentInstanceCreateJob(type);
            if (createJob->exec()) {
                Settings::self()->setHttpServerAgentIdentifier(createJob->instance().identifier());

                QString serviceString = "org.freedesktop.Akonadi.Agent." + Settings::self()->httpServerAgentIdentifier();
                org::kde::Akonadi::SyncML::Server::Settings interface(serviceString,
                                                                      QLatin1String("/Settings"),
                                                                      QDBusConnection::sessionBus(), this);
                interface.setName(QLatin1String("KCM controlled SyncML HTTP server agent"));
                interface.setPort(9100);

                /*QDBusPendingReply<QString> usernameReply = interface.username();
                usernameReply.waitForFinished();
                if (!usernameReply.isError()) {
                    m_ui.httpUsername->setText(usernameReply.value());
                }

                QDBusPendingReply<QString> passwordReply = interface.password();
                passwordReply.waitForFinished();
                if (!passwordReply.isError()) {
                    m_ui.httpPassword->setText(passwordReply.value());
                }

                QDBusPendingReply<QStringList> resourcesReply = interface.monitoredResourceNames();*/

                m_ui.httpConfigButton->setEnabled(true);
                m_ui.httpDeviceSettingsGroupBox->setEnabled(true);
            } else {
                kDebug() << "Unable to create new HTTP server agent instance";
                Settings::self()->setHttpServerAgentEnabled(false);
            }
        }
    } else {
        kDebug() << "User disabled HTTP server agent";
        AgentInstance instance = AgentManager::self()->instance(Settings::self()->httpServerAgentIdentifier());
        if (instance.isValid()) {
            kDebug() << "Remove HTTP server agent instance";
            AgentManager::self()->removeInstance(instance);
        }
        Settings::self()->setHttpServerAgentIdentifier("");
        m_ui.httpConfigButton->setEnabled(false);
        m_ui.httpDeviceSettingsGroupBox->setEnabled(false);
    }

    // Check if we shall create and initialize or remove the OBEX server agent
    if (Settings::self()->obexServerAgentEnabled()) {
        kDebug() << "User enabled OBEX server agent";
        if (Settings::self()->obexServerAgentIdentifier().isEmpty()) {
            kDebug() << "Create OBEX server agent instance";
            AgentInstanceCreateJob *createJob = new AgentInstanceCreateJob(type);
            if (createJob->exec()) {
                Settings::self()->setObexServerAgentIdentifier(createJob->instance().identifier());

                QString serviceString = "org.freedesktop.Akonadi.Agent." + Settings::self()->obexServerAgentIdentifier();
                org::kde::Akonadi::SyncML::Server::Settings interface(serviceString,
                                                                      QLatin1String("/Settings"),
                                                                      QDBusConnection::sessionBus(), this);
                interface.setName(QLatin1String("KCM controlled SyncML OBEX server agent"));
                interface.setPort(9101);
                interface.setTransport(1);  // A little ugly
                m_ui.obexConfigButton->setEnabled(true);
                m_ui.obexDeviceSettingsGroupBox->setEnabled(true);
            } else {
                kDebug() << "Unable to create new OBEX server agent instance";
                Settings::self()->setObexServerAgentEnabled(false);
            }
        }
    } else {
        kDebug() << "User disabled OBEX server agent";
        AgentInstance instance = AgentManager::self()->instance(Settings::self()->obexServerAgentIdentifier());
        if (instance.isValid()) {
            kDebug() << "Remove OBEX server agent instance";
            AgentManager::self()->removeInstance(instance);
        }
        Settings::self()->setObexServerAgentIdentifier("");
        m_ui.obexConfigButton->setEnabled(false);
        m_ui.obexDeviceSettingsGroupBox->setEnabled(false);
    }

    m_manager->updateWidgets();
    Settings::self()->writeConfig();
}

void KCMSyncMLServer::defaults()
{
    kDebug() << "Restore default settings";
    m_manager->updateWidgetsDefault();
}

void KCMSyncMLServer::remoteSettingsChanged(const QString identifier)
{
    kDebug() << "Remote settings changed for" << identifier;
    //TODO: This slot should respond to a signal sent over D-Bus

    QString serviceString = "org.freedesktop.Akonadi.Agent." + identifier;
    org::kde::Akonadi::SyncML::Server::Settings interface(serviceString,
                                                          QLatin1String("/Settings"),
                                                          QDBusConnection::sessionBus(), this);

    QDBusPendingReply<QString> usernameReply = interface.username();
    usernameReply.waitForFinished();
    if (!usernameReply.isError()) {
        m_ui.httpUsername->setText(usernameReply.value());
    }

    QDBusPendingReply<QString> passwordReply = interface.password();
    passwordReply.waitForFinished();
    if (!passwordReply.isError()) {
        m_ui.httpPassword->setText(passwordReply.value());
    }

    QDBusPendingReply<QStringList> resourcesReply = interface.monitoredResourceNames();
    resourcesReply.waitForFinished();
    if (!resourcesReply.isError()) {
        //TODO: Fill resource table
    }
}

void KCMSyncMLServer::on_httpConfigButton_clicked()
{
    AgentInstance instance = AgentManager::self()->instance(Settings::self()->httpServerAgentIdentifier());
    if (instance.isValid()) {
        instance.configure(this);
    }
}

void KCMSyncMLServer::on_obexConfigButton_clicked()
{
    AgentInstance instance = AgentManager::self()->instance(Settings::self()->obexServerAgentIdentifier());
    if (instance.isValid()) {
        instance.configure(this);
    }
}
