/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "datastore.h"
#include "settings.h"

#include <Akonadi/AgentManager>
#include <Akonadi/ResourceBase>

#include <QStringList>

using namespace Akonadi;

DataStore::DataStore(const QString &resourceName, QObject *parent)
    : QObject(parent), m_isValid(false)
{
    // Allowed are the following parameter combinations for smlDataSyncAddDatastore
    // (according to libsyncml source code):
    //
    //  content-type     | source    | meaning | Akonadi mime-type
    //  ----------------------------------------------------------
    //  text/x-vcard     |           | contact | text/directory
    //  text/x-vcalendar | cal,event | event   | text/calendar,application/x-vnd.akonadi.calendar.event
    //  text/x-vcalendar | todo      | todo    | text/calendar,application/x-vnd.akonadi.calendar.todo
    //  text/plain       |           | note    | application/x-vnd.kde.notes
    //
    // So we have to match that to the MimeTypes of supported Akonadi resources
    // to make sure they are registered correctly as libsyncml datastores.

    // Get some info from the AgentInstance corresponding to "resourceName" and
    // save that in our DataStore class
    foreach (const AgentInstance &instance, AgentManager::self()->instances()) {
        if (instance.name() == resourceName) {
            m_source = instance.identifier();
            m_target = m_source;
            foreach (const QString &mimeType, instance.type().mimeTypes()) {
                if (mimeType == "text/directory") {
                    m_contentType = "text/x-vcard";
                    break;
                } else if (mimeType == "text/calendar") {
                    m_contentType = "text/x-vcalendar";
                    break;
                } else if (mimeType == "application/x-vnd.kde.notes") {
                    m_contentType = "text/plain";
                    break;
                }
            }
            m_isValid = true;
        }
    }

    // Whether we have to do a full sync is determined by server configuration here.
    // The remote client may also change this if needed.
    m_useSlowSync = Settings::self()->useSlowSync();

    // Load other datastore-specific settings from our KConfig file
    Settings::self()->setCurrentGroup(m_source);
    Settings::self()->addItemDateTime("LastSyncTime", m_lastSyncTime, QDateTime());
    Settings::self()->addItemBool("LastSyncSuccessfull", m_lastSyncSuccessfull, false);
}

void DataStore::setUseSlowSync(bool useSlowSync)
{
    m_useSlowSync = useSlowSync;
}

void DataStore::setLastSyncTime(const QDateTime &lastSyncTime)
{
    m_lastSyncTime = lastSyncTime;
}

void DataStore::setLastSyncSuccessful(bool lastSyncSuccessfull)
{
    m_lastSyncSuccessfull = lastSyncSuccessfull;
}

QDebug &operator<<(QDebug &dbg, const DataStore &dataStore)
{
    dbg.nospace() << "DataStore("
                  << dataStore.contentType() << ","
                  << dataStore.source() << ","
                  << dataStore.target()
                  << ")";
    return dbg.space();
}
