/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "serveragent.h"
#include "serveradaptor.h"              // D-Bus adaptor
#include "settings.h"                   // KConfigXT stuff
#include "settingsadaptor.h"            // D-Bus adaptor
#include "settingsdialog.h"

#include <Akonadi/ChangeRecorder>
#include <Akonadi/Collection>
#include <Akonadi/CollectionFetchJob>
#include <Akonadi/Item>
#include <Akonadi/ItemFetchJob>
#include <Akonadi/ItemFetchScope>

#include <KDateTime>
#include <KLocalizedString>
#include <KWindowSystem>

#include <QtDBus/QDBusConnection>

#include <libsyncml/syncml.h>
#include <libsyncml/data_sync_api/defines.h>
#include <libsyncml/data_sync_api/standard.h>
#include <libsyncml/data_sync_api/callbacks.h>

using namespace Akonadi;

// Used to access the agent from inside the C callbacks
static ServerAgent *agentInstance = NULL;
// libsyncml always returns device informations with every callback
// this variable helps to print that stuff only once
static QString lastDevInfName;

static void smlRecvEventCallback(
        SmlDataSyncObject *dsObject, SmlDataSyncEventType type,
        void *userdata, SmlError *error)
{
    Q_UNUSED(userdata);
    SmlError *localError = NULL;

    switch (type) {
        case SML_DATA_SYNC_EVENT_ERROR:
            kDebug() << "Error:" << smlErrorPrint(&error);
            if (!smlDataSyncAbort(dsObject, &localError)) {
                const QString message = i18nc("@info:status",
                                              "Unable to abort data sync (libsyncml error: %1)",
                                              smlErrorPrint(&localError));
                kWarning() << message;
                smlErrorDeref(&localError);
            }
            /*Settings::self()->setLastSync(KDateTime::currentUtcDateTime().toString());
            Settings::self()->setLastSyncSuccessful(false);*/
            lastDevInfName.clear();
            break;
        case SML_DATA_SYNC_EVENT_CONNECT:
            kDebug() << "Remote device was successfully connected.";
            break;
        case SML_DATA_SYNC_EVENT_DISCONNECT:
            kDebug() << "Remote device was successfully disconnected.";
            break;
        case SML_DATA_SYNC_EVENT_FINISHED:
            kDebug() << "SyncML session finished successfully.";
            /*Settings::self()->setLastSync(KDateTime::currentUtcDateTime().toString());
            Settings::self()->setLastSyncSuccessful(true);*/
            lastDevInfName.clear();
            break;
        case SML_DATA_SYNC_EVENT_GOT_ALL_ALERTS:
            kDebug() << "All alerts of the remote device were received.";
            break;
        case SML_DATA_SYNC_EVENT_GOT_ALL_CHANGES:
            kDebug() << "All changes of the remote device were received.";
            if (!agentInstance->sendChanges(dsObject, &localError)) {
                kError() << "Unable to send changes to remote device.";
                if (!smlDataSyncAbort(dsObject, &localError)) {
                    const QString message = i18nc("@info:status",
                                                  "Unable to abort data sync (libsyncml error: %1)",
                                                  smlErrorPrint(&localError));
                    kWarning() << message;
                    smlErrorDeref(&localError);
                }
            }
            break;
        case SML_DATA_SYNC_EVENT_GOT_ALL_MAPPINGS:
            kDebug() << "All mappings of the remote device were received.";
            break;
        default:
            kDebug() << "Unknown event:" << type;
            break;
    }
}

static SmlAlertType smlRecvAlertTypeCallback(
        SmlDataSyncObject *dsObject,
        const char *source, SmlAlertType type,
        void *userdata, SmlError **error)
{
    Q_UNUSED(dsObject);
    Q_UNUSED(userdata);

    DataStore *dataStore = agentInstance->dataStore(source);
    if (!dataStore) {
        const QString message = i18nc("@info:status",
                                      "Unable to find datastore for source %1",
                                      source);
        kWarning() << message;
        smlErrorSet(error, SML_ERROR_GENERIC, message.toLatin1());
        return SML_ALERT_UNKNOWN;
    } else {
        switch (type) {
            case SML_ALERT_UNKNOWN:
                break;
            case SML_ALERT_DISPLAY:
                break;
            case SML_ALERT_SLOW_SYNC:
                break;
            case SML_ALERT_TWO_WAY:
                break;
            case SML_ALERT_ONE_WAY_FROM_CLIENT:
                break;
            case SML_ALERT_REFRESH_FROM_CLIENT:
                break;
            case SML_ALERT_ONE_WAY_FROM_SERVER:
                break;
            case SML_ALERT_REFRESH_FROM_SERVER:
                break;
            case SML_ALERT_TWO_WAY_BY_SERVER:
                break;
            case SML_ALERT_ONE_WAY_FROM_CLIENT_BY_SERVER:
                break;
            case SML_ALERT_REFRESH_FROM_CLIENT_BY_SERVER:
                break;
            case SML_ALERT_ONE_WAY_FROM_SERVER_BY_SERVER:
                break;
            case SML_ALERT_REFRESH_FROM_SERVER_BY_SERVER:
                break;
            case SML_ALERT_RESULT:
                break;
            case SML_ALERT_NEXT_MESSAGE:
                break;
            case SML_ALERT_NO_END_OF_DATA:
                break;
        }
        if (type == SML_ALERT_SLOW_SYNC) {
            kDebug() << "Received slow sync alert for datastore" << *dataStore;
            dataStore->setUseSlowSync(true);
        }
        if (dataStore->useSlowSync()) {
            type = SML_ALERT_SLOW_SYNC;
        }
        return type;
    }
}

static SmlBool smlRecvChangeCallback(
        SmlDataSyncObject *dsObject,
        const char *source, SmlChangeType type,
        const char *uid,
        char *data, unsigned int size,
        void *userdata, SmlError **error)
{
    Q_UNUSED(userdata);

    kDebug() << "Change received ...";

    // find the appropriate datasoure
    DataStore *dataStore = agentInstance->dataStore(source);
    if (!dataStore) {
        kWarning() << "Unable to find datastore that corresponds to" << source;
        return false;
    } else {
        kDebug() << "Write change for item uid" << uid << "of size" << size
                 << "with data" << data << "and userdata" << userdata
                 << "to source" << source;

        //TODO: Thread uid as remote id
        //TODO: See if we have slow sync
        //TODO: write the change

        switch (type) {
            case SML_CHANGE_UNKNOWN:
                kDebug() << "Unkown change type";
                break;
            case SML_CHANGE_ADD:
                kDebug() << "Add change";
                break;
            case SML_CHANGE_REPLACE:
                kDebug() << "Replace change";
                break;
            case SML_CHANGE_DELETE:
                kDebug() << "Delete change";
                break;
        }
        return true;
    }

    /*
    char *absolute_uid = NULL;
    char *absolute_status = NULL;

    // handle the item
    if (datastore) {
        if (datastore->directory) {
            printf("Writing item %s to directory %s.\n", uid, datastore->directory);
            // prepare absolute filenames
            char *safeUID = getSafeFilename(uid);
            absolute_uid = g_strdup_printf("%s/%s", datastore->directory, safeUID);
            absolute_status = g_strdup_printf("%s/%s", datastore->directory, STATUS_FILENAME);
            // sanity check for uid
            if (!strcmp(safeUID, STATUS_FILENAME))
            {
                smlErrorSet(error, SML_ERROR_GENERIC,
                    "The filename %s cannot be used as UID. This is a potential attack.",
                    STATUS_FILENAME);
                printf("\tIllegal filename %s detected.\n", STATUS_FILENAME);
                g_warning("Potential attack against status file %s detected.",
                    STATUS_FILENAME);
                goto error;
            }
            smlSafeCFree(&safeUID);

            // cache that the remote device knows the uid
            if (datastore->remoteHash) {
                g_hash_table_insert(datastore->remoteHash, g_strdup(absolute_uid), (char *)"1");
            }
            // the directory is checked by scanArguments
            SmlBool doWrite = FALSE;
            if (!doWrite && !g_file_test(absolute_uid, G_FILE_TEST_EXISTS))
                doWrite = TRUE;
            if (!doWrite && sessionType == SML_SESSION_TYPE_CLIENT)
                doWrite = TRUE;
            if (!doWrite && !localWinsConflicts)
                doWrite = TRUE;
            if (!doWrite &&
                g_file_test(absolute_status, G_FILE_TEST_EXISTS))
            {
                if (datastore->slow)
                {
                    // We have to check the content.
                    // If the content is identical
                    // then we simply touch the file
                    // to commit it. The file will
                    // not be uploaded because of the
                    // new timestamp.
                    char *remoteData = NULL;
                    gsize remoteSize = 0;
                    if (g_file_get_contents(absolute_uid, &remoteData, &remoteSize, NULL) &&
                        remoteSize == size &&
                        !strncmp(data, remoteData, size))
                        g_file_set_contents(absolute_uid, data, size, NULL);
                } else { //!datastore->slow
                    // let's check if there is a local change
                    struct stat fdata;
                    struct stat status;
                    if (g_stat(absolute_uid, &fdata) != 0) {
                        smlErrorSet(error, SML_ERROR_GENERIC,
                            "Cannot stat uid file %s (%d).",
                            absolute_uid, errno);
                        goto error;
                    }
                    if (g_stat(absolute_status, &status) != 0) {
                        smlErrorSet(error, SML_ERROR_GENERIC,
                            "Cannot stat status file %s (%d).",
                            absolute_status, errno);
                        goto error;
                    }
                    if (status.st_mtime > fdata.st_mtime)
                        doWrite = TRUE;
                }
            }

            printf("\tConflicts were checked.\n");
            // write the change
            if (doWrite) {
                if (type == SML_CHANGE_DELETE) {
                    g_unlink(absolute_uid);
                    printf("\tThe item was successfully deleted.\n");
                } else {
                    GError *gerror = NULL;
                    if (!g_file_set_contents(absolute_uid, data, size, &gerror)) {
                        smlErrorSet(error, SML_ERROR_GENERIC,
                            "Cannot write data to %s. %s",
                            absolute_uid, gerror->message);
                        g_error_free(gerror);
                        goto error;
                    }
                    printf("\tThe item was successfully written.\n");
                }
            } else {
                printf("\tThe item was not touched.\n");
            }
            // free absolute filenames
            smlSafeCFree(&absolute_uid);
            smlSafeCFree(&absolute_status);
        } else {
            // print received change
            printf("-----BEGIN CHANGE-----\n");
            if (type == SML_CHANGE_DELETE)
                printf("DELETE %s\n", uid);
            else
                printf("%s", data);
            printf("-----END CHANGE-----\n");
        }*/
}

/**
 * This function is called by libsyncml when it receives device informations.
 */
static SmlBool smLRecvDevInfCallback(
        SmlDataSyncObject *dsObject, SmlDevInf *devinf,
        void *userdata, SmlError **error)
{
    Q_UNUSED(dsObject);
    Q_UNUSED(userdata);
    Q_UNUSED(error);

    // Make sure that device informations that belong to the same remote
    // device are shown only once
    QString currentDevInfName = smlDevInfGetModel(devinf);
    if (currentDevInfName == lastDevInfName)
        return true;
    lastDevInfName = currentDevInfName;

    kDebug() << "Client device:" << currentDevInfName
             << "Type:" << smlDevInfGetDeviceType(devinf);
    kDebug() << "Manufacturer:" << smlDevInfGetManufacturer(devinf)
             << "OEM:" << smlDevInfGetOEM(devinf);
    kDebug() << "Firmware version:" << smlDevInfGetFirmwareVersion(devinf);
    kDebug() << "Software version:" << smlDevInfGetSoftwareVersion(devinf);
    kDebug() << "Hardware version:" << smlDevInfGetHardwareVersion(devinf);
    kDebug() << "Datastores:";
    for (unsigned int i = 0; i < smlDevInfNumDataStores(devinf); i++) {
        const SmlDevInfDataStore *datastore = smlDevInfGetNthDataStore(devinf, i);
        kDebug() << "    Locations:" << smlDevInfDataStoreGetSourceRef(datastore);
    }
    //kDebug() << "Supports UTC:" << (smlDevInfSupportsUTC(devinf) ? "Yes" : "No");
    //kDebug() << "Supports number of changes:" << (smlDevInfSupportsNumberOfChanges(devinf) ? "Yes" : "No");
    //kDebug() << "Supports large objects:" << (smlDevInfSupportsLargeObjs(devinf) ? "Yes" : "No");
    return true;
}

static SmlBool smlRecvChangeStatusCallback(SmlDataSyncObject *dsObject,
            unsigned int code,
            const char *newuid, void *userdata, SmlError **error)
{
    kDebug() << "code:" << code << "newuid:" << newuid;
    return true;
}

/*SmlBool recvMappingCallback(SmlDataSyncObject *dsObject,
            const char *source, const char *orig,
            const char *newuid, void *userdata, SmlError **error)
{
    kDebug() << "source:" << source << "orig:" << orig << "newuid" ;
    return true;
}*/

ServerAgent::ServerAgent(const QString &id)
    : AgentBase(id), m_smlDataSyncObject(NULL), m_smlError(NULL)
    , m_dataStores(new QList<DataStore *>)
    , m_isServerRunning(false)
{
    agentInstance = this;

    // Setup D-BUS related stuff
    new ServerAdaptor(this);
    new SettingsAdaptor(Settings::self());
    QDBusConnection::sessionBus().registerObject(QLatin1String("/"), this);
    QDBusConnection::sessionBus().registerObject(QLatin1String("/Settings"),
                                                 Settings::self());

    //TODO: Investigate AgentManager, AgentManagerInstance and friends to
    //      also have a visual change in the AgentInstanceWidget
    setAgentName(Settings::self()->name());
    changeRecorder()->itemFetchScope().fetchFullPayload();

    connect(changeRecorder(), SIGNAL(changesAdded()), changeRecorder(), SLOT(replayNext()));

    // Start the server if not in first run and make sure it's restartServered when settings change
    if (!Settings::self()->firstRun()) {
        startServer();
    }
}

ServerAgent::~ServerAgent()
{
    // Unregister D-Bus objects
    QDBusConnection::sessionBus().unregisterObject("/", QDBusConnection::UnregisterTree);
    QDBusConnection::sessionBus().unregisterObject("/Settings", QDBusConnection::UnregisterTree);

    // Tear down the SyncML server
    stopServer();

    // Make sure that all custom settings (see DataStore) are also saved
    Settings::self()->writeConfig();

    foreach (DataStore *ds, *m_dataStores) {
        delete ds;
    }
    delete m_dataStores;
}

DataStore *ServerAgent::dataStore(const QString &source)
{
    foreach (DataStore *dataStore, *m_dataStores) {
        if (dataStore->source() == source)
            return dataStore;
    }
    return NULL;
}

bool ServerAgent::sendChanges(SmlDataSyncObject *dsObject, SmlError **localError)
{
    QString message;

    foreach (DataStore *dataStore, *m_dataStores) {
        SmlChangeType changeType = SML_CHANGE_REPLACE;


        /*if (!smlDataSyncAddChange(dsObject, dataStore->source, changeType,
                dataStore->source,
                data, length,
                NULL, localError)) {
            goto labelErrorSendChanges;
        }*/
    }
    return smlDataSyncSendChanges(dsObject, localError);

labelErrorSendChanges:
    kError() << message;
    emit error(message);
    emit status(Broken, message);
    return false;
}

void ServerAgent::configure(WId windowId)
{
    SettingsDialog dialog;

    if (windowId) {
        KWindowSystem::setMainWindow(&dialog, windowId);
    }
    dialog.exec();
    restartServer();  // Always (re)startServer server after settings (may) have changed
}

bool ServerAgent::startServer()
{
    QString message;

    if (m_isServerRunning) {
        message = i18nc("@info:status", "Server is already running.");
        kWarning() << message;
        emit warning(message);
        return false;
    }
    // This means the agent instance's first run, not the SyncML server
    if (Settings::self()->firstRun()) {
        Settings::self()->setFirstRun(false);
    }

    SmlTransportType smlTransport = SML_TRANSPORT_HTTP_SERVER;
    SmlSessionType smlSessionType = SML_SESSION_TYPE_SERVER;
    switch (Settings::self()->transport()) {
        case 0: smlTransport = SML_TRANSPORT_HTTP_SERVER;
                smlSessionType = SML_SESSION_TYPE_SERVER;
                kDebug() << "Starting HTTP server"; break;
        case 1: smlTransport = SML_TRANSPORT_OBEX_SERVER;
                //NOTE: This is not a typo, simply libsyncml weirdness
                smlSessionType = SML_SESSION_TYPE_SERVER;
                kDebug() << "Starting OBEX/TCP server"; break;
        default:
            message = i18nc("@info:status",
                            "Invalid transport type in settings, defaulting to HTTP server");
            kWarning() << message;
            emit warning(message);
    }

    m_smlDataSyncObject = smlDataSyncNew(smlSessionType, smlTransport, &m_smlError);
    if (!m_smlDataSyncObject) {
        message = i18nc("@info:status",
                        "Unable to create data sync object (libsyncml error: %1)",
                        smlErrorPrint(&m_smlError));
        goto labelErrorStartServer;
    } else {
        if (!initDataStores()) { // Register SyncML datastores
            message = i18nc("@info:status", "Unable to initialize datastores");
            goto labelErrorStartServer;
        }
        if (!initSettings()) { // Setup all kinds of libsyncml configuration values
            message = i18nc("@info:status", "Unable to setup libsyncml");
            goto labelErrorStartServer;
        }

        // Register various types of callbacks for libsyncml
        smlDataSyncRegisterEventCallback(m_smlDataSyncObject, smlRecvEventCallback, NULL);
        smlDataSyncRegisterGetAlertTypeCallback(m_smlDataSyncObject, smlRecvAlertTypeCallback, NULL);
        smlDataSyncRegisterChangeCallback(m_smlDataSyncObject, smlRecvChangeCallback, NULL);
        smlDataSyncRegisterHandleRemoteDevInfCallback(m_smlDataSyncObject, smLRecvDevInfCallback, NULL);
        smlDataSyncRegisterChangeStatusCallback(m_smlDataSyncObject, smlRecvChangeStatusCallback);

        // Finally initialize the data sync object
        if (!smlDataSyncInit(m_smlDataSyncObject, &m_smlError)) {
            message = i18nc("@info:status",
                            "Unable to initialize data sync (libsyncml error: %1)",
                            smlErrorPrint(&m_smlError));
            goto labelErrorStartServer;
        } else {
            // Start SyncML server by running data sync and thus idle for clients
            if (!smlDataSyncRun(m_smlDataSyncObject, &m_smlError)) {
                message = i18nc("@info:status",
                                "Unable to run data sync (libsyncml error: %1)",
                                smlErrorPrint(&m_smlError));
                goto labelErrorStartServer;
            }
        }
    }

    m_isServerRunning = true;
    kDebug() << "Server running and waiting for clients ...";
    emit status(Idle);
    return true;

labelErrorStartServer:
    stopServer();
    kError() << message;
    emit error(message);
    emit status(Broken, message);
    return false;
}

bool ServerAgent::restartServer()
{
    if (m_isServerRunning) {
        stopServer();
    }
    return startServer();
}

void ServerAgent::stopServer()
{
    QString message;

    kDebug() << "Shutting down SyncML server ...";

    if (!smlDataSyncAbort(m_smlDataSyncObject, &m_smlError)) {
        message = i18nc("@info:status",
                        "Unable to abort data sync (libsyncml error: %1)",
                        smlErrorPrint(&m_smlError));
        kWarning() << message;
        emit warning(message);
    }

    if (m_smlDataSyncObject) {
        smlDataSyncObjectUnref(&m_smlDataSyncObject);
    }
    if (m_smlError) {
        smlErrorDeref(&m_smlError);
    }
    m_dataStores->clear();

    m_isServerRunning = false;
    message = i18nc("@info:status", "SyncML server stopped.");
    emit status(Broken, message);
}

void ServerAgent::itemAdded(const Akonadi::Item &item, const Akonadi::Collection &collection)
{
    kDebug() << "TODO: React on added item" << item.id() << "to collection" << collection.id();
    AgentBase::Observer::itemAdded(item, collection);
}

void ServerAgent::itemChanged(const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers)
{
    Q_UNUSED(partIdentifiers);

    kDebug() << "TODO: React on changed item" << item.id();
    AgentBase::Observer::itemChanged(item, partIdentifiers);
}

void ServerAgent::itemRemoved(const Akonadi::Item &item)
{
    kDebug() << "TODO: React on deleted item" << item.id();
    AgentBase::Observer::itemRemoved(item);
}

bool ServerAgent::initSettings()
{
    QString message;
    const char *smlSyncMLVersion = "1.2";   // Put here to avoid warnings about goto statement

    // Setup the server port
    if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_TRANSPORT_CONFIG_PORT,
                              QByteArray::number(Settings::self()->port()), &m_smlError)) {
        message = i18nc("@info:status",
                        "Unable to set server port (libsyncml error: %1)",
                        smlErrorPrint(&m_smlError));
        goto labelErrorInitSettings;
    } else {
        kDebug() << "Listening on port" << Settings::self()->port();
    }

    // Setup authentication related stuff
    if (Settings::self()->useAuthentication()) {
        if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_AUTH_USERNAME,
                                  Settings::self()->username().toLatin1(), &m_smlError)) {
            message = i18nc("@info:status",
                            "Unable to set authentication username (libsyncml error: %1)",
                            smlErrorPrint(&m_smlError));
            goto labelErrorInitSettings;
        }
        if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_AUTH_PASSWORD,
                                  Settings::self()->password().toLatin1(), &m_smlError)) {
            message = i18nc("@info:status",
                            "Unable to set authentication password (libsyncml error: %1)",
                            smlErrorPrint(&m_smlError));
            goto labelErrorInitSettings;
        }
    }

    // Do some transport specific setup
    switch (Settings::self()->transport()) {
        case 0: // Means HTTP
            if (!Settings::self()->sslKeyFile().isEmpty() &&
                !Settings::self()->sslCertificateFile().isEmpty()) {
                if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_TRANSPORT_CONFIG_SSL_KEY,
                                          Settings::self()->sslKeyFile().toLatin1(), &m_smlError)) {
                    message = i18nc("@info:status",
                                    "Unable to set SSL key file (libsyncml error: %1)",
                                    smlErrorPrint(&m_smlError));
                    goto labelErrorInitSettings;
                }
                if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_TRANSPORT_CONFIG_SSL_SERVER_CERT,
                                          Settings::self()->sslCertificateFile().toLatin1(), &m_smlError)) {
                    message = i18nc("@info:status",
                                    "Unable to set SSL certificate file (libsyncml error: %1)",
                                    smlErrorPrint(&m_smlError));
                    goto labelErrorInitSettings;
                }
            }
            break;
        case 1: // Means OBEX/TCP
            if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_CONNECTION_TYPE,
                                      SML_DATA_SYNC_CONFIG_CONNECTION_NET, &m_smlError)) {
                message = i18nc("@info:status", "Unable to set OBEX specifica");
                goto labelErrorInitSettings;
            }
            break;
    }

    // Setup the desired SyncML version the server shall provide
    switch (Settings::self()->syncMLVersion()) {
        case 0: smlSyncMLVersion = "1.2"; break;
        case 1: smlSyncMLVersion = "1.1"; break;
        case 2: smlSyncMLVersion = "1.0"; break;
        default:
            message = i18nc("@info:status",
                            "Invalid SyncML version in settings, defaulting to SyncML 1.2");
            kWarning() << message;
            emit warning(message);
    }
    if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_VERSION,
                              smlSyncMLVersion, &m_smlError)) {
        message = i18nc("@info:status",
                        "Unable to set SyncML version (libsyncml error: %1)",
                        smlErrorPrint(&m_smlError));
        kWarning() << message;
        emit warning(message);
    } else {
        kDebug() << "Using SyncML version" << smlSyncMLVersion;
    }

    if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_IDENTIFIER,
                              Settings::self()->identifier().toLatin1(), &m_smlError)) {
        message = i18nc("@info:status",
                        "Unable to set SyncML identifier (libsyncml error: %1)",
                        smlErrorPrint(&m_smlError));
        kWarning() << message;
        emit warning(message);
    } else {
        kDebug() << "Using SyncML identifier" << Settings::self()->identifier();
    }

    if (Settings::self()->useLocalTime()) {
        if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_USE_LOCALTIME,
                                  "1", &m_smlError)) {
            message = i18nc("@info:status",
                            "Unable to set local time usage (libsyncml error: %1)",
                            smlErrorPrint(&m_smlError));
            kWarning() << message;
            emit warning(message);
        }
    }

    if (Settings::self()->useWBXML()) {
        if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_USE_WBXML,
                                  "1", &m_smlError)) {
            message = i18nc("@info:status",
                            "Unable to set WBXML usage (libsyncml error: %1)",
                            smlErrorPrint(&m_smlError));
            kWarning() << message;
            emit warning(message);
        }
        if (Settings::self()->useStringTables()) {
            if (!smlDataSyncSetOption(m_smlDataSyncObject, SML_DATA_SYNC_CONFIG_USE_STRING_TABLE,
                                      "1", &m_smlError)) {
                message = i18nc("@info:status",
                                "Unable to set WBXML string table usage (libsyncml error: %1)",
                                smlErrorPrint(&m_smlError));
                kWarning() << message;
                emit warning(message);
            }
        }
    }

    return true;

labelErrorInitSettings:
    kError() << message;
    emit error(message);
    emit status(Broken, message);
    return false;
}

bool ServerAgent::initDataStores()
{
    QString message;

    // Fetching all collections recursive, startServering at the root collection
    CollectionFetchJob *job = new CollectionFetchJob(Collection::root(), CollectionFetchJob::Recursive);
    Collection::List collections;
    if (job->exec()) {
        collections = job->collections();
    }

    // Unset all previously monitored resources
    foreach (const QString &monitoredResource, changeRecorder()->resourcesMonitored()) {
        changeRecorder()->setResourceMonitored(monitoredResource.toLatin1(), false);
    }

    foreach (const QString &resourceName, Settings::self()->monitoredResourceNames()) {
        DataStore *dataStore = new DataStore(resourceName, this);

        if (!dataStore->isValid()) {
            message = i18nc("@info:status",
                            "Unable to create datastore for resource %1", resourceName);
            kWarning() << message;
            emit warning(message);
            delete dataStore;
            continue;
        } else {
            // Monitor this resource
            changeRecorder()->setResourceMonitored(resourceName.toLatin1());

            // Register datastore to libsyncml
            if (!smlDataSyncAddDatastore(m_smlDataSyncObject,
                                         dataStore->contentType().toLatin1(),
                                         NULL,
                                         dataStore->source().toLatin1(),
                                         &m_smlError)) {
                message = i18nc("@info:status",
                                "Unable to add datastore %2 (libsyncml error: %1)",
                                smlErrorPrint(&m_smlError), dataStore->source());
                kError() << message;
                emit error(message);
                emit status(Broken, message);
                delete dataStore;
                return false;
            } else {
                m_dataStores->append(dataStore);
                kDebug() << "Added datastore" << *dataStore;
            }
        }
    }

    kDebug() << "Monitored resources:" << changeRecorder()->resourcesMonitored();
    return true;
}

AKONADI_AGENT_MAIN(ServerAgent)

#include "serveragent.moc"
